#!/opt/local/bin/perl
#
#shift_matrix.sh
#
# This script builds the final gain corrected response matrix for use with  
# XSPEC. The gain history of an observation is read in from the file 
# 'xgain.fits'. A matrix shift is calculated for each gain value. The FTOOLS 
# task 'addrmf' creates the final matrix from the corresponding shifted 
# matrices and the fraction of the observation time in each gain state .
#
# The script produces two new response matrices. 
#
# 'new_resp.fits'         The weighted response matrix
#
# 'new_mean_resp.fits'    An unweighted response matrix with a single channel 
#			  shift for which the observation was in that gain 
#                         state for the largest percent of the exposure time.

                                        
system("fdump xgain.fits xgain.asc - - clobber=yes");

open(INFILE,"xgain.asc");
open(OUTFILE,">xgain.rmf");
$mean_gain=0;
$start="no";
$resp_matrix="hri_dtmat_15";

print "\nShifted Response Matrix  Weight Factor\n";
print "--------------------------------------\n";
while(<INFILE>) {
    chop($_);
    @gain=split(/\s+/,$_);
    if ($gain[1] eq "GAIN"){
	$start="yes";
    }
    if ($start eq "yes"){
	if ($gain[1]>= 1){
	    if($gain[2]>= 4.5){
	       $shift=-5.258+1.171*$gain[2];
	    }
	    elsif ($gain[3]!= 0){
	        print("The gain level fell below 4.5\n");
		print("The average channel shift of -0.33 is used.\n"); 
	       $shift=-0.33
            }
	    $shift_frmt=sprintf("%1.1f",$shift);
	    $mean_gain+=$gain[2]*$gain[3];
	    $sign=substr($shift_frmt,0,1);
	    if ($sign eq "-"){
		$a=substr($shift_frmt,1,1);
		$b=substr($shift_frmt,3,1);
		$matrix=$resp_matrix."_csm".$a.$b.".fits";
		$weight=sprintf("%1.4f",$gain[3]);
		if ($a == 0 && $b == 0) {
		    $matrix=$resp_matrix.".fits";
		}
		if (-e $matrix && $weight !=0){
		    print OUTFILE "$matrix $weight\n";
		    print "$matrix $weight\n\n";
		}
		elsif (! -e $matrix && $weight !=0) {
		    print "The file $matrix does not exist.\n";
		}    
	    }
	    else {
		$a=substr($shift_frmt,0,1);
		$b=substr($shift_frmt,2,1);
		$matrix=$resp_matrix."_cs".$a.$b.".fits";
		$weight=sprintf("%1.4f",$gain[3]);
		if ($a == 0 && $b == 0) {
		    $matrix=$resp_matrix.".fits";
		}
		if (-e $matrix && $weight !=0){
		    print OUTFILE "$matrix $weight\n";
		    print "$matrix $weight\n";
		}
		elsif (! -e $matrix && $weight !=0)  {
		    print "The file $matrix does not exist.\n";
		}    


	    }
		

	}
    }	
}
#Calculate the channel shift based on the mean gain state with a weight
#factor of 1. Then the corresponding matrix is copied to a new file.

if($mean_gain>= 4.5){
    $channel_shift=-5.258+1.171*$mean_gain;
}
else {
    $channel_shift=-0.33;
    print("The mean gain level fell below 4.5\n");
    print("A mean channel shift of -0.33 is used.\n"); 
}

$ch_shift_new=sprintf("%1.1f",$channel_shift);
            $sign=substr($ch_shift_new,0,1);
	    if ($sign eq "-"){
		$a=substr($ch_shift_new,1,1);
		$b=substr($ch_shift_new,3,1);
		$matrix=$resp_matrix."_csm".$a.$b.".fits";
		if ($a == 0 && $b == 0) {
		    $matrix=$resp_matrix.".fits";
		}
	    }
	    else {
		$a=substr($ch_shift_new,0,1);
		$b=substr($ch_shift_new,2,1);
		$matrix=$resp_matrix."_cs".$a.$b.".fits";
		if ($a == 0 && $b == 0) {
		    $matrix=$resp_matrix.".fits";
		}
	    }        

print "\nThe mean PHA channel shift is: $ch_shift_new\n";
print "-------------------------\n";
close(INFILE);
close(OUTFILE);
print "BUILDING THE FINAL MATRIX\n";
print "-------------------------\n\n";
system("addrmf xgain.rmf new_resp.fits");
print "\n\nThe new matrix is called 'new_resp.fits'\n\n";
print "Creating a response matrix for the mean gain only.\n";
print "'new_mean_resp.fits'\n"; 
system("cp $matrix new_mean_resp.fits");








