.\" @(#)plist.l 0.1 91/09/01 AXAF Science Center; IRAF host interface
.TH PARAM L "23 November 1991"
.SH SYNOPSIS
DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT
.PP
DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT
.PP
This document describes an IRAF-compatible parameter interface for host
(native OS) programs.  The interface provides a wide range of
parameter handling capabilities, including positional or keyword
assignments, prompting for missing information, saving parameter
values and defaulting parameter values, range checking, and type
checking.  The compatibility between this interface and the IRAF
parameter interface makes it easier to mix IRAF and non-IRAF programs
(especially within a layered GUI).
.PP
DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT
.PP
DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT DRAFT
.SH INTRODUCTION
The essential function of a parameter interface is to pass information
to programs, usually on the command line.  The simplest means of doing
this is to pass required information on the command line in an
order-dependent manner. For example,
.nf

	doit <input> <output> <control>

.fi
requires that the first argument be an input file, the second argument
an output file, and the third argument control information.
.PP
A somewhat more sophisticated technique is the Unix "switch" mechanism,
in which identifiers are associated with each argument.  This removes
the order-dependence and allows limited support for default values.
For example:
.nf

	doit -i <input> -c <control>

.fi
allows the output to default to some known value.  Both of these
interfaces work well for simple programs, but fail in the case
of astronomical analysis programs, where parameter organization and
manipulation can become quite complex.
.PP	
Ideally, a good parameter interface should support a several capabilities,
including:
.IP o
the ability to input parameters with or without associated keyword switches
.IP o
the ability to prompt the user for important parameters not placed on the
command line
.IP o
support for default values, including the ability to change default values
.IP o
range checking of values against user-definable limits
.IP o
type checking of values
.IP o
the ability to view and edit a program's parameters without running the
program (i.e., from the shell).
.PP
The IRAF parameter interface supports all of the functionality
mentioned above. The flexibility of the IRAF interface, combined with
its general user familiarity in the astronomical community, has led to
its being viewed as a good candidate for a general parameter interface
for host (non-IRAF) programs.  An IRAF-compatible host interface 
not only provides host programs with much-needed functionality, but
also will provide a uniform look and feel for IRAF and non-IRAF
programs.  This will make it easier to mix IRAF and non-IRAF programs.
.PP
The rest of this document describes the features of an IRAF-compatible
host parameter interface.  Much of the description of the interface is
taken directly from the \fICL Programmer's Manual\fP (E. Downey, D.
Tody, , G.H. Jacoby, September, 1983), for which grateful
acknowledgment is made.
.SH PARAMETER FILES
The host parameter interface is based on the concept of a \fBparameter
file\fP. A parameter file contains a description of each of the
parameters that it manages, including the parameter name, type,
default value, minimum, maximum, and prompt string.  Programs "open"
parameter files and access the parameters contained in the files.
Parameter values can be retrieved and changed by programs.  The
parameter files themselves contain the permanent record of the values
of parameters.  When a parameter value is permanently changed, either
by shell-level assignment, or by running a program that changes that
parameter value, the parameter file is updated with the new value.
The parameter files also can be inspected and edited from the shell, using
special parameter manipulation programs.
.PP
Parameter files are accessed by name, so that a given program can
access more than one parameter file, and a given parameter file can be
accessed by more than one program.  A mechanism exists by which a
default parameter file can be associated with a program, in the style
of IRAF.
.PP
Parameter files are searched for in three places. If an explicit
directory path is given in the parameter file name, that directory is
searched.  Otherwise, a search is performed in the current directory.
Finally, the directory pointed to by the \fBUPARM\fP environment
variable is searched.
.SH PARAMETER FILE FORMAT
The parameter file consists of comments, blank lines, and parameter
declarations. These may appear in any desired order.  Comments begin
with the sharp character, #.  All remaining characters on that line
are ignored. The maximum line length is 132 characters.
.PP
Parameter declarations within the parameter file take the form:
.nf

    name, type, mode, value, minimum, maximum, prompt

.fi
where all fields following the mode field are optional. Both the comma
and the end of the line itself serve as a field delimiter, Thus a
comma is not necessary after the last field.
.SH name
This is the name of the parameter. There is no length limit other than
the overall line length limit consideration. This is the name by which
the parameter will be known to a program. It must begin with a letter
or a dollar sign, $, but the remaining characters may be any
combination of letters, numbers, underscore, \(ul, and dollar, $.
.SH type
The type field indicates how the parameter is to be stored. It also implies
some information about what values are acceptable and how they are entered,
as discussed below under value.
.nf

	code		meaning

	b		boolean
	i		integer
	r		real
	s		string
	f or f\fIxx\fR	file name
	struct		structure	

.fi
.PP
The codes \fBb\fP, \fBi\fP and \fBr\fP indicate the usual boolean,
integer and real types. They are discussed further in the value
section, below.
.PP
There are several types that manipulate character strings.  The
characters themselves may be anything from the ASCII set.  The type
\fBs\fP is the simplest and is an ordinary character string. It is
typically used for names, flags and messages.
.PP
The \fBf\fP type is like \fBs\fP except that it is limited to legal
file names on the host operating system. The \fBf\fP may optionally be
followed by any reasonable combination of the characters \fBe\fP,
\fBn\fP, \fBr\fP, or \fBw\fP.  These indicate that checks should be
made on the file name before it is used that to ensure that it exists,
does not exist, that it exists and is readable and that it exists and
is writable, respectively. \fBStruct\fP is also like \fBs\fP but its value
is the entire next line of the parameter file.
.PP
If the type is preceded by a star, *, the parameter is
\fBlist-structured\fP.  When the parameter is referenced, the value
will come from a file, the name of which is the fourth (value) field
of the parameter declaration.  All of the basic types may be
list-structured.
.PP
The type field of a parameter can be retrieved as a string by
appending the string ".p\(ultype" to the parameter name, either in a
parameter access subroutine call, or when using the \fBpget\fP or
\fBpquery\fP programs. The type cannot be changed by programs,
however.
.SH mode
This field indicates what actions are performed when the parameter is
referenced or assigned.  \fBQuery\fP mode generally causes the user to
be queried each time the parameter is referenced.  \fBLearn\fP means
that all changes to the parameter will be permanent.  \fBAuto\fP mode
means that the effective mode of the parameter should follow the mode
of the parameter file containing the parameter; auto mode defers mode
selection to the program's mode parameter.  \fBHidden\fP mode means
that the existence of the parameter will not be evident to the user
(i.e., prompts will not be issued when the parameter is accessed)
unless its value is not acceptable.
.PP
The mode field may be any reasonable combination of query, learn,
auto and hidden. These may be spelled out and separated with plus signs, +,
or abbreviated to one character and run together. For example,
.nf

	...,auto+learn,...
and
	...,al,...

.fi
and equivalent.
.PP
The mode of a parameter can be accessed as a string by appending the
string ".p\(ulmode" to the parameter name, either in a parameter
access subroutine call, or when using the \fBpget\fP or \fBpquery\fP
programs. The mode can be changed.  For example,
.nf

	prog1 region.p_mode="h"

.fi
will cause the region parameter to become hidden for the duration
of that program, so that the parameter is not prompted for.
.SH value
This field is optional. The value field is the initial or
\fBdefault\fP value of the parameter. It has various characteristics
depending on the type of the parameter.
.PP
For boolean parameters, it should be either the three characters \fByes\fP or
the two characters \fBno\fP.
.PP
Integer and real parameters are as one would expect. Real constants need not
include a decimal point, ., if not required by the value itself.
.PP
For string and file name parameters, the field extends from the comma
following the mode field to the next comma, or the end of the line if
there is no comma. It may be surrounded by single or double quotes, '
or ", but these are not necessary unless the string is to include a
comma. The maximum length of a string is 132 characters.
.PP
Structs use the value field to indicate the number of characters of
storage to allocate to hold the value of the parameter. The value is a
string consisting of the entire next line of the parameter file.
.PP
The value field for list-structured parameters is the name of the file
containing values for the parameter. This name is subject to the same
restrictions as a parameter of type \fBfr\fP.
.PP
Thus, the value field entry for a parameter in a parameter file has
several different uses, depending on the type of the parameter.  The
term \fBvalue\fP refers to that which is used when the parameter is
used in an expression and \fBvalue field\fP refers specifically to the
fourth field of the parameter specification.
.PP
.SH minimum \fRand\fP maximum
These two fields work together to specify a validity range for the
value of the parameter. They are ignored for all types except integer,
real, string, and file name parameters and follow the same rules as
the value field for these type parameters. Their application to
filenames is to test for a simple lexical ordering. If they are both
set when the parameter is referenced, then a query will be generated
if the value of the parameter is not within range. No range checking
is done if either the minimum or maximum are undefined or if min >
max. If the parameter is list-structured, then the range checking is
applied to the entry read from the file.
.PP
The min field can specify a set of allowable \fBenumerated\fP values
instead of a range.  Enumeration is used to specify a discrete set of
allowable values.  It is specified by separating allowable values with
the '|' symbol. For example, the string '1|5|8' in the min or max
field specifies that the value of the parameter can take on any of
these three integer values, but no others.
.PP
The min and max limits of a parameter can be accessed by appending the
string ".p\(ulmin" or ".p\(ulmax" to the parameter name, either in a
parameter access subroutine call, or when using the \fBpget\fP or
\fBpquery\fP programs. The min and max can be changed.  For example,
.nf

	prog1 threshold.p_min=10

.fi
will cause the  minimum value of the threshold parameter to be set to
10 for the duration of the program.
.SH prompt
This field behaves like a string and extends from just after the sixth
comma in the parameter spec to the end of the line. It may be quoted.
Its purpose is to provide a meaningful prompt for the parameter during
a query. If no prompt string is given, then the query will just use
the name of the parameter.
.PP
The prompt of a parameter can be accessed as a string by appending the
string ".p\(ulprompt" to the parameter name, either in a parameter
access subroutine call, or when using the pget or pquery programs. The
prompt can be changed.  For example,
.nf

	pset pfile region.p_prompt="enter region descriptor"

.fi
will cause the region prompt to be changed permanently in the specified
pfile.
.SH INDIRECT PARAMETERS
The value, min, and max fields of a parameter can be \fBindirect\fPed to
point to other parameters.  If the first character of the value, min, or
max field is ")", the value of that field is taken to be the value of
the corresponding field of the parameter which is specified in the
rest of the field.  Thus, a value specified as
.nf

	)indir

.fi
takes its value from the parameter called "indir" in the same
parameter file.
.PP
Indirects can be used to point to parameters in other parameter files
as well.  If the first two characters of a value, min, or max field are
")*", then the next characters up to a "." are taken to be the name of
a parameter file (without .par extension), and the remaining string is
taken to be the name of the parameter.  Thus, a value specified as
.nf

	)*other.indir

.fi
takes its value from the parameter called "indir" in the parameter
file called "other.par".  The use of indirects can be recursive, that
is, one indirect can point to another indirect and so on.
.PP
Indirect parameters can be used to implement \fBcommon data sets\fP of
parameters, in which one parameter file controls the common parameter
values of several programs. For example, two or more programs can
indirect the value of the parameter specifying their input data file
to point indirectly to a single parameter in another parameter file.
(Note that the names of the input data file parameters need not be the
same for all of these programs.)  When the common parameter is
changed, each of these programs will pick up the new data file as
input.
.SH MODES
The parameter interface supports three modes of user interaction: query,
learn and auto.
.PP
\fBQuery\fP mode causes each parameter referenced by a program
to produce a query on the terminal. The query consists of the prompt
string for that parameter, its current value and minimum and maximum
values, if set. If there is no prompt string, then the name of the
parameter is used. When the user sees this query, she may type a simple
<CR> to accept the current value or she may enter a new value.  New values
that are entered in this way are checked for validity immediately with
regard to type and range, and the query repeats until a reasonable
value is entered.
.PP
A query will be generated regardless of the effective mode of the
parameter if it does not meet its range requirements. On the other
hand, a query will be prevented if the parameter was set on the
command line, again assuming it is not out of range. Thus, the
parameter interface relieves the application program from some of the
burden of verifying its parameters.
.PP
\fBLearn\fP mode retains the values of parameters across program runs.
The default values of parameters come from their entries in the
program's parameter file. If learn mode is not in effect, changes to
parameter values by way of command line arguments to the program or
queries do not cause the parameter file to be updated and so the
values revert back to their defaults as soon as the program ends.
Learn mode makes these changes permanent by updating the parameter
file for the program.
.PP
\fBHidden\fP mode prevents queries from being generated even if the
effective mode for the parameter is query, unless its value is out of
range. Hidden mode also prevents the default value from ever being
"learned". The only way to change the default value of a hidden
parameter is by an assignment statement.  Hidden mode is useful for
parameters that are rarely changed, so that their existence is hidden
from all but experienced users.
.SH DETERMINING MODES
.PP
The modes exist independently in a two-level hierarchy: the parameter
and the current program.  Whenever a parameter is referenced, its
\fBeffective mode\fP is calculated. To determine the effective mode,
the mode settings of the two levels are used, starting with the
parameter level. If the mode of the parameter is query or learn, that
is the effective mode.  If the parameter's mode is \fBauto\fP, then
the effective mode is that of the current program.
.PP
Thus, each layer of the hierarchy, starting at the parameter level,
defers to a higher level until it finds either query or learn (or
both).  Note that the presence of hidden mode at the parameter does
not alter this process but rather serves to override query mode,
should it be found at any given level.  As a practical example, all
the auto-mode parameters in a program can effectively be put into
query mode at once by setting the mode once at the program level to
query.
.SH SETTING AND CHANGING THE MODE
The mode of a program is maintained in a parameter called \fBmode\fP,
of type string, whose value is any reasonable combination of the
letters \fBq\fP, \fBl\fP, \fBa\fP and \fBh\fP. This parameter may be
declared and initialized as desired in the parameter file for the
program, just like any other parameter. If it does not appear in the
parameter file for a program when it runs, it will be manufactured and
supplied with a default setting of 'ql'.
.PP
As a convenience for naming modes, four string parameters \fBquery\fP,
\fBlearn\fP, \fBauto\fP and \fBhidden\fP are defined to be the
single-character strings 'q', 'l', 'a' and 'h'.
Examples of setting modes at the program and parameter
levels:
.nf

	pset pfile mode \(eq 'a'	# set program's mode to auto from shell
	prog1 mode=h			# set mode to hidden for this run

.fi
.PP
.PP
As mentioned earlier, the mode for a particular parameter is accessed
as a field of that parameter called \fBp\(ulmode\fP.  A parameter's mode
may be changed using \fBpset\fP, or by specifying a parameter assignment on
a program's command line.
.PP
The mode of a parameter may also be changed during a query for that
parameter. If the response to the query begins with a percent, %, then
the mode for the parameter may be set using the same format as that
used in the parameter file mode field.  This is useful during program
development for making a parameter hidden once its default value has
been determined.
.SH RECOMMENDED MODE SETTINGS
The recommended default modes are \fBauto\fP and \fBlearn\fP for each
program, and \fBauto\fP or \fBhidden\fP for the parameters. Auto mode
for all non-hidden parameters in a program allows them all to be
changed at once by changing the mode of the program.
.SH INPUTTING PARAMETERS ON THE PROGRAM'S COMMAND LINE
When a program using the host interface is run, parameters can be
passed to the program on the command line.  These parameter arguments
to the program are specified as a space or comma demited list and come
in two basic forms, \fBpositional\fP and \fBabsolute\fP.  The
\fBpositional\fP form is simply the value of the parameter.  Each
positional value is assigned one-to-one to the corresponding
non-hidden parameters of the task, as defined by their order in the
task's parameter file. Hidden parameters are not counted in this
ordering procedure.  Two commas may be used as a placeholder to
skip a parameter without changing it. Parameters not reached in the
matching are also not changed. For example:
.nf

	prog1 "foo.qp" "c 512 512 100" ,, "foo.tab"

.fi
sets (non-hidden) parameters 1, 2, and 4.
.PP
The \fBabsolute\fP or \fBkeyword\fP form is an assignment,
\fIparameter \(eq value\fP, where the parameter can be any parameter
of the program being run.  Here, order is not relevant.  Inputting a
parameter using keywords is useful when a parameter value is to be
changed but its position in the argument list is not known, or where
it would be awkward to arrive at its position by a large number of
positional arguments. It is also the only way of setting the value of
hidden parameters on the command line. For example,
.nf

	prog1 input="foo.qp" region="c 512 512 100" table="foo.tab"

.fi
.PP
Another form of absolute argument is the \fBswitch\fP. It is a
shorthand way of specifying the truth value of a boolean parameter. A
switch consists of the parameter followed by a plus, +, to set it to
yes, or a minus, \(mi, to set it to no. Thus, these two forms are
equivalent ways of turning off the boolean parameter \fIoption\fP:
.nf

	prog1 verify\(eqno
	prog1 verify\(mi

.fi
.PP
While they may be used together, all positional arguments must precede
absolute arguments. Here are examples of using the positional and absolute
forms together:
.nf

    prog1 100, 200, op+
    prog2 "test", 123.45, 200, param1\(eq4, op3\(mi, param2\(eq"testing"

.fi
.PP
Parameters changed on the command line will have their new values as long as
the command is executing. If learn mode is not in effect for the parameters,
they will revert back to their original values when the program ends or if the
program aborts for some reason.
.SH MANIPULATING PARAMETERS FROM THE SHELL
The parameters values in a parameter file can be manipulated directly
form the shell using the programs \fBpset\fP, \fBpget\fP, \fBplist\fP,
and \fBpquery\fP.  The pset command will set the value of one or more
parameters in a specified parameter file.  The calling
sequence is:
.nf

	pset <pfile> [<passign1> <passign2> ... <passignn>]

.fi
were <pfile> is a parameter file and <passign1> ... <passignn> are one or more
command line parameter assignments.  The parameter assignments can
either be positional or absolute (keyword).  If no parameters are
input on the command line, the program will prompt the user for all
parameter values (including hidden parameters). If a command line
parameter is followed by an "=" with no value, that parameter will be
prompted for explicitly.
.PP
The \fBpget\fP program returns the value of one or more parameters
from the specified parameter file.  The calling sequence is:
.nf

	pget <pfile> <param1> <param2> ... <paramn>

.fi
where <pfile> is a parameter file and <p1> ... <pn> are parameter
names.  The value of each parameter will be returned to stdout, separated by
new-lines.  This pget program is often used in scripts to retrieve the
value of parameters that control the action of the script.
.PP
The \fBplist\fP program lists the parameter values
from the specified parameter files.  The pget calling sequence is:
.nf

	pget <pfile1> ... <pfilen>

.fi
where <pfile1> ... <pfilen> are parameter file.  The listing follows
the IRAF \fBlparam\fP format, in which non-hidden parameters are
listed first, followed by hidden parameters (which are listed in
parenthesis).  For example:
.nf

csh> plist src

Parameters for 'src' contained in '/home/eric/uparm/src.par':

         qpoe = "/pros/xray_old/data/snr.qp" QPOE file
      (region = "CIRCLE(461,570,43.93);") region descriptor
        (mode = "ql")           


.fi
Finally, \fBpquery\fP is a program that acts similarly to \fBpget\fP,
but returns the value of only one specified parameter.  It differs
from \fBpget\fP in that it allows a string of parameter assignments to
be added to the command line.  If the desired parameter is not one of
the parameters in this assignment list, that parameter is queried for.
The returned parameter value is also updated in the parameter file,
(but other parameters in the list are not updated).  The calling
sequence for pquery is:
.nf

	pquery <pfile> <param> [set params ...]

.fi
The pquery program is used in scripts to allow a user to input a list
of parameter assignments to the script and then have the script prompt
for any that are missing from the list.  The returned parameters can
then be passed on to programs that do not use the parameter interface.
In this way, scripts can be made to mimic the action of the parameter
interface on behalf of programs which do not use the interface.
.PP
The \fBpset, pget, plist\fP, and \fBpquery\fP programs are discussed
at length in separate man pages.
.SH PARAMETER INTERFACE LIBRARY
Programs access parameters using the parameter interface library.
Fortran and C interfaces are available in both subroutine form
(which always returns en error status) and function form (which
exits on error). The following C functions are contained in the library:
.LP
.nf
.ft B
#include <ParamFile.h>
.ft
.fi
.LP
.nf
.ft B
void InitParamFile(argc, argv, start)
    unsigned int  argc;
    char          **argv;
    int           start;
.ft

.ft B
\s-1Paramfile\s0 *OpenParamFile(filename, flags)
    char *filename;
    int  flags;

.ft
.B CloseParamFile(pf)
    Paramfile\s0 *pf;

.ft B
\s-1short\s0 GetParamShort(pf, pname)
\s-1short\s0 GetParamShort(pf, pname)
\s-1int\s0 GetParamInt(pf, pname)
\s-1long\s0 GetParamLong(pf, pname)
\s-1float\s0 GetParamFloat(pf, pname)
\s-1double\s0 GetParamDouble(pf, pname)
\s-1char\s0 GetParamChar(pf, pname)
\s-1void\s0 GetParamString(pf, pname, value, nchars)
\s-1int\s0 GetParamBool(pf, pname)
    Paramfile *pf;
    char      *pname;
.ft

.ft B
\s-1void\s0 PutParamShort(pf, pname)
\s-1void\s0 PutParamInt(pf, pname, value)
\s-1void\s0 PutParamLong(pf, pname, value)
\s-1void\s0 PutParamFloat(pf, pname, value)
\s-1void\s0 PutParamDouble(pf, pname, value)
\s-1void\s0 PutParamChar(pf, pname, value)
\s-1void\s0 PutParamString(pf, pname, value)
\s-1void\s0 PutParamBool(pf, pname, value)
    Paramfile *pf;
    char      *pname;
    <type>     value;
.ft

.ft B
\s-1int\s0 AccessParam(pf, pname)
    Paramfile *pf;
    char      *pname;
.ft
.fi
These routines are discussed at length in separate man pages.
.SH ENVIRONMENT
.LP
The environment variable
.BR \s-1UPARM\s0
specifies the directory in which to look for the parameter
file, if the pathname of the file is not part of the
.I paramfile
specification.
.br
.ne 5
.SH "SEE ALSO"
.BR plist (l),
.BR pget (l),
.BR pset (l),
.BR pquery (l),
.SH AUTHOR
The original IRAF parameter interface was developed by the IRAF
development group at NOAO, led by Doug Tody.
.br
Eric Mandel and John Roll (SAO) developed the IRAF-compatible host interface.
