%{
/* ipar.y
**/

#include "pfile.h"

extern int *String2Mode();

/* These defines must be kept in sync with the %s statement 
** in the lexical analyser
**/

#define NAME	1
#define TYPE	2
#define VALUE	3
#define LINE	4
#define DQUOTE	5
#define SQUOTE	6

static ParamList *result;

int	yylineno;

Parameter *ParamCreate();
Parameter *ParamMinMaxPromptPart();

#define YYDEBUG 0
#define YYINITDEPTH 256
#define YYMAXDEPTH  257
%}

%union {
	ParamList	*plist;
	Parameter	*param;
	char		*stri;
}

%type <plist>	pfile
%type <plist>	plist

%type <param>	param
%type <param>	ptypedp
%type <param>	pstruct
%type <param>	pminmaxprompt
%type <stri>	peice
%type <stri>	lstruct

%token <param>	T_comment
%token <stri>	T_struct
%token <stri>	T_word
%token <stri>	T_line

%token 		T_nl

%%

pfile	: plist
		{ result = $1			}
	;

plist	: param
		{ if ( $1 && !$1->pvalue.value )
			VNewValue(&$1->pvalue, "", StringType);

		  if ( $1 != NULL )
			$$ = PListAdd(NULL, $1);
		}
	| plist	  param
		{ if ( $2 && !$2->pvalue.value )
			VNewValue(&$2->pvalue, "", StringType);

		  if ( $2 != NULL )
			$$ = PListAdd($1, $2);	}
	;
	  
param	: ptypedp
	| pstruct
	| T_comment T_nl
	| T_nl
		{ 		int	mode = 0;
		  $$ = ParamCreate("" , CommentType , &mode, "", NULL); 
		}
	;

ptypedp	:  T_word ',' T_word ',' T_word ',' T_word  pminmaxprompt T_nl
		{ 		int	mode;
		  $$ = ParamCreate($1
				, String2Type($3)
				, String2Mode($5, &mode), $7, $8); }
	| T_word ',' T_word ',' T_word ',' pminmaxprompt T_nl
		{ 		int	mode;
		  $$ = ParamCreate($1
				, String2Type($3)
				, String2Mode($5, &mode), NULL, $7); }
	;

pstruct	: T_word ',' T_struct ',' T_word ',' T_word pminmaxprompt T_nl
		{ 		int	mode = 0;
		  $<param>$ = ParamCreate($1
				, String2Type($3)
				, String2Mode($5, &mode), NULL, $8);

		  yybegin(LINE);
		}
	  lstruct
		{ $$ = ParamStruct($<param>10, $11);
		  yybegin(NAME)
		}
	;

lstruct : T_line     T_nl
		{ $$ = $1;		}
	| T_nl
		{ $$ = NULL;		}
	;

pminmaxprompt	:
		{ $$ = NULL;					}
	| peice
		{ $$ = NULL;
		  if ( $1 ) ParamMin(NULL, StringType, $1);
		}
	| peice peice
		{ 
		  $$ = NULL;
		  if ( $1 ) $$ = ParamMin(NULL, StringType, $1);
		  if ( $2 ) $$ = ParamMax(  $$, StringType, $2);
		}
	| peice peice peice
		{
		  $$ = NULL;
		  if ( $1 ) $$ = ParamMin( NULL, StringType, $1);
		  if ( $2 ) $$ = ParamMax(   $$, StringType, $2);
		  if ( $3 ) $$ = ParamPrompt($$, $3);
		}
	;

peice :   ','
		{ $$ = NULL					}
	| ',' T_word
		{ $$ = $2					}

%%


extern FILE *yyin;

ParamList *IRAFParFile(f)
			FILE	*f;
{
		static parser = 0;

	result = NULL;

	yylineno = 1;

	if ( parser ) {
		yyreparse();
		yyrestart(f);
	} else {
		yyin = f;
		parser = 1;
	}

	if ( yyparse() ) {
	  PFree(result);
	  return(NULL);
	}

	return result;
}

yyerror()
{
	extern FILE *yyin;
	extern int yyfield;
	static char	*fields[] = {
		"Name", "Type", "Mode", "Value", "Min", "Max", "Prompt"
	};

	if ( yyfield > 6 )
	  fprintf(stderr, "Error in parameter file: line %d: too many fields\n"
		 , yylineno);
	else
	  fprintf(stderr, "Error in parameter file: line %d: field %s\n"
		 , yylineno
		 , fields[yyfield]);
}
