/* PFile.c
**/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"
#include "pfile.h"
#include "ptemplat.h"
#include "parameter.h"
#include "putil.h"

extern ParamList *IRAFParOpen();
extern ParamList * RDBParOpen();
extern char *locatepfile();

void c_paramwrite();
void ParamFree ();

int		parerr;			/* Global error indicator	*/
paramfile	PFList;			/* Global list pointer		*/
paramfile	PFFile;			/* Global file pointer		*/

ParamFormat ParamFormats[3] = {
	 IRAFParOpen, ".par", paramfind
	, RDBParOpen, ".rdb", paramfind
	  , NULL		};

struct _ParamList _ParamFileCant;
ParamList *ParamFileCant = &_ParamFileCant;

static char pfile_last[129];

static int  ParamFileMode();

static char *ParamFName(filepath)
			char	*filepath;
{
		char	*filename;

#ifdef vms
	if ( filename = strrchr(filepath, ']') ) {
			char	*dot;

		filename++;
		if ( dot = strrchr(filename, '.') )
			*dot = '\0';

		return filename;
	}
#else
	if ( filename = strrchr(filepath, '/') )
		return ++filename;
#endif
	else
		return filepath;
}


static char *ParamFPath(filepath)
     char	*filepath;
{
  char	*filename;

#ifdef vms
  if ( filename = strrchr(filepath, ']') ) {
    char	*dot;

    *(++filename) = '\0';

    return filepath;
  }
#else
  if ( filename = strrchr(filepath, '/') ) {
    *(++filename) = '\0';
    return filepath;
#endif
  } else
    return NULL;
}

paramfile c_paramopen(filename, argv, argc, mode)
     char	*filename;
     char	*argv[];
     int	 argc;
     char	*mode;
{
  int		 i;
  ParamFile	 pf;
  ParamList	*plist = NULL;
  ParamList	*prefr = NULL;

  char		*filepath;
  char		 frontmode[10];
  char		 pbackmode[10];
  char		 fullname[256];

  strcpy(frontmode, mode);	/* Build up the search modes		*/
  strcpy(pbackmode, "r>");
  strcat(frontmode, "<");

  parerr = 0;

  if ( !filename || ( filename && *filename == '\0' ) )
    if ( argv && argc >= 2 && argv[1][0] == '@' ) {
	filename = ParamFName(&argv[1][1]);
	argv++;
	argc--;
    } else filename = ParamFName(argv[0]);


  /* save for errors */
  strncpy(pfile_last, filename, 128);
  pfile_last[128] = '0';

  if ( !mode ) mode = "rw";

  /* Look to see if the file is already open.
   */
  if( !strpbrk(mode, "f") ){
    char	*checkpath = mode;		/* Any path will do	*/

    if ( strpbrk(mode, "R") )			/* unless Restricted	*/
      checkpath = pbackmode;

    if ( (pf = ParamFileLookup(c_paramfind(filename, checkpath
					   , PARAMEXTN
					   , PARAMPATH))) )
      return pf;
  }

  /* Try each parameter file parser in turn.
   */
  for ( i = 0; !plist && !prefr && ParamFormats[i].open != NULL; i++ ) {

    /* If the open search mode is Reference or the pfile can't be found
     * in the front part of the search path.
     */
    if ( !(plist = (*ParamFormats[i].open)
	     ((*ParamFormats[i].find)(  filename, frontmode
				      , ParamFormats[i].extn
				      , PARAMPATH)
	      , mode))
	|| strpbrk(mode, "R") ) {

      /* Try to find the file in the back path
       */
      char	*refrname, *refrbase = NULL;

      refrbase = (char *)malloc(strlen(filename) + 1);	/* Kill any path */
      strcpy(refrbase, filename);
      refrname = ParamFName(refrbase);

      prefr = (*ParamFormats[i].open)
	((*ParamFormats[i].find)(refrname, pbackmode
				 , ParamFormats[i].extn
				 , PARAMPATH)
	 , pbackmode);

      free(refrbase);
      refrbase = NULL;
    }
  }

  if ( !plist && !prefr ) {		/* Can't find any pfile	*/
    parerr = PARNOFILE; 
    return NULL;
  }

  /* If reference is needed make sure it's ok
   */
  if ( strpbrk(mode, "R") && (!prefr || prefr == ParamFileCant) ) {
    parerr = PARNOREF;
    return NULL;
  }

  if ( plist == ParamFileCant ) 
    /* Something is wrong with the parameter file 
     */
    if ( strpbrk(mode, "R") )
      /* Bogus user copy but the reference is OK.
       */
      plist = NULL;
    else {
      parerr = PARCANTPARSE;
      return NULL;
    }

  if ( prefr ) {	/* If prefr was accessed then we are in R mode
			 * or plist was not found
			 */
    if ( plist ) {
      /* Open existing pfile from reference; 	"R" mode
       */
      fclose(prefr->file);	/* Get rid of the reference file's fp	*/

      prefr->file = plist->file;/* put user file's fp into ref's plist	*/
      prefr->mode = "w";
      plist->file = NULL;

      PFree(plist);		/* Toss out the users plist		*/
      plist = prefr;
    } else {
      /* Open new pfile from reference;		"R" mode or found only in back
       */

      /* The front path search failed but a reference copy was opened
       *
       * If no path was given in the input file name
       * look through front path for a writable directory
       */
      char *pathbase = NULL;

      pathbase = (char *)malloc(strlen(filename) + 1);
      strcpy(pathbase, filename);

      if ( !(filepath = ParamFPath(pathbase)) ) {
      	if ( !(filepath = Find(".", "wr<", NULL, PARAMPATH)) ) {
	  parerr = PARBACK2FRONT;
	  return NULL;
	}
	filepath[strlen(filepath) -1] = '\0';	/* Nuke the dot		*/
      }

      /* Substitute the reference file's file pointer with a newly opened
       * one in a user writable directory
       */
      plist = prefr;		/* The reference list is the new file	*/

      strcpy(fullname, filepath);
      strcat(fullname, ParamFName(plist->filename));
      fclose(plist->file);

      free(pathbase);

      if ( !(plist->file = fopen(fullname, "w")) ) {
	parerr = PARBACK2FRONT;
	return NULL;
      }
      /* Write the new guy out
       */
      (*plist->writ)(plist->file, plist, "");
    }
  }

  /* Got a plist now make it a ParamFile
   */
  pf = (struct _ParamFile *)malloc(sizeof(struct _ParamFile));
  pf->psets = plist;
		
  if ( argc > 1 )
    pf->alist = ParamFileArgs(pf->psets, &argv[1], argc -1);
  else
    pf->alist = NULL;


  /* Promote the mode parameter to the plist structure
   */
  pf->mode = ParamFileMode(pf) | (strchr(mode,'H') ? HHMODE : 0);

  /* Push the file on the Big list
   */
  pf->next = NULL;
  PFList = ListPush(PFList, pf);

  return pf;
}

void c_paramunlock(pf)
     ParamFile pf;
{
  if ( !pf ) return;

  UnLock(pf->psets->file);
}


void c_paramclose(pf)
     ParamFile	pf;
{
  ParamList	*plist;

  if ( !pf ) return;

  if (  pf->psets && strchr(pf->psets->mode, 'w') )
    c_paramwrite(pf, "l");

  /* Delete the parameter file from the global list	*/
  PFList = ListDelete(PFList, pf);

  fclose(pf->psets->file);
  free(pf->psets->filename);
  free(pf->psets->mode);
  free(pf->psets);
  free(pf);
  pf = NULL;
}




/* Sync with pfile.h */

static char *parammessage[] =  {
   "parameter error?"
 , "parameter not found"
 , "parameter not unique"
 , "parameter file not found"
 , "parameter argument error"
 , "positional parameters must come before absolutes"
 , "parameter has unknown type"
 , "parameter has bad minimum value"
 , "parameter has bad maximum value"
 , "parameter below minimum"
 , "parameter above maximum"
 , "parameter pointer missing"
 , "cannot convert parameter value"
 , "parameter has no value"
 , "command arguments missunderstood"
 , "no parameter file"
 , "no parameter converter"
 , "bad value for parameter"
 , "bad subfield name"
 , "can't specify range and max field"
 , "bad range specification"
 , "parameter type doesn't support range"
 , "value must be in range"
 , "to many positional arguments"
 , "error reading paramfile"
 , "enumerated value not unique"
 , "can't evaluate indirect parameter"
 , "can't open parameter list file"
 , "invalid enumerated value"
 , "can't copy reference parameter file to user directory"
 , "can't find reference parameter file"
};


static int exit_on_paramerror=1;

int set_paramerror(new)
     int new;
{
  int old;
  old = exit_on_paramerror;
  exit_on_paramerror = new;
  return(old);
}

void c_paramerr(level, message, name)
     int	 level;
     char	*message;
     char	*name;
{
  fprintf(stderr, "%s: %s : %s\n", message, parammessage[parerr], name);

  if ( level && exit_on_paramerror ) exit(level);

  parerr = PARNOERROR;
}


char *c_paramerrstr()
{
  char *message = parammessage[parerr];

  parerr = 0;
  return message;
}


static int ParamFileMode(pf)
     ParamFile pf;
{
  int	 hmode = HMODE;
  int	*pmode;
	
  if ( pmode = ParamGetX(pf, "mode", ModeType, &hmode) )
    return *pmode;

  parerr = 0;
  return 0;
}


void c_paramwrite(pf, mode)
     ParamFile	 pf;
     char		*mode;
{
  if ( pf->psets->writ ) (*pf->psets->writ)(pf->psets->file, pf->psets, mode);
}


ParamList *ParamFileArgs(plist, argv, argc)
     ParamList	*plist;
     char		*argv[];
     int		 argc;
{
  return IRAFParArgs(plist, argv, argc);
}


ParamFile ParamFileLookup(fname)
     char	*fname;
{
  ParamFile pf;

  if ( !fname )		return NULL;
  if ( *fname == '\0' )	return PFList;

  for ( pf = PFList; pf; pf = pf->next )
    if ( !strcmp(fname, pf->psets->filename) ) break;


  return pf;
}


char *c_paramfind(name, mode, extn, path)
     char *name, *mode, *extn, *path;
{
  char *retname;

  if ( retname = Find(name, mode, extn, path) )
    return retname;

  retname = locatepfile(name);
  return retname;
}


void c_paramlist(pf)
     ParamFile pf;
{
  Parameter 	**params;

  if ( !pf ) return;

  fprintf(stdout, "\nParameters for %s\n\n", pf->psets->filename);

  for ( params = pf->psets->param; *params; params++ ) {

    Value	 pvalue;
    Value	*xvalue;
    char	 dv[SZ_PFLINE];
    char	 *dvp;

    if ( ((*params)->pmode & HMODE) != 0 ) continue;

    if ( TypeIndex((*params)->ptype) == CommentType ) {
/*      fprintf(stdout, "%s\n", (char *) (*params)->pvalue.value); */
      continue;
    }

    dv[0] = '\0';
    dvp = VConvert(&(*params)->pvalue, dv, StringType);

    /* If the PVALUE of the value is an indirection lookup the
     * PWORTH of it. 
     */
    pvalue.value = NULL;
    pvalue.indir = NULL;
    VNewValue(&pvalue, (*params)->pvalue.value, (*params)->pvalue.type);
    if ( dvp && VIsIndirect(&pvalue) ) {
      strcat(dv, " -> ");
      if ( xvalue = VAccess(pf, *params, StringType, &pvalue) ) {
	char	*foo;
	strcat(dv, foo = QuoteValue(pvalue.value, NULL));
	free(foo);
      } else
	strcat(dv, "INDEF");
    }
    if ( pvalue.value ) free(pvalue.value);

    fprintf(stdout,   "%14s = %-16s %s\n"
	   , (*params)->pname
	   , dvp        ? dv     : ""
	   , (*params)->pprompt ? (*params)->pprompt : "");
  }

  for ( params = pf->psets->param; *params; params++ ) {

    Value	 pvalue;
    Value	*xvalue;
    char	 dv[SZ_PFLINE];
    char	 name[SZ_PFLINE];
    char	 value[SZ_PFLINE];
    char	 *dvp;

    if ( ((*params)->pmode & HMODE) == 0 ) continue;

    if ( TypeIndex((*params)->ptype) == CommentType ) {
/*      fprintf(stdout, "%s\n", (char *) (*params)->pvalue.value); */
      continue;
    }

    dv[0] = '\0';
    dvp = VConvert(&(*params)->pvalue, dv, StringType);

    /* If the PVALUE of the value is an indirection lookup the
     * PWORTH of it. 
     */
    pvalue.value = NULL;
    pvalue.indir = NULL;
    VNewValue(&pvalue, (*params)->pvalue.value, (*params)->pvalue.type);
    if ( dvp && VIsIndirect(&pvalue) ) {
      strcat(dv, " -> ");
      if ( xvalue = VAccess(pf, *params, StringType, &pvalue) )
	strcat(dv, pvalue.value);
      else
	strcat(dv, "INDEF");
    }
    if ( pvalue.value ) free(pvalue.value);

    strcpy(name, "(");
    strncat(name, (*params)->pname, SZ_PFLINE-2);
    name[SZ_PFLINE-1] = '\0';
    if( dvp ){
      strncpy(value, dv, SZ_PFLINE);
      value[SZ_PFLINE-1] = '\0';
    }
    else
      *value = '\0';
    strcat(value, ")");
    fprintf(stdout, "%14s = %-16s %s\n"
	   , name
	   , value
	   , (*params)->pprompt ? (*params)->pprompt : "");
  }
  fprintf(stdout, "\n");
}


/* Lookup a Parameter in a plist with exact strcmp
 */
Parameter *ParamLookup(plist, pname)
			ParamList	*plist;
			char		*pname;
{
  Parameter	**param;

  if ( !plist ) return NULL;

  for ( param = plist->param; *param; param++ ) {
    if ( !strcmp((*param)->pname, pname) ) return (*param);
  }
  
  return NULL;
}

/* Lookup a Parameter in a plist with uniq match
 */
char *ParamMatch(plist, pname)
     ParamList	*plist;
     char		*pname;
{
  Parameter	*a = NULL;
  Parameter	*b = NULL;
  Parameter	**param;
  char 		name[256];
  char 		*pn;

  /* don't try to look up extension part */
  strcpy(name, pname);
  if ( pn = strrchr(name, '.') ) {
	  if ( !strcmp(pn, ".p_mode") ||
	       !strcmp(pn, ".p_value")||
	       !strcmp(pn, ".p_min")  ||
	       !strcmp(pn, ".p_max")  ||
	       !strcmp(pn, ".p_prompt") ){
	    *pn = '\0';
	  }
  }

  for ( param = plist->param; *param; param++ ) {

    if ( !strcmp((*param)->pname, name) )	return pname;

    if ( ( strstr((*param)->pname, name) == (*param)->pname ) )
      if ( a && !b )	b = *param;
      else     		a = *param;
  }

  if ( a && b ) {
    fprintf(stderr, "parammatch : parameter name %s not uniq (%s, %s)\n"
	   , pname
	   , a->pname, b->pname);

    parerr = PARNOTUNIQ;
    return NULL;
  }

  if ( a == NULL ) {
    fprintf(stderr, "parammatch : can't find parameter %s in reference list\n"
	   , pname);
    return NULL;
  }

  free(pname);

  pname = NewString(a->pname);
  return pname;
}


/* Parameter structure contruction routines */

ParamList *PFree(plist)
			ParamList	*plist;
{
			Parameter	**param;

	if ( !plist ) return(ParamList *)NULL;

	for ( param = plist->param; *param; param++ )
		ParamFree(*param);

	free(plist);
        plist = (ParamList *)NULL;			
	return(plist);
}


void ParamFree(param)
     Parameter	*param;
{
  if ( param->pname )	 	free(param->pname);
  if ( param->pvalue.value ) 	free(param->pvalue.value);
  if ( param->pmin.value ) 	free(param->pmin.value);
  if ( param->pmax.value ) 	free(param->pmax.value);
  if ( param->pprompt ) 	free(param->pprompt);
	
  free(param);
}


ParamList *PListAdd(plist, param)
     ParamList	*plist;
     Parameter	*param;
{
  if ( !param ) return plist;

  if ( !plist ) {
    plist = (ParamList *)malloc(sizeof(ParamList));
    plist->nalloc = 10;
    plist->nparam = 0;
  }

  if ( plist->nparam + 1 >= plist->nalloc ) {
    plist->nalloc += PFILE_SZ;
    plist = realloc(plist, sizeof(ParamList) + 4 * (plist->nalloc - 10));
  }

  plist->param[plist->nparam++] = param;
  plist->param[plist->nparam]   = NULL;

  return plist;
}

ParamList *PListDel(plist, param)
     ParamList	*plist;
     Parameter	*param;
{
  Parameter	**p = plist->param;

  while ( *p && *p != param ) p++;

  if ( *p ) {
    int	n = plist->nparam - (++p - plist->param);

    bcopy(p, p - 1, n);
    plist->param[--plist->nparam] = NULL;
  }

  return plist;
}



int	c_paccess(pfile, pname)
     ParamFile	 pfile;
     char		*pname;
{
  if ( ParamLookup(pfile->psets, pname) ) {
    return 1;
  } else {
    parerr = 0;
    return 0;
  }
}

char *pfilelast()
{
  return(pfile_last);
}
