/*
 *
 * putil.c -- refugees from the old Os directory
 *
 */

/* Access.c
**
** Ask the os if a file is accessible.
**
	char *Access(filename, mode)
	char *Find(filename, mode, extn, path)
**/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <varargs.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include "putil.h"

static char AccessName[1024 + 2];

static int amparse(mode)
     char *mode;
{
  int xmode = 0;

  xmode |= ( strpbrk(mode, "r") != NULL ? R_OK 	: 0 );
  xmode |= ( strpbrk(mode, "w") != NULL ? W_OK 	: 0 );
  xmode |= ( strpbrk(mode, "x") != NULL ? X_OK	: 0 );
  xmode |= ( strpbrk(mode, "f") != NULL ? F_OK 	: 0 );

  return xmode;
}	

static char *findpath(name, mode, path)
     char *name;
     char *mode;
     char *path;
{
  char	pathbuff[4096];
  char	namebuff[1024];
  char	backmode[10];
  char 	*here, *found;
  int 	 mark = 0;
  int	 skip = strpbrk(mode, ">") != NULL;
  int	 pick = strpbrk(mode, "<") != NULL;

  if ( skip && pick ) return NULL;

  if ( !path || ( name[0] == '.' && name[1] == '/' ) || name[0] == '/' )
    return Access(name, mode);

  strcpy(pathbuff, path);
  path = pathbuff;

  if ( here = strpbrk(pathbuff, " :;") ) {
    mark = *here;
    *here++ = '\0';
  }
  while ( path ) {
    if ( path[0] == '$' ) {
      if ( path = getenv(&path[1]) )
	if ( found = findpath(name, mode, path) )
	  return found;
    } else {      
      if ( !skip ) {      
	if ( !strcmp(".", path) ) path[0] = '\0';

	strcpy(namebuff, path);
	if ( path[0] && path[strlen(path) - 1] != '/' )
	  strcat(namebuff, "/");
	strcat(namebuff, name);

	if ( found = Access(namebuff, mode) )
	  return found;
      }
    }

    if ( mark == ';' ) {
      if ( skip ) {
	skip = 0;
	/* Knock down the skip mode to select all
	 * directories in path after the first ";"
	 */
	strcpy(backmode, mode);
	backmode[strlen(backmode) -1] = '\0';
	mode = backmode;
      }
      if ( pick ) return NULL;
    }

    path = here;
    if ( here && (here = strpbrk(here, " :;")) ) {
      mark = *here;
      *here++ = '\0';
    }
  }

  return NULL;
}


char *Access (name, mode)
     char *name;
     char *mode;
{
  struct stat info;

  if ( stat(name, &info) !=0 ) return NULL;

  if ( mode ) {
    int m = amparse(mode);

    if ( getuid() == info.st_uid ) {
	if ( m & R_OK && !(info.st_mode & S_IRUSR) ) return NULL;
	if ( m & W_OK && !(info.st_mode & S_IWUSR) ) return NULL;
	if ( m & X_OK && !(info.st_mode & S_IXUSR) ) return NULL;
    } else
      if ( getgid() == info.st_gid ) {
	if ( m & R_OK && !(info.st_mode & S_IRGRP) ) return NULL;
	if ( m & W_OK && !(info.st_mode & S_IWGRP) ) return NULL;
	if ( m & X_OK && !(info.st_mode & S_IXGRP) ) return NULL;
      } else {
	if ( m & R_OK && !(info.st_mode & S_IROTH) ) return NULL;
	if ( m & W_OK && !(info.st_mode & S_IWOTH) ) return NULL;
	if ( m & X_OK && !(info.st_mode & S_IXOTH) ) return NULL;
      }
  }

  strcpy(AccessName, name);
  return name;
}

char *Find (name, mode, extn, path)
     char *name;
     char *mode;
     char *extn;
     char *path;
{
  char	extnbuff[1024];
  char	namebuff[1024];
  char 	*here, *found;

  if ( !extn )
    return findpath(name, mode, path);
  
  strcpy(extnbuff, extn);
  extn = extnbuff;

  if ( here = strpbrk(extnbuff, " :;") ) *here++ = '\0';

  while ( extn ) {
    if ( extn[0] == '$' ) {
      if ( extn = getenv(&extn[1]) )
	if ( found = Find(name, mode, extn, path) )
	  return found;
    } else {
      char *e = strstr(name, extn);
      
      strcpy(namebuff, name);
      if ( !e || ( e && *(e + strlen(extn))) )
	strcat(namebuff, extn);
      
      if ( (found = findpath(namebuff, mode, path)) )
	return found;
      
    }
    
    extn = here;
    if ( here && (here = strpbrk(here, " :;")) ) *here++ = '\0';
  }
  
  return NULL;
}

#ifndef vms
int Lock(f)
     FILE *f;
{
#ifdef LOCK
  struct flock lock;

  lock.l_type = F_WRLCK;
  lock.l_whence = SEEK_SET;
  lock.l_start = 0;
  lock.l_len = 0;
  return fcntl(fileno(f), F_SETLKW, &lock);
#endif
}


int UnLock(f)
     FILE *f;
{
#ifdef LOCK
  struct flock lock;

  lock.l_type = F_UNLCK;
  lock.l_whence = SEEK_SET;
  lock.l_start = 0;
  lock.l_len = 0;
  return fcntl(fileno(f), F_SETLKW, &lock);
#endif
}
#endif

/*
 * setterm.c 
 */

/* UNIX specific stuff */

#ifdef SunOS_4
extern int ioctl(/* int fd, int request, void *arg */);
#endif

/* hacks to make BSDI work */
#ifdef __386BSD__
#define _POSIX_SOURCE 1
#define TERMIOS 1
#endif

#ifdef TERMIOS
#include <termios.h>
static struct termios orig_termio, rl_termio;
#else
#include <termio.h>
static struct termio orig_termio, rl_termio;
#endif /* TERMIOS */
static int term_set = 0;	/* =1 if rl_termio set */

/* set termio so we can do our own input processing */
void  set_termio (fd)
     int fd;
{
#ifndef MSDOS
	if(term_set == 0) {
#ifdef TERMIOS
#ifdef TCGETS
		ioctl(fd, TCGETS, &orig_termio);
#else
		tcgetattr(fd, &orig_termio);
#endif /* TCGETS */
#else
		ioctl(fd, TCGETA, &orig_termio);
#endif /* TERMIOS */
		rl_termio = orig_termio;

#ifdef _POSIX_SOURCE
		rl_termio.c_iflag &= ~(BRKINT|PARMRK|INPCK|IXON|IXOFF);
#else
		rl_termio.c_iflag &= ~(BRKINT|PARMRK|INPCK|IUCLC|IXON|IXOFF);
#endif
		rl_termio.c_iflag |=  (IGNBRK|IGNPAR);

#ifndef _POSIX_SOURCE
		rl_termio.c_oflag &= ~(ONOCR);
#endif

		rl_termio.c_lflag &= ~(ICANON|ECHO|ECHOE|ECHOK|ECHONL|NOFLSH);
		rl_termio.c_lflag |=  (ISIG);

		rl_termio.c_cc[VMIN] = 1;
		rl_termio.c_cc[VTIME] = 0;

#ifdef VSUSP
		/* disable suspending process on ^Z */
		rl_termio.c_cc[VSUSP] = 0;
#endif /* VSUSP */

#ifdef VINTR
		/* disable suspending process on ^C */
		rl_termio.c_cc[VINTR] = 0;
#endif /* VINTR */

#ifdef TERMIOS
#ifdef TCSETSW
		ioctl(fd, TCSETSW, &rl_termio);
#else
		tcsetattr(fd, TCSADRAIN, &rl_termio);
#endif /* TCSETSW */
#else
		ioctl(fd, TCSETAW, &rl_termio);
#endif /* TERMIOS */
		term_set = 1;
	}
#endif /* MSDOS */
}

void reset_termio(fd)
     int fd;
{
#ifndef MSDOS
	if(term_set == 1) {
#ifdef TERMIOS
#ifdef TCSETSW
		ioctl(fd, TCSETSW, &orig_termio);
#else
		tcsetattr(fd, TCSADRAIN, &orig_termio);
#endif /* TCSETSW */
#else
		ioctl(fd, TCSETAW, &orig_termio);
#endif /* TERMIOS */
		term_set = 0;
	}
#endif /* MSDOS */
}

/*
 * 
 * list.c
 *
 */

/* list.c
**/

#include "list.h"


void *ListInsert(list, member, at)
     List *list;
     List *member;
     List *at;
{
		List	*this;
		List	*prev = NULL;

		List	*mlast;

	for ( this = list; this != at; this = this->next )
		prev = this;

	mlast = member;
	if ( mlast ) 
		while ( mlast->next != NULL ) mlast = mlast->next;

	if ( prev ) {
		prev->next 	= member;
		mlast->next 	= this;
	} else {
		if ( member ) {
			mlast->next 	= list;
			list		= member;
		};
	}

	return (void *) list;
}
	

void *ListDelete (list, member)
     List *list;
     List *member;
{
		List	*this;
		List	*prev = NULL;

	for ( this = list; this != member; this = this->next )
		prev = this;

	if ( prev )
		prev->next 	= this->next;
	else
		list		= this->next;

	member->next = NULL;
	return (void *) list;
}

/*
 *
 * pargs.c
 *
 */

#include "pargs.h"

ArgStream OpenArgv(nth, separator, argv, argc)
    int             nth;
    char            separator;
    char           *argv[];
    int             argc;
{
    ArgStream       a;

    a = malloc(sizeof(struct _ArgStream));

    a->ungotten = 0;

    a->here = argv[nth];
    a->this = nth;
    a->argv = argv;
    a->argc = argc;
    a->sep = separator;

    return a;
}


void CloseArgv(a)
    ArgStream       a;
{
    free(a);
    a = NULL;
}

char *NewString(str)
     char *str;
{
  char *s;
  if( str == NULL )
    return(NULL);
  s = (char *)malloc((unsigned)strlen(str)+1);
  strcpy(s, str);
  return(s);
}
