/* range.c
*/

#include <string.h>
#include <stdlib.h>

#include "pfile.h"
#include "ptemplat.h"
#include "parameter.h"
#include "range.h"

vtype	rangeRec = {
	  "range"
	, NULL

	, NULL
	, NULL
	, NULL
};

int VIsNotRange(value)
			Value	*value;
{
	return  !( value 
		&& value->value 
		&& TypeIndex(value->type) == StringType
		&& strpbrk(value->value, "|:!"));
}


Value *VInRange(range, value)
			Range	*range;
			Value	*value;
{
		int 	 i;
		char	*a = NULL;

	if ( !VTypeList[TypeIndex(range->type)]->compar ) {
		parerr = PARCANTRANGE;
		return NULL;
	}

	if ( range->type != value->type ) {
		fprintf (stderr, "INTERNAL TYPE MISMATCH range != value\n");
		exit(2);
	}

	/* Check is the value is an alowable sub-string of
	** an enumeration
	**/
	if ( TypeIndex(range->type) == StringType ) {

	    for ( i = 0; i < range->nval; i++ ) {
		if ( !strcmp(range->check[i].valu, value->value) )
			return value;

		if ( strstr(range->check[i].valu, value->value)
			== range->check[i].valu )
		    if ( a )  {
			parerr = PARENUMUNIQ;
			return NULL;
		    } else	a = range->check[i].valu;
	    }
	    if( a )
	      return VNewValue(value, a, StringType);
	    else{
	      parerr = PARBADENUM;
	      return NULL;
	    }
	}

		
	for ( i = 0; i < range->nval; i++ )
	    if ( VCompar(range->type, value->value, range->check[i].valu
					   , range->check[i].oper) )
			 return value;

	parerr = PARLIMITRANGE;
	return NULL;
}


Range *RangeNew(type)
			VType	type;
{
		Range	*evalue;

	evalue = (Range *)malloc(sizeof(Range));

	evalue->type = type;
	evalue->nval = 0;
	evalue->nalo = 0;

	return evalue;
}


Range *RangeAdd(range, valu, oper)
			Range	*range;
			void	*valu;
			int	 oper;
{
	if ( range->nval >= range->nalo ) {
	        if( range != NULL )
		  range = realloc(range,
			  sizeof(Range)+(range->nalo+20)*sizeof(Check));
		else
		  range = malloc(sizeof(Range)+(range->nalo+20)*sizeof(Check));

		range->nalo += 20;
	}

	bcopy(valu
		, range->check[range->nval].valu
		, VSize(range->type, valu));
	range->check[range->nval++].oper = oper;

	return range;
}


Value *VRange(pfile, type, value)
			ParamFile	 pfile;
			VType		 type;
			Value		*value;
{
		char	 string[SZ_PFLINE];
		char	*tok;
		void	*xvalue;
		Range	*range = NULL;
		

	if ( value->type == RangeType ) return value;

	if ( VIsIndirect(value) )	VDirect(pfile, value);

	if ( VIsNotRange(value) )
		return NULL;

	strcpy(string, value->value);

	range = RangeNew(type);

	for ( tok = strtok(string, "|"); tok; tok = strtok(NULL, "|") ) {
		xvalue = Convert(tok,  StringType
			       , NULL, type);

		if ( xvalue )
			range = RangeAdd(range, xvalue, PRange_EQ);
		else {
			parerr = PARBADRANGE;
			return NULL;
		}
	}

	range->str   = value->value;
	value->type  = RangeType;
	value->value = range;

	return value;
}



void *Range2String(range, string)
			Range	*range;
			char	*string;
{
		char	*str;
		int	 i;

	if ( !range ) return NULL;
	if ( !string ) return NULL;

	*string = '\0';
	for ( i = 0; i < range->nval; i++ ) {
		if ( str = Convert(range->check[i].valu, range->type
				 , NULL, StringType) ) {

			if ( *string == '\0' )
				strcat(string, str);
			else {
				strcat(string, "|");
				strcat(string, str);
			}				
		} else	return NULL;
	}

	return string;
}



Value *PRangeMinMax(value, min, max)
			Value *value;
			Value *min;
			Value *max;
{
	if ( !value ) return NULL;

	if ( ( min && value->type != min->type )
	 ||  ( max && value->type != max->type ) ) {
		fprintf(stderr, "INTERNAL TYPE MISMATCH range != value\n");
		exit(2);
	}

	if ( min && !VCompar(value->type, value->value, min->value, PRange_GE) ) {
		parerr = PARLIMITMIN;
		return NULL;
	}

	if ( max && !VCompar(value->type, value->value, max->value, PRange_LE) ) {
		parerr = PARLIMITMAX;
		return NULL;
	}

	return value;
}
