#ifndef JCMDM_H
#define JCMDM_H
#include "jcmlib.h"

typedef enum { dmTABLE, dmIMAGE, dmUNKNOWNBLOCK } dmBlockType;
typedef enum { dmSHORT, dmLONG, dmFLOAT, dmDOUBLE, dmSTRING, dmBLOCKREF, dmLOGICAL, dmBYTE, dmUSHORT, dmULONG, dmVOID } dmDataType;
#define dmNOMOREROWS -1


#ifdef DMSOURCE
#include "dmi.h"
#else
typedef void* dmDataset;
typedef void* dmBlock;
typedef void* dmDescriptor;
#endif


/*
 * Initialization
 */

void dmInit( void );
dmDataset* dmDatasetOpen( char* file );
dmDataset* dmDatasetCreate( char* filename );
int dmDatasetClose( dmDataset* ds );

dmBlock* dmBlockOpen( dmDataset* ds, char* name );
int dmBlockClose( dmBlock* block );
dmBlock* dmBlockCreate( dmDataset* ds, char* name, dmBlockType type );
dmBlock* dmImageCreate( char* name, dmDataType type, long* axes, long naxes );

int dmDatasetTableClose( dmBlock* block );
dmBlock* dmDatasetTableOpen( char* name );
dmBlock* dmDatasetTableCreate( char* name );

dmBlockType dmBlockGetType( dmBlock* block );
int dmBlockGetNo( dmBlock* block );
dmDataset* dmBlockGetDataset( dmBlock* block );
int dmDatasetGetNoBlocks( dmDataset* ds );
int dmDatasetGetCurrentBlockNo( dmDataset* ds );
dmBlock* dmDatasetNextBlock( dmDataset* ds );
dmBlock* dmDatasetMoveToBlock( dmDataset* ds, int no );
dmBlock* dmDatasetAdvanceBlocks( dmDataset* ds, int rel_no );
dmBlock* dmImageOpen( char* name );
int dmImageClose( dmBlock* block );
dmDescriptor* dmKeyWrite_d( dmBlock* block, char* name, double value, char* unit, char* desc );
/* 
 * ASCDM compatibility
 */
dmBlockType dmASCBlockType( char* typename );
void dmASCBlockTypeName( dmBlockType type, char* typename );
void dmASCDataTypeName( dmDataType type, char* typename );
dmDataType dmASCDataType( char* typename );
long dmTablePutRow( dmBlock* block, void* data );
float dmGetScalar_r( dmDescriptor* dd );

long dmTableGetNoRows( dmBlock* );

double dmGetScalar_d( dmDescriptor* );
long dmTableNextRow( dmBlock* block );
dmDescriptor* dmKeyRead_l( dmBlock*, char*, long* );
dmDescriptor* dmKeyRead_d( dmBlock* block, char* name, double* value );

void dmGetStaticArray_d( dmDescriptor*, double*, int );

dmDescriptor* dmTableOpenColumnNo( dmBlock*, long );
long dmGetScalars_d( dmDescriptor*, double*, long, long );
int dmCoordGetTransform_d( dmDescriptor*, double*, double*, double*,
 long  );
dmDescriptor* dmDescriptorGetCoord( dmDescriptor* );
dmDescriptor* dmTableOpenColumn( dmBlock*, char* ); 

/*
 * Jonathan's own routines
 *
 */

int dmBlockGetTypeName( dmBlock* block, char* name );
void dmDatasetGetBlockTypeName( dmDataset* ds, int block_no, char* name );


dmBlockType dmBlockGetType( dmBlock* block );
char* dmBlockGetTypeStr( dmBlock* block );
dmBlockType dmDatasetGetBlockType( dmDataset* ds, int block_no );
dmBlock* dmImageCreateC( char* name, char* type, long* axes, long naxes );
dmBlock* dmBlockCreateC( dmDataset* ds, char* name, char* type );

void dmGetNameC( dmDescriptor* dd, char* name );
void dmGetUnitC( dmDescriptor* dd, char* unit );
void dmGetDescC( dmDescriptor* dd, char* desc );
void dmGetDataTypeNameC( dmDescriptor* dd, char* typename );

long dmGetArrDim( dmDescriptor* dd );

char* dmGetName( dmDescriptor* dd );
char* dmGetUnit( dmDescriptor* dd );
char* dmGetDesc( dmDescriptor* dd );
dmDataType dmGetDataType( dmDescriptor* dd );
char* dmGetDataTypeName( dmDescriptor* dd );

int dmSetKernelOption( char* option );
int dmKernelList( void );
int dmBlockPrintKernel( dmBlock* block );
int dmBlockPrint( dmBlock* block, char* options );

void dmStatus( dmDataset* ds );
void dmBlockGetNameC( dmBlock* block, char* name, int size );

int dmSetScalar_d( dmDescriptor* dd, double );
dmDescriptor* dmColumnCreate( dmBlock* table, char* name,  dmDataType type, int slen, char* unit, char* desc );
#endif
