#include <stdlib.h>
#define DMSOURCE
#include "ascdm.h"


/*
 * Allocate the dmBlock object
 */

dmBlock* dmiBlockAlloc( dmDataset* ds, dmKernelBlock* kblock, char* name, dmDataType type )
{
 dmBlock* block;
 card buf;
 dmsg( 1, "dmiBlockAlloc" );
 if ( ds->nblocks >= dmMAXBLOCKS )  {
/* Should just reallocate block here; for now, fail. */
  dmiErrorSet( ERR_TOO_MANY_BLOCKS );
  return( NULL );
 }
 block = (dmBlock*) malloc( sizeof( dmBlock ) );
 sprintf( buf, "*** ALLOC %12ld dmBlock %s", (long)block, name ); dmsg( 2, buf );
 ds->nblocks++;
 ds->block[ds->nblocks-1] = block;
 block->ds = ds;
 block->no = ds->nblocks;
 cs_copy_siz( name, block->name, CSIZE );
 block->type = type;
 block->KernelBlock = kblock;
 block->key = NULL;
 block->nkeys = 0;
 block->ncols = 0;
 block->ndss  = 0;
 block->nrows = 0;
 return( block );
}

/*
 * dmiBlockFree
 *
 * Free memory allocated to a block.
 * Clear it from the list of blocks in the parent dataset.
 */
int dmiBlockFree( dmBlock* block )
{
 dmDataset* ds;
 card buf;
 int offset;
 if ( block ) { 
  ds = block->ds;
  offset = block->no - 1;
  dmiBlockStructFree( block );
  sprintf( buf, "*** FREE  %12ld dmBlock ***", (long) block ); dmsg( 2, buf );
  free( block );
  ds->block[offset] = NULL;
 }
 return( 0 );
}



/*
 * dmiBlockStructAlloc
 *
 * Allocate memory for the block descriptors.
 */
void dmiBlockStructAlloc( dmBlock* block )
{
 card buf;
 block->key    = (dmDescriptor**)malloc( dmMAXDD * sizeof( dmDescriptor* ) );
 block->column = (dmDescriptor**)malloc( dmMAXDD * sizeof( dmDescriptor* ) );
 block->dss    = (dmDescriptor**)malloc( dmMAXDD * sizeof( dmDescriptor* ) );
 sprintf( buf, "*** ALLOC %12ld dmDescriptors ***", (long)block->key ); dmsg( 2, buf );
}

/*
 * dmiBlockStructFree
 *
 * Release the structure information for a block. Delete all of the descriptors
 * in the block, then free the descriptor trees.
 *
 */
void dmiBlockStructFree( dmBlock* block ) 
{
 card buf;
 int i;
 if ( block->key ) {
  sprintf( buf, "*** FREE  %12ld dmDescriptors ***", (long)block->key ); dmsg( 2, buf );
  sprintf( buf, "Free keys %d", (int)block->nkeys );
  dmsg( 3, buf );
  for ( i = 0; i < block->nkeys; i++ ) {
   dmiDescriptorFree( block->key[i] );
  }
  dmsg( 3, "Free cols" );
  for ( i = 0; i < block->ncols; i++ ) {
   dmiDescriptorFree( block->column[i] );
  }
  dmsg( 3, "Free dss" );
  for ( i = 0; i < block->ndss; i++ ) {
   dmiDescriptorFree( block->dss[i] );
  }
  dmsg( 3, "Free descriptor arrays" );
  free( block->key );
  free( block->column );
  free( block->dss );
  block->key    = NULL;
  block->column = NULL;
  block->dss    = NULL;
  block->nkeys = 0;
  block->ncols = 0;
  block->ndss = 0;
 }
}


/*
 * dmiDescriptorAlloc
 *
 */

dmDescriptor* dmiDescriptorAlloc( dmBlock* block, char* name, dmDataType data_type, dmDescType descriptor_type, int no )
{
 dmDescriptor* dd;

 dd = (dmDescriptor*)malloc( sizeof( dmDescriptor ) );
 dmsg1( 3, "Alloc dd ", name );
 cs_copy( name, dd->name );
 dd->block = block;
 dd->data_type = data_type;
 dd->descriptor_type = descriptor_type;
 dd->no = no;
 dd->naxes = 0;
 dd->coord = NULL;
 if ( descriptor_type == ddCOORD ) {
  dmsg1( 3, "Alloc Transform ", name );
  dd->value = (dmTransform*)malloc( sizeof ( dmTransform ) );
 }
 return( dd );
}

/*
 * dmiDescriptorFree
 *
 * Free descriptor. Recursively free attached coordinate systems.
 */
void dmiDescriptorFree( dmDescriptor* dd )
{
 card buf;
 if ( dd ) {
  sprintf( buf, "Free dd %s", dd->name );  dmsg( 3, buf );
  if ( dd->coord ) dmiDescriptorFree( dd->coord );
  if ( dd->descriptor_type == ddCOORD ) {
   dmsg( 3, "Free transform" );
   free( (dmTransform*)dd->value );
  }
  free( dd );
 }
}



