/*
 * Data Model
 * Version JCM C 1.0: CFITSIO based
 * Internal definitions
 */
#ifndef DMI_H
#define DMI_H
#include "jcmlib.h"
#include "fitsio.h"
#include "dmklib.h"

#define MAXAXES 10
#define dmMAXBLOCKS 100
#define dmMAXDD     1000
#define dmREAD  0
#define dmWRITE 1
#include "dmerr.h"

typedef struct dmBlock_ dmBlock;
typedef struct dmDescriptor_ dmDescriptor;

typedef enum { ddCOLUMN, ddKEY, ddFILTER, ddCOORD } dmDescType;

typedef struct {
 double crpix;
 double crval;
 double cdelt;
} dmTransform;

typedef struct {
 dmKernel* kernel;           /* Which kernel in use for this dataset? */
 dmKernelDataset* fptr;           /* FITS file pointer */
 int no;               /* Number of dataset */
 t_filename name;      /* Dataset name */
 int iomode;          /* Read or write? */
 int nblocks;          /* Number of blocks in dataset */
 int current_block;     /* Current block number */
 logical scanned;      /* List of blocks is complete */
 dmBlock** block;       /* Block list */
} dmDataset;

struct dmBlock_ {
 dmKernelBlock* KernelBlock;  /* Kernel block structure */
 dmDataset* ds;        /* Parent dataset */
 long no; 		/* Number of block in dataset */
 dmDataType type;           /* Type of block (TABLE or IMAGE) */
 card name;		/* Name of block */
 long nkeys;		/* Number of keys in block */
 long ncols;		/* Number of columns in block */
 long ndss;		/* Number of Data Subspace filters */
 long nrows;		/* Number of table rows */
 long ncpts;		/* Number of Data Subspace rows */
 long current_row;      /* Current row number */
 dmDescriptor** key;	/* List of keys */
 dmDescriptor** column;  /* List of columns */
 dmDescriptor** dss;	/* List of Data Subspace descriptors */
};

struct dmDescriptor_ {
 card name;       /* Name */
 card unit;       /* Unit */
 card desc;       /* Descriptor */
 dmDataType data_type; /* Data type integer code */
 dmDescType descriptor_type;  /* What kind of descriptor? */
 dmBlock* block;  /* Which block it belongs to */
 int no; /* Cardinality of descriptor */
 int KernelNo; /* Pointer to kernel descriptor */
 int naxes;
 int axes[MAXAXES];
 void* value;       /* Or pointer to coord if ddtype = COORD */
 void* tlmin;
 void* tlmax;
 dmDescriptor* coord;  /* Pointer to coord descriptor */
};


void dmiInit( void );
void dmiFree( void );
int dmiErrorGet( void );
void dmiErrorSet( int status /* FITSIO error code */ );

dmDataset* dmiDatasetAlloc( char* name, dmKernel* kernel, dmKernelDataset* fptr, char* mode );
void dmiDatasetFree( dmDataset* ds );
dmBlock* dmiDatasetGetBlock( dmDataset* ds, char* name );
dmBlock* dmiBlockAlloc( dmDataset* ds, dmKernelBlock* kblock, char* name, dmDataType type );
int dmiBlockFree( dmBlock* block );
int dmiBlockOpen( dmBlock* block );
void dmiBlockStructAlloc( dmBlock* block );
void dmiBlockStructFree( dmBlock* block ) ;

dmBlock* dmiBlockFind( dmDataset* ds, char* name );
int dmiDatasetScan( dmDataset* ds, int block_no );
int dmiBlockCreate( dmBlock* block );

void dmiSyntaxParse( char* name, char* dsname, char* block_name );
dmDescriptor* dmiDescriptorAlloc( dmBlock* block, char* name, dmDataType type, dmDescType ddtype, int no );
void dmiDescriptorFree( dmDescriptor* dd );
void dmiImageSpec( dmBlock* block, dmDataType type, long* axes, long naxes );
void dmiHeaderOpen( dmBlock* block );
void dmiDescriptorFormatValue( dmDescriptor* key, char* value, int size );

int dmGetNoKernels( void );
dmKernel* dmGetKernelNo( int i );

void dmiBlockPrintHeader( dmBlock* block );
void dmiBlockPrintTable( dmBlock* block );
void dmiBlockPrintKey( dmDescriptor* key );
void dmiBlockPrintColumn( dmDescriptor* key );
void dmiStoreCoord( dmDescriptor* coord, double crpix, double crval, double cdelt );

#endif
