#ifndef DMKLIB_H
#define DMKLIB_H



/* 
 * Each kernel must define the following functions.
 * At run time, the kernel pointer selects the actual function for the current kernel.
 */

typedef void* dmKernelDataset;
typedef void* dmKernelBlock;

typedef struct {
 card name;
 dmKernelDataset* (*dmkDatasetOpen)( char* );
 dmKernelDataset* (*dmkDatasetCreate)( char* );
 int (*dmkDatasetClose)( dmKernelDataset* );
 dmKernelBlock* (*dmkBlockFindNext)( dmKernelDataset*, char*, dmBlockType* );
 dmKernelBlock* (*dmkBlockCreate)( dmKernelDataset*, char*, dmBlockType );
 int (*dmkMoveToBlock)( dmKernelBlock* );
 int (*dmkBlockClose)( dmKernelBlock* );
 int (*dmkTableOpen)( dmKernelBlock*, long*, long * );
 int (*dmkImageOpen)( dmKernelBlock* );
 int (*dmkImageSpec)( dmKernelBlock*, dmDataType, long*, long );
 int (*dmkColumnOpenNo)( dmKernelBlock*, long, char*, dmDataType*, char*, char*, int* );
 int (*dmkColumnCreate)( dmKernelBlock*, char*, dmDataType, int, char*, char* );
 int (*dmkBlockInit)( dmKernelBlock* );
 int (*dmkNextKey)( dmKernelBlock*, int, char*, dmDataType*, char*, char*, void** );
 int (*dmkSpecialKey_d)( dmKernelBlock*, char*, int, double* );
 long (*dmkColumnRead_d)( dmKernelBlock*, int, long, long, double*, long ); 
 long (*dmkColumnRead_r)( dmKernelBlock*, int, long, long, real*, long ); 
 long (*dmkColumnWrite_d)( dmKernelBlock*, int, long, long, double*, long );
 int (*dmkKeyWrite_d)( dmKernelBlock*, char*, double, char*, char* );
 int (*dmkPutRow)( dmKernelBlock*, long );
 int (*dmkBlockPrintKernel)( dmKernelBlock* );
 int (*dmkSetKernelOption)( char* );
} dmKernel;


void dmkErrorSet( int status );
dmKernel* dmKernelGet( char* name );

#endif
