#include <stdlib.h>
#include <stdio.h>
#include "jcmlib.h"
#include "dmkfits.h"


/*
 *
 * Print data on a FITS block, listing all keys and columns
 */

int FITS_BlockPrintKernel( dmKernelBlock* kblock )
{
 card buf;
 FITS_Block* block = (FITS_Block*)kblock;
 int k;

 sprintf( buf, "HDU Number %d Type %d", block->hdu, block->type );
 msg( buf );
 sprintf( buf, "Columns: %d Rows: %d", (int)block->tfields, (int)block->nrows );
 msg( buf );
 sprintf( buf, "Header: %d keys", (int)block->nkeys );
 msg( buf );
 for ( k = 0; k < block->nkeys; k++ ) {
  FITS_PrintKey( &block->keys[k] );
 }
 msg( "Columns:" );
 for ( k = 0; k < block->tfields; k++ ) {
  sprintf( buf, "%-20s %-20s %-20s", block->ttype[k], block->tunit[k], block->tform[k] );
  msg( buf );
 }
 msg( " " );
 return( 0 );
}

/* 
 * PrintKey
 *
 * Prints the value of a key, handling the different data types.
 */

void FITS_PrintKey( FITS_Key* key )
{
 card value;
 double* ptr;
 t_buf buf;
 switch( key->type ) {
  case FITS_KEY_Q:
   cs_copy( "  ", value );
   value[0] = write_q( *(logical*)key->value );
   break;
  case FITS_KEY_D:
   ptr = key->value;
   write_d( *(double*)key->value, value, CSIZE );
   break;
  case FITS_KEY_I:
   write_i( *(int*)key->value, value, CSIZE );
   break;
  case FITS_KEY_C:
  case FITS_COMMENT:
   cs_copy_siz( *(char**)key->value, value, CSIZE );
   break;
  default:
   cs_copy( " ", value );
 } 
 sprintf( buf, "%-10s %-40s %s", key->name, value, key->comment );
 msg( buf );
}

