#include <stdlib.h>
#define DMSOURCE
#include "ascdm.h"
/*
 *--------------------------------------------------------------------------------
 * ASCDM incompatibility routines
 *--------------------------------------------------------------------------------
 */
/*
 * dmBlockCreateC
 *
 * Create generic block within dataset
 *
 */

/*
 * dmBlockCreate
 *
 * Block Create, ASC version
 */
dmBlock* dmBlockCreateC( dmDataset* ds, char* name, char* typename )
{
 dmBlockType type = dmASCBlockType( typename );
 return( dmBlockCreate( ds, name, type ) );
}

dmBlock* dmBlockCreate( dmDataset* ds, char* name, dmBlockType type )
{
 dmKernelBlock* kblock;
 dmBlock* block;
 dmsg1( 1, "Block create ", name );
 kblock = ds->kernel->dmkBlockCreate( ds->fptr, name, type ); 
 dmsg1( 1, "Block alloc ", name );
 block = dmiBlockAlloc( ds, kblock, name, type );
 dmsg1( 1, "Alloc OK ", name );
 dmiBlockCreate( block );
 return( block );
}

/*
 * dmDatasetTableCreate
 *
 */
dmBlock* dmDatasetTableCreate( char* name )
{
 t_filename dsname;
 card block_name;
 dmDataset* ds;
 dmBlock* block;

 dmiSyntaxParse( name, dsname, block_name );
 dmsg1( 1, "Dataset: ", dsname );
 dmsg1( 1, "Block:", block_name );
 ds = dmDatasetCreate( dsname );
 dmsg1( 1, "Dataset create OK", dsname );
 if ( ds ) {
  block = dmBlockCreateC( ds, block_name, "TABLE" );
  return( block );
 } else {
  return( NULL );
 }
}



/*
 * dmImageCreate
 *
 * Image Create, ASC version
 */
dmBlock* dmImageCreate( char* name, dmDataType type, long* axes, long naxes )
{

 t_filename dsname;
 card block_name;
 dmBlock* block;
 dmDataset* ds;
 dmiSyntaxParse( name, dsname, block_name );
 dmsg1( 1, "Dataset: ", dsname );
 dmsg1( 1, "Block:", block_name );
 ds = dmDatasetCreate( dsname );
 if ( ds ) {
  block = dmBlockCreateC( ds, block_name, "IMAGE" );
  dmiImageSpec( block, type, axes, naxes );
  return( block );
 } else {
  return( NULL );
 }
}


/*
 * dmImageCreate
 *
 * Create Image block and dataset
 *
 */
dmBlock* dmImageCreateC( char* name, char* typename, long* axes, long naxes )
{
 dmDataType type;
 type = dmASCDataType( typename );
 return( dmImageCreate( name, type, axes, naxes ) );
}


/*
 * dmBlockGetTypeName
 *
 * Get type name for block. This routine is not part of the
 * ASCDS interface.
 */
int dmBlockGetTypeName( dmBlock* block, char* name )
{
 if ( block ) { 
  dmASCBlockTypeName( block->type, name );
  return( 0 );
 } else {
  cs_copy( " ", name );
  return( WARN_NULL_BLOCK );
 }
}

/*
 * dmDatasetGetBlockTypeName
 *
 * Get type of numbered block in dataset.
 *
 */
void dmDatasetGetBlockTypeName( dmDataset* ds, int block_no, char* name )
{
 if ( ds ) {
  if ( !ds->scanned ) dmiDatasetScan( ds, block_no );
  dmBlockGetTypeName( ds->block[block_no-1], name );
 } else {
  cs_copy( " ", name );
 }
}

/*
 * dmGetNameC
 *
 * JCM version
 */
void dmGetNameC( dmDescriptor* dd, char* name )
{
 if ( dd ) {
  cs_copy_siz( dd->name, name, WSIZE );
 } else {
  cs_copy( " ", name );
 }
}


/*
 * dmGetUnitC
 *
 */
void dmGetUnitC( dmDescriptor* dd, char* unit )
{
 if ( dd ) {
  cs_copy_siz( dd->unit, unit, WSIZE );
 } else {
  cs_copy( " ", unit );
 }

}


void dmGetDescC( dmDescriptor* dd, char* desc )
{
 if ( dd ) {
  cs_copy_siz( dd->desc, desc, CSIZE );
 } else {
  cs_copy( " ", desc );
 }

}

void dmGetDataTypeNameC( dmDescriptor* dd, char* typename )
{
 if ( dd ) {
  dmASCDataTypeName( dd->data_type, typename );
 } else {
  cs_copy( " ", typename );
 }
}
