/*
 * Memory management store 
 */
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "jcmlib.h"
#include "store.h"
/* 
 * The memory buffer
 */


Store* store_alloc( void )
{
 card buf;
 Store* store;
 store = (Store*) malloc( sizeof( Store ) );
 sprintf( buf, "*** ALLOC %12ld STORE ***", (long)store ); dmsg(1, buf );

 store->buf[0] = (char*)malloc( 20 * sizeof( char ) );
 sprintf( buf, "*** ALLOC %12ld Store ***", (long)store->buf[0] ); dmsg(1, buf );
 ar_zero_i( store->index, 6 );
 store->ns = 1;
 return( store );
}

void store_free( Store* store )
{
 card buf;
 int i;
 for ( i = 0; i < store->ns; i++ ) {
  sprintf( buf, "*** FREE  %12ld Store ***", (long)store->buf[i] ); dmsg( 1, buf );
  if ( store->type[i] == dtChar ) free( store->base[i] );
  free( store->buf[i] );
 }
 sprintf( buf, "*** FREE  %12ld STORE ***", (long)store ); dmsg( 1, buf );
 free( store );
}

double* store_alloc_d( Store* store, int n )
{
 int ns;
 card dbuf;
 store->ns++;
 ns = store->ns-1;
 store->buf[ns] = (double*)malloc( n * sizeof( double ) );
 sprintf( dbuf, "*** ALLOC %12ld Store D %d ***", (long)store->buf[ns], n );
 dmsg( 1, dbuf );
 store->nmax[ns]  = n;
 store->n[ns] = 0;
 store->base[ns] = 0;
 store->type[ns] = dtDouble;
 store->index[(int)dtDouble] = ns;
 return( store->buf[ns] );
}

double* store_d( Store* store, double* x, int n )
{
 double* ptr;
 double* base;
 int nmax;
 int ns;
 ns = store->index[(int)dtDouble];
 if ( ns == 0 || store->n[ns] + n > store->nmax[ns] ) {
  nmax = max_i( ( store->n[ns] + n ) * 2, 500 );
  store_alloc_d( store, nmax );
  ns = store->index[(int)dtDouble];
 }
 base = store->buf[ns];
 ptr = &(base[store->n[ns]]);
 while( n-- ) {
  base[(store->n[ns])++] = *x;
  x++;
 }
 return( ptr );
}



char** store_alloc_c( Store* store, int n, int bytes )
{
 char** ptr;
 char* cptr;
 int ns;
 card dbuf;
 store->ns++;
 ns = store->ns-1;
 ptr  = (char**)malloc( n * sizeof( char* ) );
 cptr = (char*)malloc( bytes * sizeof( char ) );
 store->buf[ns] = ptr;
 store->nmax[ns] =n;
 sprintf( dbuf, "*** ALLOC %12ld Store C %d %d ***", (long)store->buf[ns], n, bytes );
 dmsg( 1, dbuf );
 store->bytes[ns]  =bytes;
 ptr[0] = cptr;
 store->top[ns]   = cptr;
 store->base[ns]  = cptr;
 store->n[ns] = 0;
 store->type[ns] = dtChar;
 store->index[(int)dtChar] = ns;
 return( store->buf[ns] );
}

char** store_c( Store* store, char* x )
{
 char** pptr;
 char* ptr;
 int nmax;
 int ncmax;
 char** base;
 int ns;
 ns = store->index[(int)dtChar];
 if ( ns == 0 || store->n[ns] + 1 > store->nmax[ns] ) {
  nmax = max_i( ( store->n[ns] + 1 ) * 2, 500 );
  ncmax = max_i( 2*store->bytes[ns], 50000 );
  store_alloc_c( store, nmax, ncmax );
  ns = store->index[(int)dtChar];
 }
 base = store->buf[ns];
 pptr = &(base[(store->n[ns])]);
 ptr  = store->top[ns];
 *pptr = ptr;
 store->n[ns]++;
 store->top[ns] += strlen( x )+1;
 if ( store->top[ns] - store->base[ns] <= store->bytes[ns] ) {
  cs_copy( x, ptr );
 } else {
  tmsg( "String buffer store overflow" );
 }
 return( pptr );
}


void store_list_d( Store* store )
{
 t_word value;
 int i;
 card buf;
 int ns;
 double* base;
 
 for ( ns = 0; ns < store->ns; ns++ ) {
  if ( store->type[ns] == dtDouble ) {
   msg( " " );
   dashline( 20 );
   sprintf( buf, "Buffer D: %ld of %ld", store->n[ns], store->nmax[ns] );
   msg( buf );
   dashline( 20 );
   base = store->buf[ns];
   for ( i = 0; i < store->n[ns]; i++ ) {
    write_d( base[i], value, WSIZE );
    sprintf( buf, "D%5.5d %s", i+1, value );
    msg( buf );
   }
   msg( " " );
  }
 }
}

void store_list_i( Store* store )
{
 t_word value;
 int i;
 card buf;
 int ns;
 int* base;
 
 for ( ns = 0; ns < store->ns; ns++ ) {
  if ( store->type[ns] == dtInt ) {
   msg( " " );
   dashline( 20 );
  sprintf( buf, "Buffer I: %ld of %ld", store->n[ns], store->nmax[ns] );
  msg( buf );
  dashline( 20 );
  base = store->buf[ns];
  for ( i = 0; i < store->n[ns]; i++ ) {
   write_i( base[i], value, WSIZE );
   sprintf( buf, "I%5.5d %s", i+1, value );
   msg( buf );
  }
  msg( " " );
 }
 }
}


void store_list_q( Store* store )
{
 int i;
 card buf;
 int ns;
 logical* base; 
 for ( ns = 0; ns < store->ns; ns++ ) {
  if ( store->type[ns] == dtLogical ) {
  msg( " " );
  dashline( 20 );
  sprintf( buf, "Buffer Q: %ld of %ld", store->n[ns], store->nmax[ns] );
  msg( buf );
  dashline( 20 );
  base = store->buf[ns];
  for ( i = 0; i < store->n[ns]; i++ ) {
   
   sprintf( buf, "Q%5.5d %c", i+1, write_q( base[i] ) );
   msg( buf );
  }
  msg( " " );
 }
 }
}

void store_list_r( Store* store )
{
 t_word value;
 int i;
 card buf;
 int ns;
 real* base; 
 for ( ns = 0; ns < store->ns; ns++ ) {
  if ( store->type[ns] == dtReal ) {
  msg( " " );
  dashline( 20 );
  sprintf( buf, "Buffer R: %ld of %ld", store->n[ns], store->nmax[ns] );
  msg( buf );
  dashline( 20 );
  base = store->buf[ns];
  for ( i = 0; i < store->n[ns]; i++ ) {
   write_r( base[i], value, WSIZE );
   sprintf( buf, "R%5.5d %s", i+1, value );
   msg( buf );
  }
  msg( " " );
 }
 }
}




void store_list_c( Store* store )
{
 int i;
 card buf;
 int ns;
 char** base;
 
 for ( ns = 0; ns < store->ns; ns++ ) {
  if ( store->type[ns] == dtChar ) {
   base = store->buf[ns];
  msg( " " );
  dashline( 20 );
  sprintf( buf, "Buffer C: %ld of %ld", store->n[ns], store->nmax[ns] );
  msg( buf );
  sprintf( buf, "Buffer C: %d of %ld bytes", (store->top[ns]-store->base[ns]), store->bytes[ns] );
  msg( buf );
  dashline( 20 );
  for ( i = 0; i < store->n[ns]; i++ ) {
   sprintf( buf, "C%5.5d %s", i+1, base[i] );
   msg( buf );
  }
  msg( " " );
 }
 }
}


void store_list( Store* store )
{
 store_list_c( store );
 store_list_d( store );
 store_list_i( store );
 store_list_r( store );
 store_list_q( store );
}


long* store_alloc_i( Store* store, int n )
{
 int ns;
 card dbuf;
 store->ns++;
 ns = store->ns-1;
 store->buf[ns] = (long*)malloc( n * sizeof( long ) );
 sprintf( dbuf, "*** ALLOC %12ld Store I %d ***", (long)store->buf[ns], n );
 dmsg( 1, dbuf );
 store->nmax[ns]  = n;
 store->n[ns] = 0;
 store->base[ns] = 0;
 store->type[ns] = dtInt;
 store->index[(int)dtInt] = ns;
 return( store->buf[ns] );
}

long* store_i( Store* store, long* x, int n )
{
 long* ptr;
 long* base;
 int nmax;
 int ns;
 ns = store->index[(int)dtInt];
 if ( store->n[ns] + n > store->nmax[ns] ) {
  nmax = max_i( ( store->n[ns] + n ) * 2, 500 );
  store_alloc_i( store, nmax );
  ns = store->index[(int)dtInt];
 }
 base = store->buf[ns];
 ptr = &(base[store->n[ns]]);
 while( n-- ) {
  base[(store->n[ns])++] = *x;
  x++;
 }
 return( ptr );
}


real* store_alloc_r( Store* store, int n )
{
 int ns;
 card dbuf;
 store->ns++;
 ns = store->ns-1;
 store->buf[ns] = (real*)malloc( n * sizeof( real ) );
 sprintf( dbuf, "*** ALLOC %12ld Store R %d ***", (long)store->buf[ns], n );
 dmsg( 1, dbuf );
 store->nmax[ns]  = n;
 store->n[ns] = 0;
 store->base[ns] = 0;
 store->type[ns] = dtReal;
 store->index[(int)dtReal] = ns;
 return( store->buf[ns] );
}

real* store_r( Store* store, real* x, int n )
{
 real* ptr;
 real* base;
 int nmax;
 int ns;
 ns = store->index[(int)dtReal];
 if ( ns == 0 || store->n[ns] + n > store->nmax[ns] ) {
  nmax = max_i( ( store->n[ns] + n ) * 2, 500 );
  store_alloc_r( store, nmax );
  ns = store->index[(int)dtReal];
 }
 base = store->buf[ns];
 ptr = &(base[store->n[ns]]);
 while( n-- ) {
  base[(store->n[ns])++] = *x;
  x++;
 }
 return( ptr );
}


logical* store_alloc_q( Store* store, int n )
{
 int ns;
 card dbuf;
 store->ns++;
 ns = store->ns-1;
 store->buf[ns] = (logical*)malloc( n * sizeof( logical ) );
 sprintf( dbuf, "*** ALLOC %12ld Store Q %d ***", (long)store->buf[ns], n );
 dmsg( 1, dbuf );
 store->nmax[ns]  = n;
 store->n[ns] = 0;
 store->base[ns] = 0;
 store->type[ns] = dtLogical;
 store->index[(int)dtLogical] = ns;
 return( store->buf[ns] );
}

logical* store_q( Store* store, logical* x, int n )
{
 logical* ptr;
 logical* base;
 int nmax;
 int ns;
 ns = store->index[(int)dtLogical];
 if ( ns == 0 || store->n[ns] + n > store->nmax[ns] ) {
  nmax = max_i( ( store->n[ns] + n ) * 2, 500 );
  store_alloc_q( store, nmax );
  ns = store->index[(int)dtLogical];
 }
 base = store->buf[ns];
 ptr = &(base[store->n[ns]]);
 while( n-- ) {
  base[(store->n[ns])++] = *x;
  x++;
 }
 return( ptr );
}

