#include "jcmlib.h"

/*----------------------------------------------------------------------*/
/*
 * ar_zero_i
 *
 * Zero each element of an integer array.
 *
 */

void ar_zero_i( integer* a, const integer n )
{
 integer siz = n;
 while( siz-- ) *a++ = 0;
}

/*----------------------------------------------------------------------*/
/*
 * ar_zero_d
 *
 * Zero each element of a double array.
 *
 */

void ar_zero_d( double* a, const integer n )
{
 integer siz = n;
 while( siz-- ) *a++ = 0;
}

/*----------------------------------------------------------------------*/
/*
 * ar_copy_i
 *
 * Copy one integer array to another
 *
 */

void ar_copy_i( const integer* a, integer* b, const integer n )
{
 integer siz = n;
 while( siz-- ) *b++ = *a++;
}

/*----------------------------------------------------------------------*/
/*
 * ar_copy_d
 *
 * Copy one integer array to another
 *
 */

void ar_copy_d( const double* a, double* b, const integer n )
{
 integer siz = n;
 while( siz-- ) *b++ = *a++;
}

/*----------------------------------------------------------------------*/
/*
 * ar_dot
 *
 *
 */

integer ar_dot_i( const integer* x, const integer* y, const integer n )
{
 integer sum;
 integer i;
 
 sum = 0;
 for ( i = 0; i < n; i++ ) {
  sum += x[i] * y[i];
 }
 return( sum );
}


short ar_dot_s( const short* x, const short* y, const integer n )
{
 short sum;
 integer i;
 
 sum = 0;
 for ( i = 0; i < n; i++ ) {
  sum += x[i] * y[i];
 }
 return( sum );
}


double ar_dot_d( const double* x, const double* y, const integer n )
{
 double sum;
 integer i;

 sum = 0;
 for ( i = 0; i < n; i++ ) {
  sum += x[i] * y[i];
 }
 return( sum );
}

real ar_dot_r( const real* x, const real* y, const integer n )
{
 real sum;
 integer i;

 sum = 0;
 for ( i = 0; i < n; i++ ) {
  sum += x[i] * y[i];
 }
 return( sum );
}


/*----------------------------------------------------------------------*/
/*
 *+ar_bounds
 *
 */

void ar_bounds_i( const integer* x, const integer n, integer* xmin, integer* xmax )
{
 integer i;
 integer val;
 *xmin = x[0];
 *xmax = *xmin;
 for ( i = 1; i < n; i++ ) {
  val = x[i];
  if ( val < *xmin ) *xmin = val;
  if ( val > *xmax ) *xmax = val;
 }
}

void ar_bounds_d( const double* x, const integer n, double* xmin, double* xmax )
{
 integer i;
 double val;
 *xmin = x[0];
 *xmax = *xmin;
 for ( i = 1; i < n; i++ ) {
  val = x[i];
  if ( val < *xmin ) *xmin = val;
  if ( val > *xmax ) *xmax = val;
 }
}

void ar_bounds_s( const short* x, const integer n, short* xmin, short* xmax )
{
 integer i;
 short val;
 *xmin = x[0];
 *xmax = *xmin;
 for ( i = 1; i < n; i++ ) {
  val = x[i];
  if ( val < *xmin ) *xmin = val;
  if ( val > *xmax ) *xmax = val;
 }
}

void ar_bounds_r( const real* x, const integer n, real* xmin, real* xmax )
{
 integer i;
 real val;
 *xmin = x[0];
 *xmax = *xmin;
 for ( i = 1; i < n; i++ ) {
  val = x[i];
  if ( val < *xmin ) *xmin = val;
  if ( val > *xmax ) *xmax = val;
 }
}


/*----------------------------------------------------------------------*/
/*
 *+ar_stats
 *
 */
void ar_stats_d( const double* x, const integer n , double* mean, double* var )
{
 integer i;
 double off;
 *mean = x[0];
 *var  = 0;
 if ( n > 1 ) {
  for ( i = 1; i < n; i++ ) { *mean += x[i]; }
  *mean /= n;
  for ( i = 0; i < n; i++ ) { off = ( x[i] - *mean ) ; *var += off * off; }
  *var /= n;
 }
}


void ar_stats_i( const integer* x, const integer n , double* mean, double* var )
{
 integer i;
 double off;
 *mean = (double) x[0];
 *var  = 0;
 if ( n > 1 ) {
  for ( i = 1; i < n; i++ ) { *mean += x[i]; }
  *mean /= n;
  for ( i = 0; i < n; i++ ) { off = ( x[i] - *mean ) ; *var += off * off; }
  *var /= n;
 }
}



void ar_stats_r( const real* x, const integer n, real* mean, real* var )
{
 integer i;
 real off;
 *mean = x[0];
 *var  = 0;
 if ( n > 1 ) {
  for ( i = 1; i < n; i++ ) { *mean += x[i]; }
  *mean /= n;
  for ( i = 0; i < n; i++ ) { off = ( x[i] - *mean ) ; *var += off * off; }
  *var /= n;
 }
}


void ar_stats_s( const short* x, const integer n, real* mean, real* var )
{
 integer i;
 real off;
 *mean = (real) x[0];
 *var  = 0;
 if ( n > 1 ) {
  for ( i = 1; i < n; i++ ) { *mean += x[i]; }
  *mean /= n;
  for ( i = 0; i < n; i++ ) { off = ( x[i] - *mean ) ; *var += off * off; }
  *var /= n;
 }
}
/*----------------------------------------------------------------------*/
/*
 * ar_zero_s
 *
 * Zero each element of an integer array.
 *
 */

void ar_zero_s( short* a, const integer n )
{
 integer siz = n;
 while( siz-- ) *a++ = 0;
}

/*----------------------------------------------------------------------*/
/*
 * ar_zero_r
 *
 * Zero each element of a double array.
 *
 */

void ar_zero_r( real* a, const integer n )
{
 integer siz = n;
 while( siz-- ) *a++ = 0;
}

/*----------------------------------------------------------------------*/
/*
 * ar_copy_s
 *
 * Copy one integer array to another
 *
 */

void ar_copy_s( const short* a, short* b, const integer n )
{ 
 integer siz = n;
 while( siz-- ) *b++ = *a++;
}

/*----------------------------------------------------------------------*/
/*
 * ar_copy_r
 *
 * Copy one real array to another
 *
 */

void ar_copy_r( const real* a, real* b, const integer siz )
{ 
 integer n = siz;
 while( n-- ) *b++ = *a++;
}


/*----------------------------------------------------------------------*/
/*
 * ar_copy_q
 *
 * Copy one logical array to another
 *
 */

void ar_copy_q( const logical* a, logical* b, const integer siz )
{
 integer n = siz;
 while( n-- ) *b++ = *a++;
}

void ar_fill_s( short* x, const integer m, const short x0 )
{
 integer n = m;
 while( n-- ) { *x++ = x0; }
}

void ar_cmult_s( short* x, const integer m, const short x0 )
{
 integer n = m;
 while( n-- ) { *x++ *= x0; }
}

void ar_cadd_s( short* x, const integer m, const short x0 )
{
 integer n = m;
 while( n-- ) { *x++ += x0; }
}

void ar_add_s( const short* x, short* y, const integer m )
{
 integer n = m;
 while( n-- ) { *y++ += *x++; }
}

void ar_sub_s( const short* x, short* y, const integer m )
{
 integer n = m;

 while( n-- ) { *y++ -= *x++; }
}

void ar_wadd_s( const short* x1, const short* x2,
                const short w1, const short w2,
                short* y,
                const integer m )
{
 integer n = m;
 while( n-- ) { *y++ = w1 * (*x1++) + w2 * (*x2++); }
}

void ar_bin_s( short* x, const integer n, const short x0, const short dx )
{
 integer i;
 for ( i = 0; i < n; i++ ) {  x[i] = x0 + i * dx; }
}


void ar_fill_i( integer* x, const integer m, const integer x0 )
{
 integer n = m;
 while( n-- ) { *x++ = x0; }
}

void ar_cmult_i( integer* x, const integer m, const integer x0 )
{
 integer n = m;
 while( n-- ) { *x++ *= x0; }
}

void ar_cadd_i( integer* x, const integer m, const integer x0 )
{
 integer n = m;
 while( n-- ) { *x++ += x0; }
}

void ar_add_i( const integer* x, integer* y, const integer m )
{
 integer n = m;
 while( n-- ) { *y++ += *x++; }
}

void ar_sub_i( const integer* x, integer* y, const integer m )
{
 integer n = m;
 while( n-- ) { *y++ -= *x++; }
}

void ar_wadd_i( const integer* x1, const integer* x2,
                const integer w1, const integer w2,
                integer* y, const integer m )
{
 integer n = m;
 while( n-- ) { *y++ = w1 * (*x1++) + w2 * (*x2++); }
}

void ar_bin_i( integer* x, const integer n, const integer x0, const integer dx )
{
 integer i;
 for ( i = 0; i < n; i++ ) {  x[i] = x0 + i * dx; }
}

void ar_fill_r( real* x, const integer m, const real x0 )
{
 integer n = m;
 while( n-- ) { *x++ = x0; }
}

void ar_cmult_r( real* x, const integer m, const real x0 )
{
 integer n = m;
 while( n-- ) { *x++ *= x0; }
}

void ar_cadd_r( real* x, const integer m, const real x0 )
{
 integer n = m;
 while( n-- ) { *x++ += x0; }
}

void ar_add_r( const real* x, real* y, const integer m )
{
 integer n = m;
 while( n-- ) { *y++ += *x++; }
}

void ar_sub_r( const real* x, real* y, const integer m )
{
 integer n = m;
 while( n-- ) { *y++ -= *x++; }
}

void ar_wadd_r( const real* x1, const real* x2,
                const real w1, const real w2,
                 real* y, const integer m )

{
 integer n = m;
 while( n-- ) { *y++ = w1 * (*x1++) + w2 * (*x2++); }
}

void ar_bin_r( real* x, const integer n, const real x0, const real dx )
{
 integer i;
 for ( i = 0; i < n; i++ ) {  x[i] = x0 + i * dx; }
}

void ar_fill_d( double* x, const integer m, const double x0 )
{
 integer n = m;
 while( n-- ) { *x++ = x0; }
}

void ar_cmult_d( double* x, const integer m, const double x0 )
{
 integer n = m;
 while( n-- ) { *x++ *= x0; }
}

void ar_cadd_d( double* x, const integer m, const double x0 )
{
 integer n = m;
 while( n-- ) { *x++ += x0; }
}

void ar_add_d( const double* x, double* y, const integer m )
{
 integer n = m;
 while( n-- ) { *y++ += *x++; }
}

void ar_sub_d( const double* x, double* y, const integer m )
{
 integer n = m;
 while( n-- ) { *y++ -= *x++; }
}

void ar_wadd_d( const double* x1, const double* x2,
                const double w1,  const double w2,
                double* y,
                const integer siz )
{
 integer n = siz;
 while( n-- ) { *y++ = w1 * (*x1++) + w2 * (*x2++); }
}

void ar_bin_d( double* x, const integer n, const double x0, const double dx )
{
 integer i;
 for ( i = 0; i < n; i++ ) {  x[i] = x0 + i * dx; }
}








