#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include "jcmlib.h"


/*
 * New simple alloc routines for vectors
 */

/*
 *--------------------------------------------------------------------------------
 */
integer** ar_alloc_i( const integer dim, const integer n )
{
 integer** p;
 integer* array;
 integer i;
 array = (integer *)malloc( n * dim * sizeof( integer ) );
 p = (integer** )malloc( n * sizeof( integer * ) );
 if ( !p ) err_set( JCM_ERR_MALLOC );
 for ( i = 0; i < n; i++ ) {
  p[i] = array + dim * i;
 }
 return( p );
}

/*
 *--------------------------------------------------------------------------------
 */
void ar_free_i( integer** p )
{
 free( p[0] );
 free( p );
}

/*
 *--------------------------------------------------------------------------------
 */

short** ar_alloc_s( const integer dim, const integer n )
{
 short** p;
 short* array;
 integer i;
 array = (short *)malloc( n * dim * sizeof( short ) );
 p = (short** )malloc( n * sizeof( short * ) );
 if ( !p ) err_set( JCM_ERR_MALLOC );
 for ( i = 0; i < n; i++ ) {
  p[i] = array + dim * i;
 }
 return( p );
}

/*
 *--------------------------------------------------------------------------------
 */
void ar_free_s( short** p )
{
 free( p[0] );
 free( p );
}

/*
 *--------------------------------------------------------------------------------
 */

double** ar_alloc_d( const integer dim, const integer n )
{
 double** p;
 double* array;
 integer i;
 array = (double *)malloc( n * dim * sizeof( double ) );
 p = (double** )malloc( n * sizeof( double * ) );
 if ( !p ) err_set( JCM_ERR_MALLOC );
 for ( i = 0; i < n; i++ ) {
  p[i] = array + dim * i;
 }
 return( p );
}
/*
 *--------------------------------------------------------------------------------
 */

void ar_free_d( double** p )
{
 free( p[0] );
 free( p );
}

/*
 *--------------------------------------------------------------------------------
 */

real** ar_alloc_r( const integer dim, const integer n )
{
 real** p;
 real* array;
 integer i;
 array = (real *)malloc( n * dim * sizeof( real ) );
 p = (real** )malloc( n * sizeof( real * ) );
 if ( !p ) err_set( JCM_ERR_MALLOC );
 for ( i = 0; i < n; i++ ) {
  p[i] = array + dim * i;
 }
 return( p );
}
/*
 *--------------------------------------------------------------------------------
 */

void ar_free_r( real** p )
{
 free( p[0] );
 free( p );
}

/*
 *--------------------------------------------------------------------------------
 */
/*
 * Inspired by Numerical Recipes 
 */

/*
 * Allocate an array a(start:end)
 */


real* alloc_array_r( 
    const integer start,
    const integer end
)
{
 real* ptr;		/* Pointer to vector start */

 ptr = ( real * )malloc( (size_t) ( ( end - start + 1 ) * sizeof( real ) ) );
 if ( !ptr ) err_set( JCM_ERR_MALLOC );
 return( ptr - start );
}
/*
 *--------------------------------------------------------------------------------
 */

integer* alloc_array_i( 
    const integer start,
    const integer end
)
{
 integer* ptr;		/* Pointer to vector start */

 ptr = ( integer * )malloc( (size_t) ( ( end - start + 1 ) * sizeof( integer ) ) );
 if ( !ptr ) err_set( JCM_ERR_MALLOC );
 return( ptr - start );
}

/*
 *--------------------------------------------------------------------------------
 */
short* alloc_array_s( 
    const integer start,
    const integer end
)
{
 short* ptr;		/* Pointer to vector start */

 ptr = ( short * )malloc( (size_t) ( ( end - start + 1 ) * sizeof( short ) ) );
 if ( !ptr ) err_set( JCM_ERR_MALLOC );
 return( ptr - start );
}

/*
 *--------------------------------------------------------------------------------
 */
double* alloc_array_d( 
    const integer start,
    const integer end
)
{
 double* ptr;		/* Pointer to vector start */

 ptr = ( double* )malloc( (size_t) ( ( end - start + 1 ) * sizeof( double ) ) );
 if ( !ptr ) err_set( JCM_ERR_MALLOC );
 return( ptr - start );
}

/*
 *--------------------------------------------------------------------------------
 */

char** alloc_array_c( const integer size, const integer n )
{
 char** word;
 char* buf;
 integer i;

 /* Reserve n+1 bytes for each string */
 buf  = (char *)calloc(  n * (size + 1),  sizeof( char ) );
 word = (char**) malloc( (size_t) ( n * sizeof( char* ) ) );
 if ( !word ) err_set( JCM_ERR_MALLOC );
 word[0] = buf;
 /* Now assign the pointers to each string */
 for ( i = 1; i < n; i++ ) {
  word[i] = word[i-1] + size + 1;
 }
 return( word );
}

/*
 *--------------------------------------------------------------------------------
 */
void free_array_c( char** word )
{
 free( word[0] );
 free( word    );
}

/* 
 * Following Press, we use a pointer to an array of pointers to matrix rows. 
 */
/*
 *--------------------------------------------------------------------------------
 */

real** alloc_matrix_r( 
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer nx = xmax - xmin + 1;
 integer ny = ymax - ymin + 1;
 real*  ptr;
 real** row_ptr;
 integer row;

 /* Allocate ny pointers to float */
 row_ptr = (real **) malloc( (size_t)( nx * sizeof( float* ) ) );
 if ( !row_ptr ) err_set( JCM_ERR_MALLOC );
 row_ptr -= xmin;
 /* Allocate nx * ny floats */
 ptr = (real *) malloc( (size_t) ( ny * nx * sizeof( real ) ) );
 ptr -= ymin;

 row_ptr[xmin] = ptr;
 for ( row = xmin + 1; row <= xmax; row++ ) {
  row_ptr[row] = row_ptr[row-1] + ny;
 }
 return( row_ptr );
}

/*
 *--------------------------------------------------------------------------------
 */

double** alloc_matrix_d( 
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer nx = xmax - xmin + 1;
 integer ny = ymax - ymin + 1;
 double*  ptr;
 double** row_ptr;
 integer row;

 /* Allocate ny pointers to float */
 row_ptr = (double **) malloc( (size_t)( nx * sizeof( double* ) ) );
 if ( !row_ptr ) err_set( JCM_ERR_MALLOC );
 row_ptr -= xmin;
 /* Allocate nx * ny floats */
 ptr = (double *) malloc( (size_t) ( ny * nx * sizeof( double ) ) );
 ptr -= ymin;

 row_ptr[xmin] = ptr;
 for ( row = xmin + 1; row <= xmax; row++ ) {
  row_ptr[row] = row_ptr[row-1] + ny;
 }
 return( row_ptr );
}

/*
 *--------------------------------------------------------------------------------
 */
integer** alloc_matrix_i( 
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer nx = xmax - xmin + 1;
 integer ny = ymax - ymin + 1;
 integer*  ptr;
 integer** row_ptr;
 integer row;

 /* Allocate ny pointers to float */
 row_ptr = (integer**) malloc( (size_t)( nx * sizeof( integer* ) ) );
 if ( !row_ptr ) err_set( JCM_ERR_MALLOC );
 row_ptr -= xmin;
 /* Allocate nx * ny floats */
 ptr = (integer *) malloc( (size_t) ( ny * nx * sizeof( integer ) ) );
 ptr -= ymin;

 row_ptr[xmin] = ptr;
 for ( row = xmin + 1; row <= xmax; row++ ) {
  row_ptr[row] = row_ptr[row-1] + ny;
 }
 return( row_ptr );
}

/*
 *--------------------------------------------------------------------------------
 */

short** alloc_matrix_s( 
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer nx = xmax - xmin + 1;
 integer ny = ymax - ymin + 1;
 short*  ptr;
 short** row_ptr;
 integer row;

 /* Allocate ny pointers to float */
 row_ptr = (short**) malloc( (size_t)( nx * sizeof( short* ) ) );
 if ( !row_ptr ) err_set( JCM_ERR_MALLOC );
 row_ptr -= xmin;
 /* Allocate nx * ny floats */
 ptr = (short *) malloc( (size_t) ( ny * nx * sizeof( short ) ) );
 ptr -= ymin;

 row_ptr[xmin] = ptr;
 for ( row = xmin + 1; row <= xmax; row++ ) {
  row_ptr[row] = row_ptr[row-1] + ny;
 }
 return( row_ptr );
}

/*
 *--------------------------------------------------------------------------------
 */

void free_array_r( 
 real* v
)
{ 
 free( (char*) v );
}

/*
 *--------------------------------------------------------------------------------
 */
void free_array_d( 
 double* v
)
{ 
 free( (char*) v );
}
/*
 *--------------------------------------------------------------------------------
 */
void free_array_i( 
 integer* v
)
{ 
 free( (char*) v );
}
/*
 *--------------------------------------------------------------------------------
 */
void free_array_s( 
 short* v
)
{ 
 free( (char*) v );
}
/*
 *--------------------------------------------------------------------------------
 */
void free_matrix_s(
 short** matrix,
 const integer xmin,
 const integer ymin
)
{
 free( (char*) (matrix[xmin]+ymin) );  /* Free columns*/
 free( (char*) (matrix+xmin) );		/* Free rows*/
}
/*
 *--------------------------------------------------------------------------------
 */
void free_matrix_i(
 integer** matrix,
 const integer xmin,
 const integer ymin
)
{
 free( (char*) (matrix[xmin]+ymin) );  /* Free columns*/
 free( (char*) (matrix+xmin) );		/* Free rows*/
}
/*
 *--------------------------------------------------------------------------------
 */
void free_matrix_r(
 real** matrix,
 const integer xmin,
 const integer ymin
)
{
 free( (char*) (matrix[xmin]+ymin) );  /* Free columns*/
 free( (char*) (matrix+xmin) );		/* Free rows*/
}
/*
 *--------------------------------------------------------------------------------
 */
void free_matrix_d(
 double** matrix,
 const integer xmin,
 const integer ymin
)
{
 free( (char*) (matrix[xmin]+ymin) );  /* Free columns*/
 free( (char*) (matrix+xmin) );		/* Free rows*/
}



