#include "jcmlib.h"

/* Bit functions; note bits are numbered 1 to n */

/*
 *  Give me a number with a single bit set in the specified position.
 */
unsigned short single_bit_s( const integer bit_no )
{
 unsigned short value = 1;
 return( value << bit_no );
}

unsigned long int single_bit_i( const integer bit_no )
{
 unsigned long int value = 1;
 return( value << bit_no );
}

/*
 * Set bit in value 
 */
void bit_set_i( integer* value, const integer bit_no )
{
 *value |= single_bit_i( bit_no );
}


void bit_set_s( short* value, const integer bit_no )
{
 *value |= single_bit_i( bit_no );
}

void bit_clr_i(  integer* value, const integer bit_no )
{
 *value &= ~single_bit_i( bit_no );
}

void bit_clr_s(  short* value, const integer bit_no )
{
 *value &= ~ single_bit_s( bit_no );
}


short bit_shft_s( short value, const integer bits_to_shift )
{
 if ( bits_to_shift >= 0 ) {
  return( value << bits_to_shift );
 } else { 
  return( value >> -bits_to_shift );
 }
}

integer bit_shft_i( integer value, const integer bits_to_shift )
{
 if ( bits_to_shift >= 0 ) {
  return( value << bits_to_shift );
 } else { 
  return( value >> -bits_to_shift );
 }
}

logical bit_test_i( integer value, const integer bit_no )
{
 integer tmp;
 tmp =  single_bit_i( bit_no ) & value ;
 return( tmp != 0 );
}

logical bit_test_s( short value, const integer bit_no )
{
 short tmp;
 logical result;
 tmp =  single_bit_s( bit_no ) & value ;
 result = ( tmp != 0 );
 return( result );
}

integer bit_unpack_i( const integer value, const integer start_bit, const integer n_bits )
{
 integer result;
 unsigned long int mask;

 mask = ~( ~0 << n_bits );  /* n ones at the right */
 result = mask & ( value >> start_bit );
 return( result );
}

integer bit_unpack_s( const short value, const integer start_bit, const integer n_bits )
{
 unsigned short result;
 unsigned short mask;
 unsigned long int iresult;

 mask = ~( ~0 << n_bits );  /* n ones at the right */
 result = mask & ( value >> start_bit );
 iresult = (unsigned long int) result;
 return( (integer)iresult );
}

integer bit_unpack_i_slow( const integer value, const integer start_bit, const integer n_bits )
{
 integer bit_no;
 integer result = 0;
 for ( bit_no = 0; bit_no < n_bits; bit_no++ ) {
  if ( bit_test_i( value, start_bit + bit_no ) ) {
   bit_set_i( &result, bit_no );
  }
 }
 return( result );
}

integer bit_unpack_s_slow( const short value, const integer start_bit, const integer n_bits )
{
 integer bit_no;
 short result = 0;
 for ( bit_no = 0; bit_no < n_bits; bit_no++ ) {
  if ( bit_test_s( value, start_bit + bit_no ) ) {
   bit_set_s( &result, bit_no );
  }
 }
 return( (integer)result );
}


