#include <stdio.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "bits.h"
#include "sys.h"

static void ieee_get( const char* buf, char* ptr, const integer n, const integer size );
static void ieee_put( const char* ptr, char* buf, const integer n, const integer size );


void ieee_put_r( const real* data, char* buf, const integer n )
{
 ieee_put( (char *)data, buf, n, (integer)sizeof( real ) );
}
void ieee_put_d( const double* data, char* buf, const integer n )
{
 ieee_put( (char *)data, buf, n, (integer)sizeof( double ) );
}
void ieee_put_s( const short* data, char* buf, const integer n )
{
 ieee_put( (char *)data, buf, n, (integer)sizeof( short ) );
}
void ieee_put_i( const integer* data, char* buf, const integer n )
{
 ieee_put( (char *)data, buf, n, (integer)sizeof( integer ) );
}


void ieee_get_r( const char* buf, real* data, const integer n )
{
 ieee_get( buf, (char *)data, n, sizeof( real ) );
}

void ieee_get_d( const char* buf, double* data, const integer n )
{
 ieee_get( buf, (char *)data, n, sizeof( double ) );
}

void ieee_get_s( const char* buf, short* data, const integer n )
{
 ieee_get( buf, (char *)data, n, sizeof( short ) );
}

void ieee_get_i( const char* buf, integer* data, const integer n )
{
 ieee_get( buf, (char *)data, n, sizeof( integer ) );
}

static void ieee_get( const char* buf, char* ptr, const integer nn, integer size )
{
 integer m;
 integer n = nn;
 integer nbytes;

 m = size;
 nbytes = size * n; 
#ifdef BYTE_SWAPPED
 while( n-- ) {
  while ( m-- ) {
   *ptr++ = *(buf + m );
  }
  m = size;
  buf += size;
 }
#else
 while ( nbytes-- ) {
   *ptr++ = *buf++;
 }
#endif
}

static void ieee_put( const char* ptr, char* buf, const integer nn, const integer size )
{
 integer nbytes;
 integer n = nn;
 integer m;
 nbytes = n * size;
 m = size;
#ifdef BYTE_SWAPPED
 while( n-- ) {
  while( m-- ) {
   *buf++ = *(ptr + m );
  }
  m = size;
  ptr += size;
 }
#else
 while( nbytes-- )
 {
  *buf++ = *ptr++;
 }
#endif
}

void sbf_put_r( real* data, integer n )
{
#ifdef BYTE_SWAPPED
 integer bytes[4];
 integer result_bytes[4];
 integer byte_no;
 while( n-- ) {
  bytes_split_r( *data, bytes );
  for ( byte_no = 0; byte_no < 4; byte_no++ ) {
   result_bytes[byte_no] = bytes[3-byte_no];
  }
  *data++ = bytes_combine_r( result_bytes );
 }
#endif
}

void sbf_put_i( integer* data, integer n )
{
#ifdef BYTE_SWAPPED
 integer bytes[4];
 integer result_bytes[4];
 integer byte_no;

 while( n-- ) {
  bytes_split_i( *data, bytes );
  for ( byte_no = 0; byte_no < 4; byte_no++ ) {
   result_bytes[byte_no] = bytes[3-byte_no];
  }
  *data++ = bytes_combine_i( result_bytes );
 }
#endif
}

void sbf_put_s( short* data, integer n )
{
#ifdef BYTE_SWAPPED
 integer bytes[2];
 integer result_bytes[2];
 integer byte_no;

 while( n-- ) {
  bytes_split_s( *data, bytes );
  for ( byte_no = 0; byte_no < 2; byte_no++ ) {
   result_bytes[byte_no] = bytes[1-byte_no];
  }
  *data++ = bytes_combine_s( result_bytes );
 }
#endif
}


void sbf_put_d( double* data, integer n )
{
#ifdef BYTE_SWAPPED

 integer bytes[8];
 integer result_bytes[8];
 integer* ptr;
 integer byte_cno;

 while( n-- ) {
  ptr = (integer *)data;
  bytes_split_i( *ptr, bytes );
  bytes_split_i( *(ptr+1), bytes+4 );
  for ( byte_cno = 0; byte_cno < 8; byte_cno++ ) {
    result_bytes[byte_cno] = bytes[7-byte_cno];
  }
  *ptr = bytes_combine_i( result_bytes );
  *(ptr+1) = bytes_combine_i( result_bytes+4 );
  data++;
 }
#endif
}


void bytes_split_i( const integer value, integer* bytes )
{
 integer byte_no;
 for ( byte_no = 1; byte_no <= 4; byte_no++ ) {
  bytes[byte_no-1] = byte_get_i( value, byte_no );
 }
}

void bytes_split_r( const real value, integer* bytes )
{
 integer byte_no;
 for ( byte_no = 1; byte_no <= 4; byte_no++ ) {
  bytes[byte_no-1] = byte_get_r( value, byte_no );
 }
}

void bytes_split_d( const double value, integer* bytes )
{
 integer byte_no;
 for ( byte_no = 1; byte_no <= 8; byte_no++ ) {
  bytes[byte_no-1] = byte_get_d( value, byte_no );
 }
}

void bytes_split_s( const short value, integer* bytes )
{
 integer byte_no;
 for ( byte_no = 1; byte_no <= 2; byte_no++ ) {
  bytes[byte_no-1] = byte_get_s( value, byte_no );
 }
}

void bytes_split_c( const char* value, integer* bytes, const integer n_bytes )
{
 integer byte_no;
 for ( byte_no = 1; byte_no <= n_bytes; byte_no++ ) {
  bytes[byte_no-1] = byte_get_c( value, byte_no );
 }
}


integer bytes_combine_i( const integer* bytes )
{
 integer byte_no;
 integer value = 0;
 for ( byte_no = 1; byte_no <= 4; byte_no++ ) {
  byte_set_i( &value, bytes[byte_no-1], byte_no );
 }
 return( value );
}

short bytes_combine_s( const integer* bytes )
{
 integer byte_no;
 short value = 0;
 for ( byte_no = 1; byte_no <= 2; byte_no++ ) {
  byte_set_s( &value, bytes[byte_no-1], byte_no );
 }
 return( value );
}


real bytes_combine_r( const integer* bytes )
{
 integer byte_no;
 real value = 0;
 for ( byte_no = 1; byte_no <= 4; byte_no++ ) {
  byte_set_r( &value, bytes[byte_no-1], byte_no );
 }
 return( value );
}


double bytes_combine_d( const integer* bytes )
{
 integer byte_no;
 double value = 0;
 for ( byte_no = 1; byte_no <= 8; byte_no++ ) {
  byte_set_d( &value, bytes[byte_no-1], byte_no );
 }
 return( value );
}


void bytes_combine_c( const integer* bytes, char* value, const integer n_bytes )
{
 integer byte_no;

 for ( byte_no = 1; byte_no <= n_bytes; byte_no++ ) {
  byte_set_c( value, bytes[byte_no-1], byte_no );
 }
}



void byte_set_s( short* number, const integer value, const integer byte_no )
{
 short buffer;
 buffer = bit_shft_s( (short)value, BYTE_BITS * ( byte_no - 1 ) );
 *number |= buffer;
}


void byte_set_i( integer* number, const integer value, const integer byte_no )
{
 integer buffer;
 buffer = bit_shft_i( value, BYTE_BITS * ( byte_no - 1 ) );
 *number |= buffer;
}

void byte_set_r( real* number, const integer value, const integer byte_no )
{
 integer* ptr;
 ptr = (integer *)number;
 byte_set_i( ptr, value, byte_no );
}


void byte_set_d( double* number, const integer value, const integer byte_no )
{
 integer* ptr;
 integer segment;
 integer ibyte;

 ptr = (integer *)number;
 sys_byte_d( byte_no, &segment, &ibyte );
 byte_set_i( ptr + segment - 1, value, ibyte );
}

integer byte_get_i( const integer value, const integer byte_no )
{
 integer start_bit;
 integer result;

 start_bit = ( byte_no - 1 ) * BYTE_BITS;
 result = bit_unpack_i( value, start_bit, BYTE_BITS );
 return( result ); 
}


integer byte_get_r( const real value, const integer byte_no )
{
 integer* buffer;
 integer result;
 buffer = (integer *)(&value);
 result =  byte_get_i( *buffer, byte_no );
 return( result );
}

integer byte_get_s( const short value, const integer byte_no )
{
 integer start_bit;
 integer result;
 start_bit = ( byte_no - 1 ) * BYTE_BITS;
 result = bit_unpack_s( value, start_bit, BYTE_BITS );
 return( result ); 
}

integer byte_get_d( const double value, const integer byte_no )
{
 integer* ptr;
 integer segment;
 integer ibyte;
 integer result;
 
 ptr = (integer *)(&value);
 sys_byte_d( byte_no, &segment, &ibyte );
 result = byte_get_i( *(ptr + segment - 1), ibyte );
 return( result );
}


integer byte_get_c( const char* value, const integer byte_no )
{
 unsigned char c;

 c = (unsigned char)(value[byte_no-1]);
 return( (integer)c );
}

void byte_set_c( char* data, const integer value, const integer byte_no )
{
 data[byte_no-1] = (unsigned char)value;
}


void sys_byte_d( const integer byte_no, integer* segment, integer* ibyte )
{
 if ( byte_no > 4 ) {
  *ibyte = byte_no - 4;
  *segment = OS_WORD2;
 } else {
  *ibyte = byte_no;
  *segment = OS_WORD1;
 }
}
