#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "jcmlib.h"
#include "jcmp.h"

static integer stdin_is_tty = -1;

/*
 *  These routines use the fcntl.h file access mechanism
 */

integer cio_open( char* filename, char* jmode, char* mode )
{
 integer fid;
 integer fd;
 integer imode;
  if ( cs_eq( mode, "cw" ) ) { 
   fd = creat( filename, 0644 );
  } else {
   imode = cs_eq(mode, "crw" ) ? O_RDWR : O_RDONLY;
   fd = open( filename, imode );
  }
  fid = fio_cfile_alloc( filename, jmode, fd );
  return( fid );
}

/*
 *  Close a CIO file. Use FIO close instead which calls this but cleans
 * up better.
 */
void cio_close( integer fid )
{
 integer fd;
 fd = fio_get_cfd( fid );
 (void)close( fd );
}


/*
 *  Routines to read and write bytes from a CIO file.
 */
logical cio_read( integer fid, char* buf, integer n, integer* m )
{
 integer fd;
 fd = fio_get_cfd( fid );
 *m= read( fd, buf, n );
 return( *m == n );
}

logical cio_write( integer fid, char* buf, integer n, integer* m )
{
 integer fd;
 fd = fio_get_cfd( fid );
 *m= write( fd, buf, n );
 return( *m == n );
}

/*
 *  Routines to open a CIO file.
 *  Support "*" as a pseudo-filename meaning the terminal.
 */

integer cio_open_r( char* filename )
{
 integer file_no;
 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "CIO R", 0 );
 }
 return(file_no); 
}

integer cio_open_w( char* filename )
{
 integer file_no;
 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "CIO W", 0 );
 }
 return(file_no); 
}

integer cio_open_rw( char* filename )
{
 integer file_no;
 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "CIO RW", 0 );
 }
 return(file_no); 
}

/*
 *  Wrap POSIX isatty
 */

logical cio_tty( integer fid )
{
 integer fd;
 if ( fid > 0 ) {
  fd = fio_get_cfd( fid );
  return( isatty( fd ) );
 } else {
  return( isatty( 0 ) );
 }
}

/* Read a single byte. Usually used in raw mode */

integer cio_read_byte( integer fid )
{
 logical status;
 char buf;
 integer fd;
 integer byte;

 fd = fio_get_cfd( fid );
 status = ( read( fd, &buf, 1 ) < 1 );
 if ( status ) {
  byte  = (integer) ( buf & 127 );
 } else {
  byte = -1;
 }
 return( byte );
}

/*
 *  Open a raw mode file. Used for graphics cursors.
 */

integer cio_open_raw( void )
{
 integer fid;
 logical start;
 if ( ( start =  stdin_is_tty < 0 ) ) {
  stdin_is_tty = cio_tty( 0 ) ? 1 : 0;
 }
 if ( stdin_is_tty > 0 ) {
  fid = cio_open_rw( "/dev/tty" );
  return( fid );
  if ( fid > 0 ) {
   if ( start ) cio_setup_raw( fid );
   cio_set_raw( fid );
  }
 } else {
  tmsg( "Failed to open raw mode: stdin is not tty" );
  return( -1 );
 }
}

/*
 * Read a byte from a raw file. Used to get a cursor value.
 */


integer cio_read_raw( integer fid )
{
 const integer CTRLC = 3;
 const integer err = -1;
 integer c;


 if ( fid == 0 ) {
  c = err;
 } else {
  c = cio_read_byte( fid );
  if ( c < 0 ) { 
   cio_close( fid ); 
  } else if ( c == CTRLC ) {
   tmsg( "CTRL/C seen but ignored" );
  }
 }
 return( c );
}


