#include <stdlib.h>
#include "jcmlib.h"




complex merge_z( const complex z1, const complex z2, const logical mask )
{
 return( mask? z1 : z2 );
}

dcomplex merge_dz( const dcomplex z1, const dcomplex z2, const logical mask )
{
 return( mask? z1: z2 );
}

void ar_merge_z( const complex* z1, const complex* z2, complex* z, const logical* mask, const integer size )
{
 integer i;
 for( i = 0; i < size; i++ ) {
  z[i] = mask[i] ? z1[i] : z2[i];
 }
}

void ar_merge_dz( const dcomplex* z1, const dcomplex* z2, dcomplex* z, const logical* mask, const integer size )
{
 integer i;
 for( i = 0; i < size; i++ ) {
  z[i] = mask[i] ? z1[i] : z2[i];
 }
}


void ar_copy_z( const complex* z1, complex* z2, const integer size )
{
 integer n = size;
 while( n-- ) { *z2++ = *z1++;  }
}

void ar_copy_dz( const dcomplex* z1, dcomplex* z2, const integer size )
{
 integer n = size;
 while( n-- ) { *z2++ = *z1++; }
}


integer ar_pack_z( const complex* z1, const logical* mask, complex* z2, const integer n )
{
 integer m;			/* (o) Packed array size */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )     z2[m++] = z1[i];
 }
 return( m );
}

integer ar_pack_dz( const dcomplex* z1, const logical* mask, dcomplex* z2, const integer n )
{
 integer m;			/* (o) Packed array size */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )     z2[m++] = z1[i];
 }
 return( m );
}


integer ar_unpack_z( const complex* z1, const logical* mask, complex* z2, 
 const complex z0, const integer n )
{
 integer m;			/* (o) Packed array size: number of nondefault elements */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )  {
   z2[i] = z1[m++]; 
  } else {
   z2[i] = z0;
  }
 }
 return( m );
}



integer ar_unpack_dz( const dcomplex* z1, const logical* mask, dcomplex* z2, 
 const dcomplex z0, const integer n )
{
 integer m;			/* (o) Packed array size: number of nondefault elements */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )  {
   z2[i] = z1[m++]; 
  } else {
   z2[i] = z0;
  }
 }
 return( m );
}


void ar_cshift_z( 
 const complex* x,
 complex* y,
 const integer n,
 const integer shift
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = ( i + shift ) % n;
  y[i] = x[j];
 }
}


void ar_eoshift_z( 
 const complex* x,
 complex* y,
 const integer n,
 const integer shift,
 const complex fill

)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = i + shift;
  if ( j >= 0 && j < n ) {
   y[i] = x[j];
  } else {
   y[i] = fill;
  }
 }
}



void ar_transpose_z( 
  const complex** x,
  complex** y,
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer row;
 integer col;
 for ( row = xmin; row <= xmax; row++ ) {
  for ( col = ymin; col <= ymax; col++ ) {
   y[col][row] = x[row][col];
  }
 }
}


void ar_cshift_dz( 
 const dcomplex* x,
 dcomplex* y,
 const integer n,
 const integer shift
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = ( i + shift ) % n;
  y[i] = x[j];
 }
}


void ar_eoshift_dz( 
 const dcomplex* x,
 dcomplex* y,
 const integer n,
 const integer shift,
 const dcomplex fill

)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = i + shift;
  if ( j >= 0 && j < n ) {
   y[i] = x[j];
  } else {
   y[i] = fill;
  }
 }
}



void ar_transpose_dz( 
  const dcomplex** x,
  dcomplex** y,
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer row;
 integer col;
 for ( row = xmin; row <= xmax; row++ ) {
  for ( col = ymin; col <= ymax; col++ ) {
   y[col][row] = x[row][col];
  }
 }
}



complex ar_dot_z( const complex* x, const complex* y, const integer size )
{
 integer i;
 complex  sum = { 0.0, 0.0 };
 for ( i = 0; i < size; i++ ) {
  zinc( zprod( x[i], y[i] ), &sum );
 }
 return( sum );
}

complex zconj( const complex z1 )
{
 complex z;
 z.x = z1.x;
 z.y = -z1.y;
 return( z );
}
dcomplex dzconj( const dcomplex z1 )
{
 dcomplex z;
 z.x = z1.x;
 z.y = -z1.y;
 return( z );
}


complex zmult( const complex z1, const complex z2 )
{
 complex z;
 z.x = z1.x * z2.x - z1.y * z2.y;
 z.y = z1.x * z2.y + z2.x * z1.y;
 return( z );
}

complex zprod( const complex z1, const complex z2 )
{
 complex z;
 z.x = z1.x * z2.x + z1.y * z2.y;
 z.y =  z1.x * z2.y - z2.x * z1.y;
 return( z );
}

dcomplex dzprod( const dcomplex z1, const dcomplex z2 )
{
 dcomplex z;
 z.x = z1.x * z2.x + z1.y * z2.y;
 z.y =  z1.x * z2.y - z2.x * z1.y;
 return( z );
}

complex zadd( const complex z1, const complex z2 )
{
 complex z;
 z.x = z1.x + z2.x;
 z.y = z1.y + z2.y;
 return( z );
}

void zinc( const complex z1, complex* z2 )
{
 z2->x += z1.x;
 z2->y += z1.y;
}


real rreal( const complex z )
{
 return( z.x );
}

real rimag( const complex z )
{
 return( z.y );
}

double dreal( const dcomplex z )
{
 return( z.x );
}

double dimag( const dcomplex z )
{
 return( z.y );
}


dcomplex ar_dot_dz( const dcomplex* x, const dcomplex* y, const integer size )
{
 integer i;
 dcomplex sum = { 0.0, 0.0 };
 for ( i = 0; i < size; i++ ) {
  
  dzinc( dzprod( x[i], y[i] ), &sum );
 }
 return( sum );
}

complex zset( const real x, const real y )
{
 complex z;
 z.x = x;
 z.y = y;
 return( z );
}
dcomplex dzset( const double x, const double y )
{
 dcomplex z;
 z.x = x;
 z.y = y;
 return( z );
}

dcomplex dzmult( const dcomplex z1, const dcomplex z2 )
{
 dcomplex z;
 z.x = z1.x * z2.x - z1.y * z2.y;
 z.y = z1.x * z2.y + z2.x * z1.y;
 return( z );
}

dcomplex dzadd( const dcomplex z1, const dcomplex z2 )
{
 dcomplex z;
 z.x = z1.x + z2.x;
 z.y = z1.y + z2.y;
 return( z );
}

void dzinc( const dcomplex z1, dcomplex* z2 )
{
 z2->x += z1.x;
 z2->y += z1.y;
}



void ar_add_z( const complex* z1, complex* z2, const integer n )
{
 integer m = n;
 while ( m-- ) {
  zinc( *z1, z2 );
  z1++; z2++;
 }
}

void ar_sub_z( const complex* z1, complex* z2, const integer m )
{
 integer n = m;
 while ( n-- ) {
  z2->x -= z1->x;
  z2->y -= z1->y;
  z1++; z2++;
 }
}


void ar_zero_z( complex* z, const integer m )
{
 integer n = m;
 while( n-- ) {
  z->x = 0.0;
  z->y = 0.0;
  z++;
 }
}

void ar_fill_z( complex* z, const integer m, const complex z0 )
{
 integer n = m;
 while( n-- ) {
  *z++ = z0;
 }
}



void ar_add_dz( const dcomplex* z1, dcomplex* z2, const integer m )
{
 integer n = m;
 while ( n-- ) {
  dzinc( *z1, z2 );
  z1++; z2++;
 }
}

void ar_sub_dz( const dcomplex* z1, dcomplex* z2, const integer m )
{
 integer n = m;
 while ( n-- ) {
  z2->x -= z1->x;
  z2->y -= z1->y;
  z1++; z2++;
 }
}


void ar_zero_dz( dcomplex* z, const integer m )
{
 integer n = m;
 while( n-- ) {
  z->x = 0.0;
  z->y = 0.0;
  z++;
 }
}

void ar_fill_dz( dcomplex* z, const integer m, const dcomplex z0 )
{
 integer n = m;
 while( n-- ) {
  *z++ = z0;
 }
}

void ar_bin_z( complex* z, const integer n, const complex z0, const complex dz )
{
 integer i;
 for ( i = 0; i < n; i++ ) {  
  z[i].x = z0.x + i * dz.x;
  z[i].y = z0.y + i * dz.y;
 }
 
}

void ar_cmult_z( complex* z, const integer m, const complex z0 ) 
{
 integer n = m;
 while( n-- ) {
  *z++ = zmult( *z, z0 );  
 }
}

void ar_cadd_z( complex* z, const integer m, const complex z0 )
{
 integer n = m;
 while( n-- ) {
  zinc( z0, z ); 
  z++; 
 }
}

void ar_wadd_z( const complex* z1, const complex* z2, const complex w1, const complex w2,
  complex* z, const integer m )
{
 integer n = m;
 while( n-- ) { 
  *z++ = zadd( zmult( *z1++, w1 ), zmult( *z2++, w2 ) );
 }
}


void ar_bin_dz( dcomplex* z, const integer n, const dcomplex z0, const dcomplex dz )
{
 integer i;
 for ( i = 0; i < n; i++ ) {  
  z[i].x = z0.x + i * dz.x;
  z[i].y = z0.y + i * dz.y;
 }
 
}

void ar_cmult_dz( dcomplex* z, const integer m, const dcomplex z0 ) 
{
 integer n = m;
 while( n-- ) {
  *z++ = dzmult( *z, z0 );  
 }
}

void ar_cadd_dz( dcomplex* z, const integer n, const dcomplex z0 )
{
 integer m = n;
 while( m-- ) {
  dzinc( z0, z ); 
  z++; 
 }
}

void ar_wadd_dz( const dcomplex* z1, const dcomplex* z2, const dcomplex w1, const dcomplex w2,
  dcomplex* z, const integer m )
{
 integer n = m;
 while( n-- ) { 
  *z++ = dzadd( dzmult( *z1++, w1 ), dzmult( *z2++, w2 ) );
 }
}

complex ar_product_z( 
 const complex* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;
 complex product = { 1.0, 0.0};
 while( n-- ) {
  product = zmult( product, *a++ );
 }
 return( product );
}


dcomplex ar_product_dz( 
 const dcomplex* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;
 dcomplex product = { 1.0, 0.0};
 while( n-- ) {
  product = dzmult( product, *a++ );
 }
 return( product );
}


complex ar_sum_z( 
 const complex* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;
 complex sum = { 0.0, 0.0};
 while( n-- ) zinc( *a++, &sum );
 return( sum );
}

dcomplex ar_sum_dz( 
 const dcomplex* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;
 dcomplex sum = { 0.0, 0.0};
 while( n-- ) dzinc( *a++, &sum );
 return( sum );
}





void free_matrix_z(
 complex** matrix,
 const integer xmin,
 const integer ymin
)
{
 free( (char*) (matrix[ymin]+xmin) );  /* Free columns*/
 free( (char*) (matrix+ymin) );		/* Free rows*/
}


void free_matrix_dz(
 dcomplex** matrix,
 const integer xmin,
 const integer ymin
)
{
 free( (char*) (matrix[ymin]+xmin) );  /* Free columns*/
 free( (char*) (matrix+ymin) );		/* Free rows*/
}



complex** alloc_matrix_z( 
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer nx = xmax - xmin + 1;
 integer ny = ymax - ymin + 1;
 complex*  ptr;
 complex** row_ptr;
 integer row;

 /* Allocate ny pointers to float */
 row_ptr = (complex **) malloc( (size_t)( nx * sizeof( complex* ) ) );
 row_ptr -= xmin;
 /* Allocate nx * ny floats */
 ptr = (complex *) malloc( (size_t) ( ny * nx * sizeof( complex ) ) );
 ptr -= ymin;

 row_ptr[xmin] = ptr;
 for ( row = xmin + 1; row <= xmax; row++ ) {
  row_ptr[row] = row_ptr[row-1] + ny;
 }
 return( row_ptr );
}


dcomplex** alloc_matrix_dz( 
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer nx = xmax - xmin + 1;
 integer ny = ymax - ymin + 1;
 dcomplex*  ptr;
 dcomplex** row_ptr;
 integer row;

 /* Allocate ny pointers to float */
 row_ptr = (dcomplex **) malloc( (size_t)( nx * sizeof( dcomplex* ) ) );
 row_ptr -= xmin;
 /* Allocate nx * ny floats */
 ptr = (dcomplex *) malloc( (size_t) ( ny * nx * sizeof( dcomplex ) ) );
 ptr -= ymin;

 row_ptr[xmin] = ptr;
 for ( row = xmin + 1; row <= xmax; row++ ) {
  row_ptr[row] = row_ptr[row-1] + ny;
 }
 return( row_ptr );
}




void ar_copy_dzz(
 const dcomplex* x,
 complex* y,
 const integer m
)
{
 integer n = m;
 while( n-- ) { 
  y->x = (real) x->x;
  y->y = (real) x->y;
  x++; y++;
 }
}

void ar_copy_zdz(
 const complex* x,
 dcomplex* y,
 const integer m
)
{
 integer n = m;
 while( n-- ) { 
  y->x = (double) x->x;
  y->y = (double) x->y;
  x++; y++;
 }
}



logical gt_z( const complex z1, const complex z2 )
{
 return( z1.x > z2.x  && z1.y > z2.y );
}

logical ge_z( const complex z1, const complex z2 )
{
 return( z1.x >= z2.x  && z1.y >= z2.y );
}

logical lt_z( const complex z1, const complex z2 )
{
 return( z1.x < z2.x  && z1.y < z2.y );
}

logical le_z( const complex z1, const complex z2 )
{
 return( z1.x <= z2.x  && z1.y <= z2.y );
}

logical gt_dz( const dcomplex z1, const dcomplex z2 )
{
 return( z1.x > z2.x  && z1.y > z2.y );
}

logical ge_dz( const dcomplex z1, const dcomplex z2 )
{
 return( z1.x >= z2.x  && z1.y >= z2.y );
}

logical lt_dz( const dcomplex z1, const dcomplex z2 )
{
 return( z1.x < z2.x  && z1.y < z2.y );
}

logical le_dz( const dcomplex z1, const dcomplex z2 )
{
 return( z1.x <= z2.x  && z1.y <= z2.y );
}

 
real argd_z( const complex z )
{
 return( argd_r( z.x, z.y ) );
}

double argd_dz( const dcomplex z )
{
 return( argd_d( z.x, z.y ) );
}


logical clip_z( const complex z, const complex z1, const complex z2 )
{
 return( ge_z( z, z1 ) && le_z( z, z2 ) );
}
logical clip_dz( const dcomplex z, const dcomplex z1, const dcomplex z2 )
{
 return( ge_dz( z, z1 ) && le_dz( z, z2 ) );
}


