#include <stdio.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "esc_types.h"
#include "gtoken.h"

#define TPIO_BUFLEN 512
#define IO_DEPTH    5

static logical vv = FALSE;

char** tpio_set( void );
static logical tpio_cmd_mode = FALSE;
static char tpio_buffer[IO_DEPTH][TPIO_BUFLEN];
static char* tpio_cmd = tpio_buffer[0];
static char* tpio_bptr[IO_DEPTH];
static char** tpio_ptr = NULL;
static integer tpio_nloop = 0;
#define MAX_LOOPS 10
t_word  tpio_lvar[MAX_LOOPS];

static integer loop_value[MAX_LOOPS];
static integer loop_step[MAX_LOOPS];
static integer loop_maxval[MAX_LOOPS];
static integer loop_addr[MAX_LOOPS];

void tpio_loc( integer* level, integer* pos );

/*
 *--------------------------------------------------------------------------------
 * Internal routine to give buffer status
 */

void tpio_loc( integer* level, integer* pos )
{
 *level = iofs_get_level();
 if ( *tpio_ptr == NULL ) {
  *pos = -1;
 } else {
  *pos = *tpio_ptr - tpio_buffer[*level] + 1;
 }
}

/*
 *--------------------------------------------------------------------------------
 * Internal debugging
 */
void vprint( char* buf )
{
 if ( vv ) {
   msg( buf );
   fio_list();
   iofs_print_state();
 }
}

/*
 *--------------------------------------------------------------------------------
 * Clear one level of the buffer
 */
void tpio_cclear( integer level )
{
 tpio_bptr[level] = tpio_buffer[level];
 cs_zero( tpio_bptr[level] );
}


/*----------------------------------------------------------------------*/
/*
 * tpio_dbg
 * Uses <stdio.h>
 */

void tpio_dbg( void )
{
 printf("TPIO BUFFER POINTER %p\n", tpio_ptr );
 if ( tpio_ptr != NULL ) printf("TPIO BUFFER HEAD    %p\n", *tpio_ptr );
}


/*----------------------------------------------------------------------*/
/*
 * tpio_init
 *
 * Zero the buffer and set the stack to bottom level
 * Uses csproto.h
 */

void tpio_init( void )
{
 integer level;
 for ( level = 0; level < IO_DEPTH; level++ ) {
  tpio_bptr[level] = NULL;
  cs_zero( tpio_buffer[level] );
 }
 tpio_ptr = tpio_bptr;
}

/*----------------------------------------------------------------------*/
/* Internal buffer manipulation */
char** tpio_set( void )
{
 integer level;
 level = iofs_get_level();
 if ( tpio_bptr[level] == NULL ) tpio_bptr[level] = tpio_buffer[level];
 tpio_ptr = &(tpio_bptr[level]);
 return( tpio_ptr );
}
/*
 * tpio_getbuf
 *
 * For interactive use, prompt for new buffer and read it from the terminal
 * using iofs_read.
 * Strip comments (tabs and continuations are handled in iofs_read).
 * In non-interactive use, read buffer from input stream; keep trying
 * if you get a blank line or a line starting with comment as the first character. Keeps going until
 * it gets either end of file, or a line with data. The data returned to
 * the internal buffer can be blank if either 1) end of file is found or 2)
 * the line consists of whitespace followed by comment.
 *
 * Uses csproto.h
 * Uses escproto.h
 * Uses esc_types.h
 * Uses ioproto.h : iofs_read
 */


/*
 *--------------------------------------------------------------------------------
 */
logical tpio_getbuf( char* prompt )
{
  logical retry;
  char cmt_char, esc_char;
  logical status;
  char* buf;
  char** ptr;
  card ebuf;

  ptr = tpio_set();
  buf = *ptr;
  retry = TRUE;
  esc_char = *esc_get_char( ESC_CHAR );
  cmt_char = *esc_get_char( CMT_CHAR );
  while( retry || *buf == cmt_char ) {
   status = iofs_read( buf, prompt, TPIO_BUFLEN );
   if ( dbg_level(10) ) {
    sprintf( ebuf, "TPIO new buffer:%s", buf );   dmsg( 10, ebuf );
   }
   retry  = ( status  && cs_is_blank( buf ) && !io_int(0) );
  }

  cs_decmt( buf, esc_char, cmt_char );
  return( status );
}

/*----------------------------------------------------------------------*/
/*
 *
 * tpio_loop_check
 *
 * Test for \END if in loop.
 * Uses csproto.h, esc_types.h, escproto.h
 */

static logical tpio_loop_check( char* buf )
{
 char end_string[5] = "\\END";

 if ( tpio_nloop > 0 ) {
  if ( fio_any_eof( 1 ) ) {   
   end_string[0] = *esc_get_char( ESC_CHAR );
   return ( !cs_eq( end_string, buf ) );
  }
 } 
 return( FALSE );
}

/*----------------------------------------------------------------------*/
/*
 * tpio_next_name
 *
 * Get the next word in the buffer
 * Uses tokenproto.h
 */

static void tpio_next_name( char* buf )
{
  integer siz;
  siz = FSIZE;
  cs_get_token( tpio_ptr, buf, TMODE_WORD, siz );
}

/*----------------------------------------------------------------------*/
/*
 * tpio_cmdfile
 *
 * Open file and use as input 
 * Uses csproto.h
 */

static void tpio_cmdfile( char* buf )
{
 t_filename filename;
 if( *buf == '\0' ) {
  tpio_next_name( filename );
 } else {
  cs_copy( buf, filename );
 }
 tpio_in( filename );
}

/*----------------------------------------------------------------------*/
/*
 * tpio_logfile
 *
 * Uses csproto.h
 * 
 */

static void tpio_logfile( char* buf )
{
 t_filename filename;
 logical append, syserr;
 integer lptr;
 char* ptr;
 lptr = 1;
 ptr = buf;
 append = (*ptr == '>' );
 if ( append ) ptr++;
 syserr = (*ptr == '&' );
 if ( syserr ) ptr++;
 io_opop( 1 );
 cs_copy( ptr, filename );
 if ( cs_is_blank( filename ) ) tpio_next_name( filename );
 tpio_out( filename, append, syserr );
}

/*----------------------------------------------------------------------*/
/*
 * tpio_esc
 * Uses csproto.h
 */

static logical tpio_esc( char* buf )
{
 logical flag;
 card ebuf;
 card ubuf;
 integer ival;
 char* ptr;
 t_word token;
 t_word token1;
 card tbuf;
 card embuf;


 cs_copy( " ", embuf );
 flag = TRUE;
 cs_copy( buf, ebuf );
 cs_upper( ebuf );
 ptr = ebuf;
 if( buf[0] == '<' )
  tpio_cmdfile( &buf[1] );
 else if ( buf[0] == '>' )
  tpio_logfile( &buf[1] );
 else if ( cs_eq_siz( buf, "[*", 2 ) ) {
  ival = read_i( cs_upto( &buf[2], ']', ubuf ) );
  printf("DEBUG SET %ld\n", (long)ival );
  dbg_set( ival );
 } else if ( cs_eq( ptr, "SET" ) ) {
  dmsg( 10, "ESC <Reset special chars>" );
  cs_get_token( tpio_ptr, token, TMODE_WORD, WSIZE );
  dmsg1( 10, "ARG = ", token );
  if ( cs_eq( token, "?" ) ) {
   esc_status();
  } else {
   cs_get_token( tpio_ptr, token1, TMODE_WORD, WSIZE );
   esc_reset( token, *token1 );
  }
 } else if ( cs_eq( ptr, "VER" ) ) {
  ver_msg();
 } else if ( cs_eq( ptr, "CLEAR" ) ) {
  jcm_err_msg( "Cleared error state" );
 } else if ( cs_eq( ptr, "DO" ) ) {
  loop_do();
 } else if ( cs_eq( ptr, "END" ) ) {
  loop_end();
 } else if ( cs_eq( ptr, "ECHO" ) ) {
  loop_echo();
 } else if ( cs_eq( ptr, "EXIT" ) ) {
  io_ipop(1);
 } else if ( *ptr == '?' ) { 
  if ( cs_eq( ptr, "?IOFS" ) ) {
   iofs_print_state();
  } else if ( cs_eq( ptr, "?FIO" ) ) {
   fio_list();
  } else if ( cs_eq( ptr, "?BIO" ) ){
   io_print_state();
  } else if ( cs_eq( ptr, "?PAGER" ) ){
   pager_print_state();
  } else if ( cs_eq( ptr, "?PATH" ) ){
   fio_path_list();
  } else if ( cs_eq( ptr, "?TPIO" ) ) {
   tpio_print_state();
  } else if ( cs_eq( ptr, "?ESC" ) ) {
   esc_status();
  } else {
   cs_copy( ptr, embuf );
  } 
 } else {
  cs_copy( ptr, embuf );
 }
 flag = !( cs_is_blank( buf ) || !cs_is_blank( embuf ) );
 if ( !flag ) {
  sprintf( tbuf, " [ESC] Unknown escape command-:%-s", embuf );
  dmsg( 5, tbuf );
 }
 return( flag );
}

/*----------------------------------------------------------------------*/
/*
 * tpio_test_esc
 * Uses csproto.h escproto.h esc_types.h
 */

static logical tpio_test_esc( char* buf )
{
 logical flag;
 char esc_char;
 esc_char = *esc_get_char( ESC_CHAR );
 flag = tpio_loop_check( buf );
 if ( !flag ) {
  if ( buf[0] == esc_char ) {
   if ( buf[1] == esc_char ) { cs_copy( &buf[1], buf ); return( flag ); }
   flag = tpio_esc( &buf[1] );
  }
 }
 return( flag );
}


/*----------------------------------------------------------------------*/
/*
 * tpio_token
 *
 * Read new buffer if needed until token found. Handle escapes.
 * Pop all stacks if EOF found.
 * Uses tokenproto.h, ioproto.h
 */

void tpio_token( char* prompt, integer mode, char* token, integer siz )
{
 logical loop;
 logical status;
 card tbuf;
 integer level, pos;

 loop = TRUE;
 status = FALSE;
 while ( loop ) {
  if ( dbg_level(10) ) {
   tpio_loc( &level, &pos );
   sprintf( tbuf, "TPIO TOKEN on Level %-6ld Pos %-6ld", (long)level, (long)pos );
   dmsg( 10, tbuf );
  }
  if ( *tpio_ptr == NULL ||  **tpio_ptr == '\0' ) 
  { 
    status = tpio_getbuf( prompt );
   if ( dbg_level(10) ) {
    tpio_loc( &level, &pos );
    sprintf( tbuf, "TPIO new buffer on Level %-6ld Pos %-6ld", (long)level, 0L );
    dmsg( 10, tbuf );
   }
  }
  cs_get_token( tpio_ptr, token, mode, siz );
  loop = tpio_test_esc( token );
 }

 if ( dbg_level(10) ) {
  sprintf( tbuf, "TPIO TOKEN =:%s: REM =:%s: %c", *tpio_ptr, token,
   status ? ' ' : '*' );
  dmsg( 10, tbuf );
 }
}

/*----------------------------------------------------------------------*/
/*
 * tpio_in
 * Uses ioproto.h fileproto.h
 * Resets TPIO input to the file named in the argument
 */

void tpio_in( char* buf )
{
 integer in;
 card tbuf;
 if ( cs_eq( buf, "*" ) ) {
  in = 0;
  dmsg( 9 , "Opened interactive command level" );
 } else {
  in = fio_open_r( buf );
  if ( in == 0 ) {
   jcm_err_msg1( "Failed to open command file ", buf );
   io_iclear();
  } else {
   sprintf( tbuf, "OPEN COMMAND FILE %s CHANNEL %ld", buf, (long)in );
   dmsg( 9, tbuf );
  }
 }
 io_ipush( in );
}

/*----------------------------------------------------------------------*/
/*
 * tpio_out
 * Uses ioproto.h fileproto.h
 * Resets TPIO output to the file in the argument
 * If syserr is true, sends errors there too.
 * If append is true, appends output to an existing file.
 */

void tpio_out( char* buf, logical append, logical syserr )
{
 integer out;
 if ( cs_eq( buf, "*" ) ) {
  out = 0; 
 } else if ( append ) {
  out = fio_open_a( buf );
 } else {
  out = fio_open_w( buf );
 }
 io_opush( out );
 if ( syserr ) io_err( out );
}

/*
 *--------------------------------------------------------------------------------
 * Handles TPIO loop control
 */

void loop_do( void )
{
 card buf0;
 char* buf;
 char* var;
 integer iloop;
 buf = buf0;

 cs_get_token( tpio_ptr, buf, TMODE_STRING, CSIZE );
 if ( tpio_nloop > MAX_LOOPS ) {
  jcm_err_msg( "TPIO Error: Loops nested too deeply" );
 } else {
  tpio_nloop++;
  iloop = tpio_nloop-1;
  var = tpio_lvar[iloop];
  cs_copy_token( &buf, "=", " ", var, WSIZE );
  if ( cs_eq( var, "EOF" ) ) {
   loop_value[iloop] = 0;
   loop_step[iloop]  = 0;
   loop_maxval[iloop] = 0;
  } else {
   loop_value[iloop] = cs_get_i( &buf );
   loop_maxval[iloop] = cs_get_i( &buf );
   loop_step[iloop] = ( buf == NULL ) ? 1 : cs_get_i( &buf );
  }
  loop_addr[iloop] = fio_get_lineno( io_chan( 0 ) );
 }
}

/*
 *--------------------------------------------------------------------------------
 * Handles loop iteration and end of loop condition
 */
void loop_end( void )
{
 integer iloop;
 logical finish;
 if ( tpio_nloop == 0 ) {
  jcm_err_msg( "TPIO: Loop END without DO" );
  
 } else {
  iloop = tpio_nloop-1;
  if ( loop_step[iloop] == 0 ) {
   finish = fio_any_eof( 1 );
  } else {
   loop_value[iloop] += loop_step[iloop];
   if ( loop_step[iloop] >= 0 ) {
    finish = ( loop_value[iloop] > loop_maxval[iloop] );
   } else {
    finish = ( loop_value[iloop] < loop_maxval[iloop] );
   }
  }
  if ( finish ) {
   loop_value[iloop] = 0;
   tpio_nloop--;
  } else {
   loop_goto( loop_addr[iloop] );
  }
 }
}

/*
 *--------------------------------------------------------------------------------
 * Go to a specified line in a sequential text file
 */

void loop_goto( integer addr )
{
 integer i;
 card buf;
 io_rewind();
 for ( i = 0; i < addr; i++ ) {
  io_input( buf, CSIZE );
 }
}

/*
 *--------------------------------------------------------------------------------
 * Echo text substituting $I macros
 */

void loop_echo( void )
{
 char macro_char;
 t_buf buf;
 t_word old, new;
 integer i;
 cs_get_token( tpio_ptr, buf, TMODE_STRING, CSIZE );
 macro_char = *esc_get_char( MACRO_CHAR );
 if ( cs_index_char( buf, macro_char ) > 0 ) {
  for ( i = 0; i < tpio_nloop; i++ ) {
   *old = macro_char;
   cs_copy( tpio_lvar[i], old+1 );
   write_i( loop_value[i], new, WSIZE );
   string_replace( buf, old, new );
  }
 }
 msg( buf );
}

/*
 *--------------------------------------------------------------------------------
 * Display TPIO internal state
 */

void tpio_disp( integer i )
{
 integer idbg;
 char clev[8];
 idbg = dbg_get();
 if ( idbg >= i ) {
  sprintf( clev, "[*%ld]", (long)i );
  if ( idbg > 10 ) { 
   fio_list();
   iofs_print_state();
  } else {
   iofs_print_top( "I", clev );
  }
  tpio_print_state();
 } 

}
/*
 *--------------------------------------------------------------------------------
 */


void tpio_print_state( void )
{
 integer level;
 char buf[512];
 t_word spacer;
 integer min_lev;
 integer lev;
 char* gptr;
 integer pos;
 t_word eflag;
 t_buf disp;
 t_word stat;


 level = iofs_get_level();
 msg( " " );
 dashline( 40 );
 msg( " JCMLIB Command Processor (TPIO) Status " );
 dashline( 40 );
 sprintf( buf, "  TPIO Stack Level: %ld", (long)level );
 msg( buf );
     msg( "  ------------------- " );
     msg( "  | Level Pos Flag  | " );
 cs_copy( "  |                 | " , spacer );
 min_lev = tpio_cmd_mode ? 0 : 1;
 for ( lev = level; lev >= min_lev; lev-- ) {
  gptr = tpio_buffer[lev];
  if ( tpio_bptr[lev] == NULL ) {
   pos = -1;
   cs_copy( "Empty", eflag );
  } else {
   pos = tpio_bptr[lev] - tpio_buffer[lev];
   cs_copy( " ", eflag );
  }
  sprintf( stat, "  | %-6ld%-4ld%-6s| ", (long)lev, (long)pos, eflag );
  if ( pos == 0 ) {
   cs_copy( ":^", disp );
  } else if ( pos < 0 ) {
   cs_copy( "<", disp );
  } else if ( pos > 250 ) {
   cs_copy( ":?", disp );
  } else {
   sprintf( disp, ":%-250s", " " );
   disp[pos] = '^';
  }
  sprintf( buf, "%-21s:%s", stat, tpio_buffer[lev] );
  msg( buf );
  sprintf( buf, "%-21s%s", spacer, disp );
  msg( buf );
 } 
 msg( "  ------------------ " );
 msg( " " );
}

/*
 *--------------------------------------------------------------------------------
 * Top level TPIO special command routine
 */


void tpio_cc( char* prompt, char* buf, integer siz )
{
 
 if ( tpio_cmd_mode ) {
  tpio_cc_cmd( prompt, &tpio_cmd_mode, buf, siz );
 }
 if ( !tpio_cmd_mode ) {
  tpio_c( prompt, buf, siz  );
  if ( dbg_get() >=1 ) {
   dmsg1( 1, "TPIO_CC READ WORD ", buf );
  }
  tpio_cc_start_check( buf, &tpio_cmd_mode );
  if ( tpio_cmd_mode ) tpio_cc_cmd( prompt, &tpio_cmd_mode, buf, siz );
 }
}

/*
 *--------------------------------------------------------------------------------
 * Check if we are in a special mode
 */

void tpio_cc_start_check( char* buf, logical* mode )
{
 card ubuf;
 if ( *buf == *esc_get_char( ESC_CHAR ) ) {
  cs_copy( buf+1, ubuf );
  if ( cs_is_blank( ubuf ) || cs_eq( ubuf, "TABLE" ) ) {
   *mode = TRUE;
   dmsg( 10, "TPIO_CC SET CMD TRUE" );
   cs_get_token( tpio_ptr, tpio_cmd, TMODE_LINE, BSIZE );
   tpio_cclear( 0 );
  }
 }
}

/*
 *--------------------------------------------------------------------------------
 */

void tpio_cc_cmd( char* prompt, logical* mode, char* buf, integer siz )
{
 logical eot;
 logical found_esc;

 if ( cs_is_blank( tpio_cmd ) ) {
  *mode = FALSE;
  dmsg( 10, "TPIO_CC SET CMD FALSE (EMPTY)" );
 } else {
  tpio_cc_check( prompt, &eot, &found_esc, buf, siz );
  if ( eot ) {
   *mode = FALSE;
   dmsg( 10, "TPIO_CC SET CMD FALSE" );
  }
  if ( *mode  && !found_esc ) {
   cs_get_token( tpio_ptr, buf, TMODE_WORD, siz );
   dmsg1( 10, "TPIO_CC READ CMD ", buf );
  }
 }
}

/*
 *--------------------------------------------------------------------------------
 */

void tpio_cc_check( char* prompt, logical* eot, logical* found_esc, char* buf, integer siz )
{
 t_word etoken;
 char esc;
 char* save_ptr;

 esc = *esc_get_char( ESC_CHAR );
 *eot = FALSE;
 *found_esc = FALSE;

 if ( !eot ) {
  save_ptr = *tpio_ptr;
  cs_get_token( tpio_ptr, etoken, TMODE_WORD, WSIZE );
  dmsg1( 10, "TPIO_CC CHECK ", etoken );
  if ( *etoken == esc && *(etoken+1) != esc ) {
   cs_upper( etoken );
   tpio_cc_esc( etoken + 1, prompt, eot, found_esc, buf, siz );
  }
 }
 if ( ! (*found_esc) ) {
  *tpio_ptr = save_ptr;
 }
}

/*
 *--------------------------------------------------------------------------------
 */

void tpio_cc_esc( char* ecmd, char* prompt, logical* eot, 
 logical* found_esc, char* buf, integer siz )
{
 *(found_esc) = TRUE;
 if ( cs_is_blank( ecmd ) || cs_eq( ecmd, "ENDTABLE" ) ) {
  *eot = TRUE;
  dmsg( 10, "TPIO_CC EOT FOUND" );
 } else if ( cs_eq( ecmd, "C" ) ) {
  tpio_c( prompt, buf, siz );
 } else if ( cs_eq( ecmd, "D" ) ) {
  cs_get_token( tpio_ptr, buf, TMODE_WORD, siz );
  dmsg1( 10, "TPIO_CC SKIP CMD ", buf );
  cs_copy( " ", buf );
 } else {
  *found_esc = FALSE;
 }

}

/*
 *--------------------------------------------------------------------------------
 */


void tpio_setbuf( char** arr, integer n )
{
 char sep;
 char* ptr;
 integer i;
 integer siz = TPIO_BUFLEN;
 sep = *esc_get_char( STRING_SEP );


 ptr = *tpio_set();
 if (!cs_is_blank(ptr) ) {
  *ptr++ = sep;
 }
 for ( i = 0; i < n; i++ ) { 
  cs_put_token( arr[i], &ptr, sep, &siz );
 }
}


/*
 *  Specialized command argument processing for PRECESS family
 * This still needs some work to make it a clone of the Fortran version.
 * 1997 Nov 27 JCM
 */
/*
 *--------------------------------------------------------------------------------
 */

void tpio_cmdargs( 
 char** args,         /* (i) Command line args */
 integer nargs,       /* (i) Number of args */
 logical* interact,   /* (o) Interactive? */
 logical* opts        /* (o) Command line was not blank */
)
{
 t_buf argline;
 card clargs; 
 integer in;
 integer out;
 card buf;
 char* ptr;
 t_filename infile;
 t_filename outfile;
 t_word dargs;
 char esc[2];
 integer id;

/* Reconstitute the original command line */
 sys_cmdline( args, nargs, argline, BSIZE );
 esc[0] = *esc_get_char( ARG_SEP );
 esc[1] = '\0';
 *interact = TRUE;
 if ( ( *opts = !cs_is_blank( argline )) ) {

/* There are some command line arguments */

  ptr = argline;
/* Read the first token and check if it is DEBUG or DEBUGn */
  cs_copy_token( &ptr, esc, " ", clargs, CSIZE );
  cs_copy_siz( clargs, dargs, 5 );
  cs_upper( dargs );
  if ( cs_eq( dargs, "DEBUG" ) ) {
   if ( !cs_is_blank( clargs+5 ) ) {
    id = read_i( clargs+5 );
   } else {
    id = 10;
   }
   if ( ptr == NULL ) {
    cs_copy( " ", clargs );
   } else {
    cs_copy_token( &ptr, esc, " ", clargs, CSIZE );
   }
  }
/* Get another token: this will be the input file, open it */
  *interact = ( ptr == NULL );
  if ( ptr != NULL ) {
   cs_copy_token( &ptr, esc, " ", infile, FSIZE );
   if ( !cs_is_blank( infile ) ) {
    in = fio_open_r( infile );
    io_ipush( in );
    if ( in > 0 ) { sprintf( buf, "Opened input file %s", infile ); tmsg( buf ); }
   }
  }
/* Get another token: this will be the output file,open it */
  if ( ptr != NULL ) {
   cs_copy_token( &ptr, esc, " ", outfile, FSIZE );
   if ( !cs_is_blank( outfile ) ) {
    out = fio_open_w( outfile );
    io_opush( out );
    if ( out > 0 ) { sprintf( buf, "Opened output file %s", outfile ); tmsg( buf ); }
   } 
  }
/*  If you have arguments, put them in the TPIO buffer to be executed */
  if ( !cs_is_blank( clargs ) ) {
   ptr = clargs;
   tpio_setbuf( &ptr, 1 );
  }
 }
 
}

