#include <stdio.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "fio_file.h"
#include "fio.h"


/*
 *----------------------------------------------------------------------
 * Routines 
 *----------------------------------------------------------------------
 */


/*
 *--------------------------------------------------------------------------------
 * Setting JCM FILE parameters
 *--------------------------------------------------------------------------------
 */

 
/*----------------------------------------------------------------------*/

integer fio_status( integer fid, integer mode )
{

/*
 * This routine uses the fio_file structure definition.
 */
 fio_file* fio_File;
 integer status;
 if ( fid > 0 ) {
  fio_File = fio_file_ptr( fid );
  status = fio_File->status;
  if ( mode == FIO_CLEAR ) fio_File->status = FILE_OPEN;
  return( status );
 } else {
  return( 0 );
 }
}


void fio_set_status_pos( integer fid, logical ios, integer pos ) 
{
 fio_file* ptr;
 if ( fid == 0 ) return;
 ptr = fio_file_ptr( fid );
 ptr->status = ios ? FILE_OPEN : FILE_EOF;
 if ( ios ) {
  ptr->line++;
  ptr->pos = pos;
 }
}

void fio_set_status( integer fid, logical ios ) 
{
 fio_file* ptr;
 if ( fid == 0 ) return;
 ptr = fio_file_ptr( fid );
 ptr->status = ios ? FILE_OPEN : FILE_EOF;
}


void fio_set_pos( integer fid, integer pos )
{
 fio_file_ptr( fid )->pos = pos;
}

void fio_delta_pos( integer fid, integer pos )
{
 fio_file_ptr( fid )->pos += pos;
}


void fio_rewind( integer fid )
{
 rewind( fio_file_fptr( fid ) );
 fio_line_reset( fid );
}


logical fio_eof( integer fid, integer mode )
{
 return( fio_status( fid, mode ) == FILE_EOF );
}
/*----------------------------------------------------------------------*/
/*
 *
 * Set the values in a JCM file structure.
 *
 */


static void fio_checkin( integer File_no, const char* Name, char* Type, integer Status, FILE* Fptr, integer stream )
{

/*
 * This routine uses the fio_file structure definition.
 * Uses:
 *  csproto.h: cs_copy
 * <stdio.h>: FILE* 
 */
  fio_file* fio_File;

  fio_File = fio_file_ptr( File_no );
  cs_copy( Name, fio_File->Name );
  cs_copy( Type, fio_File->type );
  fio_File->status = Status;
  fio_File->fptr = Fptr;
  fio_File->stream = stream;
  fio_line_reset( File_no );
}

void fio_line_reset( integer fid )
{
 fio_file_ptr( fid )->pos = 0;
 fio_file_ptr( fid )->line = 0;
}
/*----------------------------------------------------------------------*/
/*
 * fio_file_alloc
 *
 * Allocate and set a new JCM file given its properties, but only if it is
 * a STDIO file.
 * Uses:
 *   <stdio.h>:  printf
 *   copen:      fio_alloc, fio_file_set
 */

integer fio_file_alloc( const char* Name, char* Type, integer Status, FILE* Fptr )
{

/*
 * This routine uses the fio_file structure definition.
 */

  integer file_no;

  if ( Fptr == NULL ) {
   file_no = NULL_FILE; 
  } else {
   file_no = fio_alloc();

   fio_checkin( file_no, Name, Type, Status, Fptr, fileno( Fptr ) );
  }
  return( file_no );
}

integer fio_cfile_alloc( char* Name, char* Type, integer fd )
{

/*
 * This routine uses the fio_file structure definition.
 */

  integer file_no;

  if ( fd < 0 ) {
   file_no = NULL_FILE; 
  } else {
   file_no = fio_alloc();

   fio_checkin( file_no, Name, Type, FILE_OPEN, (FILE *)NULL, fd );
  }
  return( file_no );
}


/*----------------------------------------------------------------------*/
/*
 *  fio_file_clear
 *
 */

void fio_file_clear( integer file_no ) 
{
/*
 * This routine uses the fio_file structure definition.
 */
 if ( file_no > 0 ) {
  fio_checkin( file_no, " ", " ", FILE_CLOSED, NULL, 0 );   
 }
}



integer fio_get_cfd( integer fid )
{
 return( fio_file_ptr( fid )->stream );
}

