#include <stdio.h>
#include <string.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "fio_file.h"

static logical fio_set_mode( char* jmode, char* mode );

/*----------------------------------------------------------------------*/
/* 
 * fio_read_line
 *
 * Read a line from an ASCII file.
 * Uses  stdin and type FILE*  from <stdio.h>
 * Uses fgets from <stdio.h>
 * Uses NULL_FILE from fio_file.h
 * Uses fileproto.h: fio_file_fptr and fio_set_eof
 * Uses csproto.h: cs_null and cs_remove_nl from csproto.h
 */


logical fio_write( integer fid, const char* buf )
{
 logical ios;
 if ( fid == 0 ) {
  ios = tty_write( buf );
 } else {
  ios = fio_put( fid, buf, TRUE, TRUE );
 }
 return( ios );
}

logical fio_write_prompt( integer fid, char* buf )
{
 logical ios;
 if ( fid == 0 ) {
  ios = tty_writel( buf );
 } else {
  ios = fio_put( fid, buf, FALSE, FALSE );
 }
 return( ios );
}

integer fio_terminate_buf( integer pos, const char* buf, char* buf2, logical strip, logical nl, integer siz )
{
 integer i;
 if ( strip ) {
  i = min_i( cs_ends( buf ), siz-1 );
 } else {
  i = min_i( strlen( buf ), siz-1 );
 }
 cs_copy_siz( buf, buf2, i );
 if( nl ) {
  buf2[i++] = '\n';
  pos = 0;
 } else {
  pos += i;
 }
 buf2[i] = '\0';
 return( pos );
}

logical fio_read( integer fid, char* line, integer lmax )
{
 logical ios;
 if ( fid == 0 ) {
  ios = tty_read( line, lmax );   
 } else {
  ios = fio_get( fid, line, lmax );
  fio_set_status_pos( fid, ios, 0 );
 }
 return( ios );
}




integer fio_open_r( const char* filename )
{


 integer file_no;
 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "SEQ R", 0 );
 }
 return(file_no); 
}


integer fio_open_retry( const char* filename, char* mode, integer rec_bytes )
{
 integer fid;
 logical retry;
 logical loop;
 t_filename fullname;
 fid = 0;
 if (! cs_is_blank( filename ) ) {
  cs_copy_siz( filename, fullname, FSIZE );
  retry = fio_get_retry();
  loop = ( fid == 0 );
  while ( loop ) {
   fid = fio_open_file( fullname, mode, rec_bytes );
   loop = ( fid == 0 ) && retry;
   if ( loop ) {
    jcm_err_msg( "OPEN RETRY: Try again, full filename (Q to quit): " );
    tty_read( fullname, FSIZE );
    if ( cs_eq( fullname, "Q" ) || cs_eq( fullname, "q" ) ) {
     loop = FALSE;
     cs_copy( " ", fullname );
     jcm_err_msg1( "OPEN RETRY: Abandoning attempt to open file ", filename );
     err_set( JCM_FIO_OPEN_FAIL );
    }
   }
  }
 }
 return( fid );
}

/*
 *  Here we convert from JCMLIB's file modes to stdio.h's file modes.
 */
static logical fio_set_mode( char* jmode, char* mode )
{ 
 static char* mode_list[] = { "SEQ R", "SEQ W", "SEQ A", "SIO R", "SIO W", "SIO RW",
                       "CIO R", "CIO W", "CIO RW" };
 static char* mode_val[]  = { "r", "w", "a", "r", "w", "r+", "cr", "cw", "crw" };
 static logical mode_exist[] = { TRUE, FALSE, TRUE, TRUE, FALSE, TRUE, TRUE, 
                          FALSE, TRUE };
 integer nmodes = 9;
 integer mode_no;
 mode_no = cmatch_c( jmode, mode_list, nmodes );
 if ( mode_no >= 0 ) {
  cs_copy( mode_val[mode_no], mode );
  return( mode_exist[mode_no] );
 } else {
  cs_copy( "x", mode );
  err_set( JCM_FIO_BAD_MODE );
  return( FALSE );
 }
}

integer fio_open_file( const char* filename, char* jmode, integer rec_bytes )
{
 FILE *file_ptr; 
 integer fid; 
 t_word mode;
 t_filename truefile;
 t_filename pathfile;
 integer k;

 cs_copy_siz( filename, pathfile, FSIZE );
 k = cs_ends( pathfile );
 pathfile[k] = '\0';
 if ( fio_set_mode( jmode, mode ) ) {
  fio_inquire_path( pathfile, truefile, FSIZE );
 } else {
  cs_copy( pathfile, truefile );
 }
 fid = 0;
 if ( cs_ss_char( "rwa", *mode ) ) {
  file_ptr = fopen( truefile, mode );
  fid = fio_file_alloc( filename, jmode, FILE_OPEN, file_ptr );
 } else if ( *mode == 'c' ) {
  fid = cio_open( truefile, jmode, mode );
 } else {
  printf( "FIO OPEN FILE: Bad mode %s %ld\n", mode, rec_bytes );
 }
 if ( fid == 0 ) { 
  err_set( JCM_FIO_OPEN_FAIL );
 }
 return( fid );
}
/*----------------------------------------------------------------------*/
/*
 * fio_open_w
 *
 * Open an ASCII file to write using C stdio
 * Uses
 *  <stdio.h>  fopen, FILE *
 * fileproto.h: fio_file_alloc
 *  csproto.h: cs_is_blank
 */

integer fio_open_w( const char* filename )
{
 integer file_no;
 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "SEQ W", 0 );
 }
 return(file_no); 
}

integer fio_open_a( const char* filename )
{
 integer file_no;

 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "SEQ A", 0 );
 }
 return(file_no); 
}

/*
 *  Open a direct access file, for compatibility with Fortran.
 *  Not fully supported yet; use SIO instead.
 */
integer fio_open_d( const char* filename, integer rec_bytes )
{
 integer file_no;

 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "SEQ D", rec_bytes );
 }
 return(file_no); 
}


/*----------------------------------------------------------------------*/
/*
 * fio_close
 *
 * Close a file opened using JCMLIB.
 * Uses
 *  <stdio.h> FILE*, fclose
 */

void fio_close( integer fid )
{

 t_word type;
 FILE* fptr;
 if ( fid > 0 ) {
  fio_get_type( fid, type, WSIZE );
  if ( cs_eq_siz( type, "CIO", 3 ) ){
   if ( cs_eq( type, "CIO RAW" ) ) cio_clear_raw( fid );
   cio_close( fid );
  } else {
   fptr = fio_file_fptr( fid );
   fclose( fptr );
  }
  fio_file_clear( fid );
 }
}


