/*======================================================================*/
/*
 *   cs lib 
 *
 *   C string routines to manipulate strings
 *
 *   string.h: strlen 		used in cs_remove_nl
 *
 */
/*======================================================================*/
#include <string.h>
#include "jcmlib.h"



/*----------------------------------------------------------------------*/
/*
 *   cs_remove_nl
 *
 *   Remove trailing \r and \n characters from a line.
 *
 *   1996 Aug 26 JCM
 *   Args:
 *	ptr		(i/o)  string to remove trailing newlines from
 */

void cs_remove_nl( char* ptr )
{
 integer length;

 length = strlen( ptr );
 if ( length ) {
  ptr += length - 1;
  if ( *ptr == '\n' ) { *ptr-- = '\0'; length--; }
  if ( length  && *ptr == '\r' ) *ptr = '\0';
 }
}


/*----------------------------------------------------------------------*/
/*
 *   cs_detab
 *
 *   Change tabs to spaces in a C string
 *   1996 Aug 26 JCM
 *   Args:
 *	buf		(i/o)	string to remove tabs from
 *
 */

void cs_detab( char* buf )
{
 while( *buf != '\0' ) {
  if ( *buf == '\t' ) *buf = ' ';
  buf++;
 }
}


/*----------------------------------------------------------------------*/
/*
 *   cs_decmt
 *
 *   Remove comments from a string by placing a null at the first comment character.
 *   Goes through the string changing occurrences of esc-cmt to cmt until
 *   a null is encountered or until a cmt without an esc is encountered.
 * 
 *   1996 Aug 26 JCM
 *   
 *   Args:
 *	buf		(i/o) 	String to remove comments from
 *	esc		(i)	Escape character
 *	cmt		(i)	Comment character
 *
 */


void cs_decmt( char* buf, const char esc, const char cmt )
{
 while ( *buf != '\0' ) {
  if ( buf[0] == esc && buf[1] == cmt ) {
   cs_copy( buf+1, buf );   /* Move everything back one byte */
  }  
  else if ( *buf == cmt ) {
   *buf = '\0';
   return;
  }
  buf++;
 }
}


/*----------------------------------------------------------------------*/
/*   cs_compress_spaces
 *  
 *   Copy source string to destination, while
 *   reducing multiple spaces to a single space.
 *   1996 Aug 26
 *
 *   Args:
 *	source		(i)	Source string
 *	dest		(o)	Output string
 */
void cs_compress_spaces( const char* source, char* dest )
{
 while ( ( *dest = *source++ ) != '\0' ) 
 {
  if ( *dest++ == ' ' ) 
  {
   while (*source == ' ' ) source++; 
  }
 }
}


/*----------------------------------------------------------------------*/
/*
 *+cs_upto
 * Copy source to dest up to first occurrence of character targ
 */
char* cs_upto( const char* source, const char targ, char* cs_upto_buf )
{
 char* ptr;
 ptr = cs_upto_buf;
 while ( *source != '\0' && *source != targ ) {
  *ptr++ = *source++;
 }
 *ptr = '\0';
 return( cs_upto_buf );
}




void string_replace( char* cbuf, const char* old, const char* new )
{
 integer lold;
 integer lnew;
 integer lbuf;
 t_buf buf;
 integer i;
 char* iptr;
 char* optr;
 char* iend;
 lold = strlen( old );
 lnew = strlen( new );
 lbuf = strlen( cbuf );
 iend = cbuf + lbuf - 1;
 i = 1;
 iptr = cbuf;
 optr = buf;
 while ( i > 0 ) { 
  i = cs_index( iptr, old );  /* Location of OLD */
  if ( i > 0 ) {
   cs_get_ss( iptr, optr, 1, i-1 );
   iptr += i-1;
   optr += i-1;
   cs_put_ss( new, optr, 1, lnew );
   iptr += lold;
   optr += lnew;
   if ( iptr > iend ) i = 0;
  }
 }
 if ( iptr < iend ) {
  cs_copy( iptr, optr );
 } else {
  *optr = '\0';
 }
 cs_copy( buf, cbuf );
}
