
#include <stdio.h>
#include "jcmlib.h"
/*======================================================================*/
/*
 * Debug routines
 *
 * Static storage:
 *	debug_val		Debug level, 0 = no debug
 *
 */


#define NPKG 20
static integer npkg  = 0;
static integer global_dbg = 0;
static integer pkglev[NPKG];
static char** pkgnam;
static logical trace = FALSE;
#define DBG_TREESIZE 20
static integer ntree = 0;
static t_word tree_rtn[DBG_TREESIZE];
static integer tree_num[DBG_TREESIZE];

void dbg_setpkg( const char* pkg, const integer level )
{
 card tbuf;
 integer num;
 if ( cs_eq( pkg, "GLOBAL" ) ) {
  global_dbg = level;
  if ( level > 0 ) {
   sprintf( tbuf, "[*] Debug level set to [*%ld]", (long)level );
   jcm_err_msg( tbuf );
  }
 } else if ( cs_eq( pkg, "ALL" ) ) {
  for ( num = 0; num < npkg; num++ ) {
   pkglev[num] = level;
  }
 } else {
  num = dbg_addpkg( pkg );
  pkglev[num] = level;
 }
}


void dbg_set( const integer level )
{
 dbg_setpkg( "GLOBAL", level );
}

void dbg_listpkg( void )
{
 integer i;
 card buf;
 jcm_err_msg( "[*] DEBUG PACKAGES ACTIVE" );
 sprintf( buf, "%-32s%s\n", "*Package*", "Level" );
 jcm_err_msg( buf );
 sprintf( buf, "%-32s[*%ld]", "GLOBAL", (long)global_dbg );
 jcm_err_msg( buf );
 for ( i = 0; i < npkg; i++ ) {
  sprintf( buf, "%-32s[*%ld]", pkgnam[i], (long)pkglev[i] );
  jcm_err_msg( buf );
 }

}


void dbg_trace( const logical q )
{
 trace = q;
}


void dbg_push( const char* package, const char* routine )
{
 if ( ntree < DBG_TREESIZE ) {
  ntree++;
  tree_num[ntree-1] = dbg_addpkg( package );
  cs_copy( routine, tree_rtn[ntree-1]);
  if ( trace ) dbg_disptree();
 }
}


void dbg_pop( void )
{
 ntree--;
 if ( trace ) dbg_disptree();
}


void dbg_disptree( void )
{
 integer i;
 card buf;
 char* ptr;
 integer siz = CSIZE;

 cs_copy( "[*->] ", buf );
 ptr = buf+6;
 for ( i = 0; i < ntree; i++ ) {
  cs_put_token( tree_rtn[i], &ptr, '/', &siz );
  if ( !siz ) {
   jcm_err_msg( buf );
   sprintf( buf, "%-6s", " " );
   ptr = buf+6;
   siz = CSIZE-6;
   cs_put_token( tree_rtn[i], &ptr, '/', &siz );
  }
 }
 *ptr-- = '\0';
 if ( *ptr == '/' ) *ptr = ' ';
 jcm_err_msg( buf );
}


void dbg_disppkg( void )
{
 t_buf buf;
 sprintf( buf, "%-32s%s", pkgnam[tree_num[ntree-1]], tree_rtn[ntree-1] );
 jcm_err_msg( buf );
}


/*----------------------------------------------------------------------*/
/*
 * debug
 *
 * Set debug level
 * 1996 Aug 26 JCM
 * Args
 * 	level		(i)  	debug level
 */


/*----------------------------------------------------------------------*/
/*
 * dlev
 *
 * Test debug level; return true if debug level is greater than or equal to argument.
 * Args
 * 	level		(i)  	debug level
 *
 */

logical dbg_level( const integer level )
{
 return( level <= dbg_get() );
}


integer dbg_get( void )
{
 return( max_i( global_dbg, pkglev[tree_num[ntree-1]]) );
}


void dmsg_tag( const integer level, char* buf )
{
 sprintf( buf, "[*%ld]", (long)level );
}

void dmsg( const integer level, const char* buf )
{
 t_buf tbuf;
 t_word tmp;
 if ( level <= dbg_get() ) {
  dmsg_tag( level, tmp );
  sprintf( tbuf, "%-6s%s", tmp, buf );
  jcm_err_msg( tbuf );
 }
}

void dmsg1( const integer level, const char* buf, const char* buf2 )
{
 t_buf tbuf;
 t_word tmp;
 if ( level <= dbg_get() ) {
  dmsg_tag( level, tmp );
  sprintf( tbuf, "%-6s%s%s", tmp, buf, buf2 );
  jcm_err_msg( tbuf );
 }
}


integer dbg_addpkg( const char* pkg )
{
 integer n;
 integer num;
 n = npkg;
 if ( npkg == 0 || pkgnam == NULL ) pkgnam = alloc_array_c( WSIZE, NPKG );
 num = cs_list_add( pkgnam, &npkg, DBG_TREESIZE, pkg );
 if ( npkg > n ) pkglev[npkg-1] = 0;
 return( num-1 );
}



void dbg_fmttree( char* buf, const integer m )
{
 integer siz = m;
 char* ptr;
 integer i;
 if ( ntree > 0 ) {
  cs_copy( "[*->] ", buf );
  ptr = buf + 6;
  siz -= 6;
  for ( i = 0; i < ntree; i++ ) {
   if (!siz ) cs_put_token( tree_rtn[i], &ptr, '/', &siz );
  }
  *ptr = '\0';
 } else {
  cs_copy( " ", buf );
 }
}
