/* Program Control Characters */

#include "jcmlib.h"
#include "jcmp.h"
#include "esc_types.h"



struct esc_struct {
 char echar;
 char edefault;
 char name[8];
};



static struct esc_struct esc_data[ESC_NTYPES] = {
 { '\\', '\\', "ESC" },
 { '(', '(', "LPAR" },
 { ')', ')', "RPAR" },
 { ',', ',', "WORD" },
 { ';', ';', "STRING" },
 { '$', '$', "SYS" },
 { '$', '$', "CONT" },
 { '#', '#', "CMT" },
 { '$', '$', "MACRO" },
 { ' ', ' ', "ARG" }
} ;


void esc_init( void )
{
 integer type;
 for ( type = 0; type < ESC_NTYPES; type++ ) {
  esc_data[type].echar  = esc_data[type].edefault;
 }
}

void esc_get_struct( integer type, char* echar, char* edefault, char* name )
{ 
 /* Return the values in the esc_struct for this control character type. */

 *echar = esc_data[type].echar;
 *edefault = esc_data[type].edefault;
 cs_copy( esc_data[type].name, name );
}

/*----------------------------------------------------------------------*/
/*
 * esc_get_char
 *
 */

char* esc_get_char( const integer type )
{
 return( &esc_data[type].echar );
}

char esc_special_char( const integer type )
{
 return( esc_data[type].echar );
}


void esc_reset( const char* name, const char val )
{
 integer type;
 for ( type = 0; type < ESC_NTYPES; type++ ) {
  if ( cs_uceq( name, esc_data[type].name ) ) esc_data[type].echar = val;
 }
}


char esc_get( const char* name )
{
 integer type;
 for ( type = 0; type < ESC_NTYPES; type++ ) {
  if ( cs_eq( name, esc_data[type].name ) ) return( esc_data[type].echar );
 }
 jcm_err_msg1( "ESC: unknown keyword ", name );
 return( ' ' );
}
