#include "jcmlib.h"

/*
 *   The routines in this file are implementations of the
 *   Fortran 90 array intrinsics, or at least the functionality
 *  that is easy to carry over into C.
 */

integer maxval_i( 
 const integer* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 integer xmin;
 integer xmax;
 integer imin;
 integer imax;
 ar_minmax_i( x, n, &xmin, &xmax, &imin, &imax );
 return( xmax ); 
}

short maxval_s( 
 const short* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 short xmin;
 short xmax;
 integer imin;
 integer imax;
 ar_minmax_s( x, n, &xmin, &xmax, &imin, &imax );
 return( xmax ); 
}

real maxval_r( 
 const real* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 real xmin;
 real xmax;
 integer imin;
 integer imax;
 ar_minmax_r( x, n, &xmin, &xmax, &imin, &imax );
 return( xmax ); 
}

double maxval_d( 
 const double* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 double xmin;
 double xmax;
 integer imin;
 integer imax;
 ar_minmax_d( x, n, &xmin, &xmax, &imin, &imax );
 return( xmax ); 
}

/*
 * --------------------------------------------------------------------------------
 */

integer minval_i( 
 const integer* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 integer xmin;
 integer xmax;
 integer imin;
 integer imax;
 ar_minmax_i( x, n, &xmin, &xmax, &imin, &imax );
 return( xmin ); 
}

short minval_s( 
 const short* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 short xmin;
 short xmax;
 integer imin;
 integer imax;
 ar_minmax_s( x, n, &xmin, &xmax, &imin, &imax );
 return( xmin ); 
}

real minval_r( 
const  real* x,		/* (i) Array */
const  integer  n			/* (i) Array size */
 )
{
 real xmin;
 real xmax;
 integer imin;
 integer imax;
 ar_minmax_r( x, n, &xmin, &xmax, &imin, &imax );
 return( xmin ); 
}

double minval_d( 
 const double* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 double xmin;
 double xmax;
 integer imin;
 integer imax;
 ar_minmax_d( x, n, &xmin, &xmax, &imin, &imax );
 return( xmin ); 
}

/*
 * --------------------------------------------------------------------------------
 */

integer minloc_i( 
 const integer* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 integer xmin;
 integer xmax;
 integer imin;
 integer imax;
 ar_minmax_i( x, n, &xmin, &xmax, &imin, &imax );
 return( imin ); 
}

integer minloc_s( 
 const short* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 short xmin;
 short xmax;
 integer imin;
 integer imax;
 ar_minmax_s( x, n, &xmin, &xmax, &imin, &imax );
 return( imin ); 
}

integer minloc_r( 
 const real* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 real xmin;
 real xmax;
 integer imin;
 integer imax;
 ar_minmax_r( x, n, &xmin, &xmax, &imin, &imax );
 return( imin ); 
}

integer minloc_d( 
 const double* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 double xmin;
 double xmax;
 integer imin;
 integer imax;
 ar_minmax_d( x, n, &xmin, &xmax, &imin, &imax );
 return( imin ); 
}


/*
 * --------------------------------------------------------------------------------
 */

integer maxloc_i( 
 const integer* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 integer xmin;
 integer xmax;
 integer imin;
 integer imax;
 ar_minmax_i( x, n, &xmin, &xmax, &imin, &imax );
 return( imax ); 
}

integer maxloc_s( 
 const short* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 short xmin;
 short xmax;
 integer imin;
 integer imax;
 ar_minmax_s( x, n, &xmin, &xmax, &imin, &imax );
 return( imax ); 
}

integer maxloc_r( 
 const real* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 real xmin;
 real xmax;
 integer imin;
 integer imax;
 ar_minmax_r( x, n, &xmin, &xmax, &imin, &imax );
 return( imax ); 
}

integer maxloc_d( 
 const double* x,		/* (i) Array */
 const integer  n			/* (i) Array size */
 )
{
 double xmin;
 double xmax;
 integer imin;
 integer imax;
 ar_minmax_d( x, n, &xmin, &xmax, &imin, &imax );
 return( imax ); 
}

/*
 * --------------------------------------------------------------------------------
 */

/*
 * --------------------------------------------------------------------------------
 */

void ar_minmax_d( 
 const double* x,		/* (i) Array */
 const integer  n,		/* (i) Array size */
 double* xmin,		/* (o) Min value */
 double* xmax,		/* (o) Max value */
 integer*  imin,		/* (o) C Index of min value */
 integer*  imax		/* (o) C Index of max value */
)
{
 integer i;
 *xmin = x[0];
 *xmax = *xmin;
 *imin = 0;
 *imax = 0;
 for ( i = 1; i < n; i++ )
 {
  if( x[i] < *xmin ) {
   *xmin = x[i];
   *imin = i;
  }
  else if ( x[i] > *xmax ) 
  {
   *xmax = x[i];
   *imax = i;
  }
 }
 (*imin)++; /* Fortran index */
 (*imax)++;
}

void ar_minmax_r( 
 const real* x,		/* (i) Array */
 const integer  n,		/* (i) Array size */
 real* xmin,		/* (o) Min value */
 real* xmax,		/* (o) Max value */
 integer*  imin,		/* (o) C Index of min value */
 integer*  imax		/* (o) C Index of max value */
)
{
 integer i;
 *xmin = x[0];
 *xmax = *xmin;
 *imin = 0;
 *imax = 0;
 for ( i = 1; i < n; i++ )
 {
  if( x[i] < *xmin ) {
   *xmin = x[i];
   *imin = i;
  }
  else if ( x[i] > *xmax ) 
  {
   *xmax = x[i];
   *imax = i;
  }
 }
 (*imin)++; /* Fortran index */
 (*imax)++;
}


void ar_minmax_i( 
 const integer* x,		/* (i) Array */
 const integer  n,		/* (i) Array size */
 integer* xmin,		/* (o) Min value */
 integer* xmax,		/* (o) Max value */
 integer*  imin,		/* (o) C Index of min value */
 integer*  imax		/* (o) C Index of max value */
)
{
 integer i;
 *xmin = x[0];
 *xmax = *xmin;
 *imin = 0;
 *imax = 0;
 for ( i = 1; i < n; i++ )
 {
  if( x[i] < *xmin ) {
   *xmin = x[i];
   *imin = i;
  }
  else if ( x[i] > *xmax ) 
  {
   *xmax = x[i];
   *imax = i;
  }
 }
 (*imin)++; /* Fortran index */
 (*imax)++;
}



void ar_minmax_s( 
 const short* x,		/* (i) Array */
 const integer  n,		/* (i) Array size */
 short* xmin,		/* (o) Min value */
 short* xmax,		/* (o) Max value */
 integer*  imin,		/* (o) C Index of min value */
 integer*  imax		/* (o) C Index of max value */
)
{
 integer i;
 *xmin = x[0];
 *xmax = *xmin;
 *imin = 0;
 *imax = 0;
 for ( i = 1; i < n; i++ )
 {
  if( x[i] < *xmin ) {
   *xmin = x[i];
   *imin = i;
  }
  else if ( x[i] > *xmax ) 
  {
   *xmax = x[i];
   *imax = i;
  }
 }
 (*imin)++; /* Fortran index */
 (*imax)++;
}


/*
 * --------------------------------------------------------------------------------
 */

integer ar_product_i( 
 const integer* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;
 integer product = 1;
 while( n-- ) product *= *a++;
 return( product );
}

short ar_product_s( 
 const short* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;

 short product = 1;
 while( n-- ) product *= *a++;
 return( product );
}


real ar_product_r( 
 const real* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;

 real product = 1;
 while( n-- ) product *= *a++;
 return( product );
}

double ar_product_d( 
 const double* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;

 double product = 1;
 while( n-- ) product *= *a++;
 return( product );
}



/*
 * --------------------------------------------------------------------------------
 */

integer ar_sum_i( 
 const integer* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;

 integer sum = 0;
 while( n-- ) sum += *a++;
 return( sum );
}

short ar_sum_s( 
 const short* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;

 short sum = 0;
 while( n-- ) sum += *a++;
 return( sum );
}


real ar_sum_r( 
 const real* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;

 real sum = 0;
 while( n-- ) sum += *a++;
 return( sum );
}

double ar_sum_d( 
 const double* a,	/* (i) Array */
 const integer m 		/* (i) Array size */
 )
{
 integer n = m;

 double sum = 0;
 while( n-- ) sum += *a++;
 return( sum );
}


/*
 * --------------------------------------------------------------------------------
 */

integer merge_i( 
 const integer i,
 const integer j,
 const logical mask
)
{
 return( mask ? i : j );
}


short merge_s( 
 const short i,
 const short j,
 const logical mask
)
{
 return( mask ? i : j );
}


real merge_r( 
 const real i,
 const real j,
 const logical mask
)
{
 return( mask ? i : j );
}

double merge_d( 
 const double i,
 const double j,
 const logical mask
)
{
 return( mask ? i : j );
}



char * merge_c( 
 char * i,
 char * j,
 const logical mask
)
{
 return( mask ? i : j );
}


void ar_merge_s( 
 const short* true,
 const short* false,
 short* merge,
 const logical* mask,
 const integer size
)
{
 integer i;
 for ( i = 0; i < size; i++ ) 
 {
  merge[i] = mask[i] ? true[i] : false[i];
 }
}

void ar_merge_i( 
 const integer* true,
 const integer* false,
 integer* merge,
 const logical* mask,
 const integer size
)
{
 integer i;
 for ( i = 0; i < size; i++ ) 
 {
  merge[i] = mask[i] ? true[i] : false[i];
 }
}

void ar_merge_d( 
 const double* true,
 const double* false,
 double* merge,
 const logical* mask,
 const integer size
)
{
 integer i;
 for ( i = 0; i < size; i++ ) 
 {
  merge[i] = mask[i] ? true[i] : false[i];
 }
}

void ar_merge_r( 
 const real* true,
 const real* false,
 real* merge,
 const logical* mask,
 const integer size
)
{
 integer i;
 for ( i = 0; i < size; i++ ) 
 {
  merge[i] = mask[i] ? true[i] : false[i];
 }
}

void ar_merge_cp( 
 char** true,
 char** false,
 char** merge,
 const logical* mask,
 const integer size
)
{
 integer i;
 for ( i = 0; i < size; i++ ) 
 { 
  merge[i] = mask[i] ? true[i] : false[i];
 }
}

void ar_merge_c( 
 const char** true,
 const char** false,
 char** merge,
 const logical* mask,
 const integer size
)
{
 integer i;
 const char* ptr;
 for ( i = 0; i < size; i++ ) 
 { 
  ptr = mask[i] ? true[i] : false[i];
  cs_copy( ptr, merge[i] );
 }
}



/*
 * --------------------------------------------------------------------------------
 *  The pack and unpack routines use a packing mask as follows:
 *
 *   Suppose x =  4 3 8 2 20 0 1
 *         mask = T T F T F  F T
 *  Then pack( x, mask, y, 7 ) = 4 and y = 4 3 2 1  (corresponding to the true
 *   values of mask )
 *  Now 
 *    unpack( y, mask, z, 99, 7 ) = 4 and z = 4 3 99 2 99 99 1
 *  with the false bits of mask filled in with the fill value.
 */

integer ar_pack_s(
 const short* x,		/* (i) Array */
 const logical* mask,		/* (i) Packing mask */
 short* y,		/* (o) Packed array */
 const integer n			/* (i) Array size */
)
{
 integer m;			/* (o) Packed array size */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )     y[m++] = x[i];
 }
 return( m );
}

integer ar_pack_i(
 const integer* x,		/* (i) Array */
 const logical* mask,		/* (i) Packing mask */
 integer* y,		/* (o) Packed array */
 const integer n			/* (i) Array size */
)
{
 integer m;			/* (o) Packed array size */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )    y[m++] = x[i];
 }
 return( m );
}

integer ar_pack_r(
 const real* x,		/* (i) Array */
 const logical* mask,		/* (i) Packing mask */
 real* y,		/* (o) Packed array */
 const integer n			/* (i) Array size */
)
{
 integer m;			/* (o) Packed array size */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )   y[m++] = x[i];
 }
 return( m );
}

integer ar_pack_d(
 const double* x,		/* (i) Array */
 const logical* mask,		/* (i) Packing mask */
 double* y,		/* (o) Packed array */
 const integer n			/* (i) Array size */
)
{
 integer m;			/* (o) Packed array size */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )    y[m++] = x[i];
 }
 return( m );
}


/*
 *
 * The ar pack cp routine, in this implementation,
 * returns pointers to the same storage as the input.
 */

integer ar_pack_cp(
 char** x,		/* (i) Array */
 const logical* mask,		/* (i) Packing mask */
 char** y,		/* (o) Packed array */
 const integer n			/* (i) Array size */
)
{
 integer m;			/* (o) Packed array size */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )  y[m++] = x[i];
 }
 return( m );
}

integer ar_pack_c(
 const char** x,		/* (i) Array */
 const logical* mask,		/* (i) Packing mask */
 char** y,		/* (o) Packed array */
 const integer n			/* (i) Array size */
)
{
 integer m;			/* (o) Packed array size */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] ) {
   cs_copy( x[i], y[m] ); 
   m++;
  }
 }
 return( m );
}




integer ar_unpack_s(
 const short* y,		/* (i) Packed array */
 const logical* mask,		/* (i) Packing mask */
 short* x,		/* (o) Output array */
 const short  x0, 		/* (i) Fill value */
 const integer n			/* (i) Array size for output */
)
{
 integer m;			/* (o) Packed array size: number of nondefault elements */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )  {
   x[i] = y[m++]; 
  } else {
   x[i] = x0;
  }
 }
 return( m );
}

integer ar_unpack_i(
 const integer* y,		/* (i) Packed array */
 const logical* mask,		/* (i) Packing mask */
 integer* x,		/* (o) Output array */
 const integer  x0, 		/* (i) Fill value */
 const integer n			/* (i) Array size for output */
)
{
 integer m;			/* (o) Packed array size: number of nondefault elements */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )  {
   x[i] = y[m++]; 
  } else {
   x[i] = x0;
  }
 }
 return( m );
}

integer ar_unpack_r(
 const real* y,		/* (i) Packed array */
 const logical* mask,		/* (i) Packing mask */
 real* x,		/* (o) Output array */
 const real  x0, 		/* (i) Fill value */
 const integer n			/* (i) Array size for output */
)
{
 integer m;			/* (o) Packed array size: number of nondefault elements */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )  {
   x[i] = y[m++]; 
  } else {
   x[i] = x0;
  }
 }
 return( m );
}

integer ar_unpack_d(
 const double* y,		/* (i) Packed array */
 const logical* mask,		/* (i) Packing mask */
 double* x,		/* (o) Output array */
 const double  x0, 		/* (i) Fill value */
 const integer n			/* (i) Array size for output */
)
{
 integer m;			/* (o) Packed array size: number of nondefault elements */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )  {
   x[i] = y[m++]; 
  } else {
   x[i] = x0;
  }
 }
 return( m );
}


integer ar_unpack_cp(
 char** y,		/* (i) Packed array */
 const logical* mask,		/* (i) Packing mask */
 char** x,		/* (o) Output array */
 char*  x0, 		/* (i) Fill value */
 const integer n			/* (i) Array size for output */
)
{
 integer m;			/* (o) Packed array size: number of nondefault elements */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )  {
   x[i] = y[m++]; 
  } else {
   x[i] = x0;
  }
 }
 return( m );
}



integer ar_unpack_c(
 const char** y,		/* (i) Packed array */
 const logical* mask,		/* (i) Packing mask */
 char** x,		/* (o) Output array */
 const char*  x0, 		/* (i) Fill value */
 const integer n			/* (i) Array size for output */
)
{
 integer m;			/* (o) Packed array size: number of nondefault elements */
 integer i;
 m = 0;
 for ( i = 0; i < n; i++ ) {
  if ( mask[i] )  {
   cs_copy( y[m], x[i] );
   m++;
  } else {
   cs_copy( x0, x[i] );
  }
 }
 return( m );
}



/*
 * CSHIFT: circular shift array left 
 * EOSHIFT: End off shift
 *    If  x =  4 3 28 11 0 2
 *    Then cshift( x, y, 6, +4 ) gives
 *        y =  0 2 4 3 28 11
 *    while eoshift( x, y, 6, +4, 99 ) gives
 *        y =  0 2 99 99 99 99
 */

void ar_cshift_s( 
 const short* x,
 short* y,
 const integer n,
 const integer shift
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = ( i + shift ) % n;
  y[i] = x[j];
 }
}


void ar_cshift_i( 
 const integer* x,
 integer* y,
 const integer n,
 const integer shift
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = ( i + shift ) % n;
  y[i] = x[j];
 }
}

void ar_cshift_r( 
 const real* x,
 real* y,
 const integer n,
 const integer shift
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = ( i + shift ) % n;
  y[i] = x[j];
 }
}


void ar_cshift_d( 
 const double* x,
 double* y,
 const integer n,
 const integer shift
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = ( i + shift ) % n;
  y[i] = x[j];
 }
}

void ar_cshift_cp( 
 char** x,
 char** y,
 const integer n,
 const integer shift
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = ( i + shift ) % n;
  y[i] = x[j];
 }
}


void ar_cshift_c( 
 const  char** x,
 char** y,
 const integer n,
 const integer shift
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = ( i + shift ) % n;
  cs_copy( x[j], y[i] );
 }
}




void ar_eoshift_s( 
 const short* x,
 short* y,
 const integer n,
 const integer shift,
 const short fill
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = i + shift;
  if ( j >= 0 && j < n ) {
   y[i] = x[j];
  } else {
   y[i] = fill;
  }
 }
}


void ar_eoshift_i( 
 const integer* x,
 integer* y,
 const integer n,
 const integer shift,
 const integer fill

)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = i + shift;
  if ( j >= 0 && j < n ) {
   y[i] = x[j];
  } else {
   y[i] = fill;
  }
 }
}

void ar_eoshift_r( 
 const real* x,
 real* y,
 const integer n,
 const integer shift,
 const real fill
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = i + shift;
  if ( j >= 0 && j < n ) {
   y[i] = x[j];
  } else {
   y[i] = fill;
  }
 }
}


void ar_eoshift_d( 
 const double* x,
 double* y,
 const integer n,
 const integer shift,
 const double fill

)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = i + shift;
  if ( j >= 0 && j < n ) {
   y[i] = x[j];
  } else {
   y[i] = fill;
  }
 }
}

void ar_eoshift_cp( 
 char** x,
 char** y,
 const integer n,
 const integer shift,
 char* fill

)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = i + shift;
  if ( j >= 0 && j < n ) {
   y[i] = x[j];
  } else {
   y[i] = fill;
  }
 }
}

void ar_eoshift_c( 
 const char** x,
 char** y,
 const integer n,
 const integer shift,
 const char* fill
)
{
 integer i;
 integer j;
 for ( i = 0; i < n; i++ ) {
  j = i + shift;
  if ( j >= 0 && j < n ) {
   cs_copy( x[j], y[i] );
  } else {
   cs_copy( fill, y[i] );
  }
 }
}



/*
 *  Transpose a matrix
 */

void ar_transpose_s( 
  const short** x,
  short** y,
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer row;
 integer col;
 for ( row = xmin; row <= xmax; row++ ) {
  for ( col = ymin; col <= ymax; col++ ) {
   y[col][row] = x[row][col];
  }
 }
}

void ar_transpose_i( 
  const integer** x,
  integer** y,
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer row;
 integer col;
 for ( row = xmin; row <= xmax; row++ ) {
  for ( col = ymin; col <= ymax; col++ ) {
   y[col][row] = x[row][col];
  }
 }
}

void ar_transpose_r( 
  const real** x,
  real** y,
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer row;
 integer col;
 for ( row = xmin; row <= xmax; row++ ) {
  for ( col = ymin; col <= ymax; col++ ) {
   y[col][row] = x[row][col];
  }
 }
}

void ar_transpose_d( 
  const double** x,
  double** y,
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer row;
 integer col;
 for ( row = xmin; row <= xmax; row++ ) {
  for ( col = ymin; col <= ymax; col++ ) {
   y[col][row] = x[row][col];
  }
 }
}


void ar_transpose_q( 
  const logical** x,
  logical** y,
  const integer xmin,
  const integer xmax,
  const integer ymin,
  const integer ymax
)
{
 integer row;
 integer col;
 for ( row = xmin; row <= xmax; row++ ) {
  for ( col = ymin; col <= ymax; col++ ) {
   y[col][row] = x[row][col];
  }
 }
}



/*
 *   Copy between different precisions
 */

void ar_copy_si( 
  const short* x,
  integer* y,
  const integer m
)
{
 integer n = m;

 while( n-- ) {
  *y++ = (integer) *x++;
 }
}

void ar_copy_is(
 const  integer* x,
 short* y,
 const integer m
)
{
 integer n = m;
 while( n-- ) {
  *y++ = (short) *x++;
 }
}

void ar_copy_rd( 
  const real* x,
  double* y,
  const integer m
)
{
 integer n = m;

 while( n-- ) {
   *y++ = (double) *x++;
 }
}

void ar_copy_dr(
 const double* x,
 real* y,
 const integer m
)
{
 integer n = m;
 while( n-- ) {
   *y++ = (real) *x++;
 }
}



