/*======================================================================*/
/*
 * Fortran to C string routines. These routines convert between
 * null terminated and space-terminated, fixed-length strings.
 */
/*======================================================================*/
#include <string.h>
#include <stdlib.h>
#include "jcmlib.h"


/*
 *  Copy Fortran string to allocated C string
 */
char* fc_AllocString( char* fstring, long maxlen )
{
 char* cstring;
 long size;
 size = fs_ends( fstring, maxlen );
 cstring = (char*)malloc( (maxlen+1)*sizeof(char) );
 cs_copy_siz( fstring, cstring, size );
 cstring[size] = '\0';
 return( cstring );
}

/*
 *  Copy a C string directly into a Fortran string array
 */
void fc_StringCopyItem( char* cstring, char* fstring, long maxlen, long i )
{
 char* ptr;
 ptr = fstring + maxlen * i;
 cs_copy_cf( cstring, ptr, maxlen ); 
}

/*
 *  Extract a Fortran string array element into a C null terminated string.
 */
void fc_StringPutItem( char* fstring, long maxlen, long i, char* cstring )
{
 char* ptr;
 ptr = fstring + maxlen * i;
 cs_copy_fc( ptr, cstring, maxlen, maxlen ); 
}

void fc_FreeString( char* cstring, char* fstring, long maxlen )
{
 cs_copy_cf( cstring, fstring, maxlen ); 
 free( cstring );
}

/*
 *  Allocate an array of C strings and copy an array of Fortran
 *  strings to it (if the array is not null).
 */

char** fc_AllocStringArray( char* array, long len, long n )
{
 long i;
 char* item;
 char** p;
 p = alloc_array_c( len+1, n );
 if ( array ) {
  for ( i = 0; i < n; i++ ) {
   item = array + i * len;
   cs_copy_fc( item, p[i], len, len );
  }
 }
 return( p );
}



/*
 *  Copy an array of C strings to an array of equal length Fortran strings.
 *  n is the array size, rlen is the Fortran string size.
 */
void fc_FreeStringArray( char** result, char* value, long rlen, long n )
{
 long i;
 char* ptr;
 if ( value != NULL ) { 
  for ( i = 0; i < n; i++ ) {
   ptr = value + rlen * i;
   cs_copy_cf( result[i], ptr, rlen ); 
  }
 }
 free_array_c( result );
} 

void cs_copy_cf( char* cname, char* fname,  long maxlen )
{

 while( *cname != '\0' && maxlen-- ) *fname++ = *cname++; 
/* Blank fill */
 while( maxlen-- ) *fname++ = ' '; 
}



void cs_copy_fc( char* fname, char* cname, long fsiz, long siz )
{
 long rsiz;
 rsiz = ( fsiz < siz ) ?  fsiz : siz;
 cs_copy_siz( fname, cname, rsiz );
}
