#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "fio_file.h"
#include "fio.h"

integer fio_get_lineno( integer fid )
{
 return( fio_file_ptr( fid )->line );
}

FILE* fio_file_fptr( integer fid )
{

/*
 * This routine uses the fio_file structure definition.
 */

 if ( fid > 0 ) {
  return( fio_file_ptr( fid )->fptr );
 } else {
  return( NULL );
 }
}

/*======================================================================*/
/*
 *  External file routines
 */
/*======================================================================*/

/*----------------------------------------------------------------------*/
/*
 *  fio_test_eof
 *
 *  Test if file status to EOF
 *
 */

logical fio_test_eof( integer fid )
{
/*
 * Set EOF status
 */
 if ( fid > 0 ) 
  return ( fio_file_ptr( fid )->status == FILE_EOF );
 else
  return ( FALSE );
}

/*
 *--------------------------------------------------------------------------------
 */
/*
 *  Test whether the file exists.
 */
logical fio_inquire( char* name )
{
 struct stat stat_var;
 return( stat( name, &stat_var ) == 0 );
}

/*
 *--------------------------------------------------------------------------------
 */

/*
 *   Test whether you have access permission to the file.
 */
integer fio_access( const char* name )
{
 struct stat stat_var;
 int status;
 uid_t uid, fuid;
 gid_t gid, fgid;
 mode_t mode;

 logical r, w;

 if ( !name ) return( FIO_NOFILE );

 status = stat( name, &stat_var );
 if ( status == 0 ) {
/*  Is it world accessible? */
  mode = stat_var.st_mode;
  r = ( mode & S_IROTH );
  w = ( mode & S_IWOTH );
  if ( !r || !w ) {
/* Is it group accessible for your group? */
   fgid = stat_var.st_gid;
   gid = getegid();
   if ( gid == fgid ) {
    r = r || ( mode & S_IRGRP );
    w = w || ( mode & S_IWGRP );
    if ( !r || !w ) { 
/*  Is it accessible for your effective UID? */
     uid = geteuid();
     fuid= stat_var.st_uid;
     if ( uid == fuid ) {
      r =  r || ( mode & S_IRUSR );
      w =  w || ( mode & S_IWUSR );
     }
    }
   }
  }
/* OK, now we know whether you can read or write to it. */
  if ( r && w ) {
   return( FIO_READWRITE );
  } else if ( r ) {
   return( FIO_READONLY );
  } else {
   return( FIO_NOPERM );
  }
 } else {
  return( FIO_NOFILE );
 }
} 
/*----------------------------------------------------------------------*/
/*
 *  fio_clear_eof
 *
 *  Set file status to Open
 *
 */

void fio_clear_eof( integer fid )
{
/*
 * Set EOF status
 */
 FILE* fptr;
 if ( fid > 0 ) {
  fio_file_ptr( fid )->status = FILE_OPEN;
  fptr = fio_file_fptr( fid );
  if ( fptr != NULL ) clearerr( fptr );
 }
}
 
 
/*----------------------------------------------------------------------*/
/*
 * fio_get_name
 *
 * Return the name of the file
 *
 */

void fio_get_name( integer fid, char* name, const integer maxlen )
{

/*
 * This routine uses the fio_file structure definition.
 */
 if ( fid > 0 ) {
  cs_copy_siz( fio_file_ptr( fid )->Name, name, maxlen );
 } else {
  cs_copy( " ", name );
 }
}

/*
 *--------------------------------------------------------------------------------
 */

integer fio_get_recl( integer fid )
{
 if ( fid > 0 ) {
  return( fio_file_ptr( fid )->recl );
 } else {
  return( 0 );
 }
}
/*
 *--------------------------------------------------------------------------------
 */

void fio_set_recl( integer fid, integer recl )
{
 if ( fid > 0 ) {
  fio_file_ptr(fid)->recl = recl;
 }
}
 
/*----------------------------------------------------------------------*/
void fio_set_type( integer fid, char* type )
{
 if ( fid > 0 ) {
  cs_copy( type, fio_file_ptr( fid )->type );
 }
}

/*
 *--------------------------------------------------------------------------------
 */
void fio_get_type( integer fid, char* type, const integer maxlen )
{

/*
 * This routine uses the fio_file structure definition.
 */

 if ( fid > 0 ) {
  cs_copy_siz( fio_file_ptr( fid )->type, type, maxlen );
 } else {
  cs_copy( " ", type );
 }
}

/*
 *--------------------------------------------------------------------------------
 */
integer fio_get_pos( integer fid )
{
 if ( fid > 0 ) {
  return( fio_file_ptr( fid )->pos );
 } else {
  return( 0 );
 }
}

/*
 *--------------------------------------------------------------------------------
 */

logical fio_write_eol( integer fid )
{
 if( fio_get_pos( fid ) > 0 ) {
  return( fio_write( fid, " " ) );
 } else {
  return( TRUE );
 } 
}
