#include <stdio.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "fio_file.h"
#include "fio.h"
#include "fiodata.h"

void fio_init( void )
{
 cs_copy( " ", io_dir );
}

/*
 *--------------------------------------------------------------------------------
 * Directory handling: the IO_DIR variable
 *
 *--------------------------------------------------------------------------------
 */
void fio_change_dir( char* dir )
{
 t_filename result;
 sys_filepath( io_dir, dir, " ", result, FSIZE );
 cs_copy( result, io_dir );
}

void fio_get_dir( char* dir )
{
 cs_copy( io_dir, dir );
}

/*
 *--------------------------------------------------------------------------------
 * RETRY
 *--------------------------------------------------------------------------------
 */
void fio_set_retry( logical q )
{
 retry = q;
}

logical fio_get_retry( void )
{
 return( retry );
}


/*
 *--------------------------------------------------------------------------------
 * Access to JCM FILE parameters
 *--------------------------------------------------------------------------------
 */

integer fio_file_n( void )
{
 return( fio_nfiles );
}



/*
 *
 * fio_file_ptr
 *
 * Return pointer to JCM file structure 
 *
 */

fio_file* fio_file_ptr( integer file )
{

 /* This routine uses fio_file static storage */

 if ( file > 0 ) {
 return( &JCMLIB_Files[file-1] );
 } else {
 return( NULL );
 }
}



/*----------------------------------------------------------------------*/
/*
 * fio_alloc
 *
 * Return next file number
 * Uses: <stdio.h> printf
 */
 
integer fio_alloc( void )
{
 /* This routine uses fio_file static storage */

 integer fid;
 for ( fid = 1; fid <= fio_nfiles; fid++ )
 { 
  if ( fio_file_ptr( fid )->status == FILE_CLOSED ) return( fid );
 }
 if ( fio_nfiles >= MAX_FILES ) {
  printf("JCM FILE Overflow\n");
 } else {
  fio_nfiles++;
 }
 return( fio_nfiles );
}
/*----------------------------------------------------------------------*/

logical fio_get( integer fid, char* buf, integer siz )
{
 logical ios;
 FILE* fptr;

 if ( fid == 0 ) {
  fid = tty_in;
  fptr = stdin;
 } else {
  fptr = fio_file_fptr( fid );
 }
 ios = ( fgets( buf, siz, fptr ) != NULL ); /* TRUE if good */
 if ( ios ) ios = ( ferror( fptr ) == 0 ) && ( feof( fptr ) == 0 );
 if ( ios ) {
   cs_remove_nl( buf );
 } else {
   cs_copy( " ", buf );
 }
 return( ios );
}


logical fio_put( integer fid, const char* buf, logical strip, logical nl )
{
 const integer siz = 1023;
 char buf2[1024];
 const char* ptr;
 logical ios;
 integer pos;
 FILE* fptr;
 integer clen;

 if( fid == 0 ){
  fid = tty_in;
  fptr = stdout;
 } else {
  fptr = fio_file_fptr( fid );
 }
 ios = TRUE;
 ptr = buf;
 clen = cs_ends( buf );
 while( clen > siz && ios ) {
  cs_copy_siz( ptr, buf2, siz );
  ios = ( fputs( buf2, fptr ) >= 0 );
  ptr  += siz;
  clen -= siz;
 }
 pos = fio_terminate_buf( fio_get_pos( fid ), ptr, buf2, strip, nl, siz );
 ios = ( fputs( buf2, fptr ) >= 0 ); /* TRUE if good */
 if ( ios ) ios = ( ferror( fptr ) == 0 ) && ( feof( fptr ) == 0 );
 fio_set_status_pos( fid, ios, pos ); 
 return( ios );
}



logical fio_any_eof( integer mode )
{
 logical eof;
 integer fid;
 eof = FALSE;
 for ( fid = 1; fid <= fio_nfiles; fid++ ) {
  if ( fio_file_ptr( fid )->status == FILE_EOF ) {
   eof = TRUE;
   if ( mode == 0 ) fio_clear_eof( fid );
  }
 }
 return( eof );
}


integer fio_tty_in( void )
{
 return( tty_in );
}

integer fio_tty_out( void )
{
 return( tty_out );
}
integer fio_tty_err( void )
{
 return( tty_err );
}

logical fio_interactive( integer fid )
{
 return( fid == 0 || fid == tty_in || fid == tty_out );
}

