#include <stdio.h>
#include "jcmlib.h"

/*----------------------------------------------------------------------*/
/*+af1_parse_fmt:   Parse a Fortran style format; return C format, data type and field length */
void Fortran_Format_Parse( const char*  Fortran_Format, char* C_Format, char* DataType, int* Field_Length )
{
 const char* ptr;
 int length; /* Field length */
 int dp;  /* Decimal points */
 int dp_pos;
 char type;
 char tmp[80];

 type = *Fortran_Format;
 ptr =   Fortran_Format + 1;
 if ( cs_is_blank( ptr ) ) {
  length = ( type == 'A' ) ? 0 : 1;   /* Special case of 'A' meaning whole string */
  dp = 0;
 } else {
  dp_pos = cs_index( ptr, "." );    /* cs_index returns the 1-based position in the string, or 0 on failure */
  if ( dp_pos == 0 ) {   /* No decimal point */
   dp = 0;
   length = read_i( ptr );   /* read_i is a fancier version of atoi */
  } else {
   cs_get_ss( ptr, tmp, 1, dp_pos-1 );  /* cs_get_ss extracts the substring into tmp (if only C did substrings...) */
   length = read_i( tmp );   /* Read integer up to position of decimal point */
   ptr += dp_pos;
   dp = read_i( ptr );       /* Read integer following decimal point */
  }
 }

 switch( type ) {
  case 'I' :                /*  Note that Fortran makes no distinction of %d and %ld */
   if ( dp > 0 ) {
    sprintf( C_Format, "%%%d.%dld ", length, dp );     /* I4.3 -> %4.3ld; */
   } else {
    sprintf( C_Format, "%%%dld", length );       /* I4 -> %4ld */
   }
   break;
  
  case 'A' :
   if ( length == 0 ) {
    sprintf( C_Format, "%%s" );     /* A -> %s, special case */
   } else { 
    sprintf( C_Format, "%%-%d.%ds ", length, length );     /* A4 -> %-4.4s */
   }
   break;
  
  case 'F' :
   sprintf( C_Format, "%%%d.%df ", length, dp );   /* F4.2 -> %4.2f */
   break;

  case 'E' :
   sprintf( C_Format, "%%%d.%dE ", length , dp );   /* E4.2 -> %4.2E */
   break;

  case 'G' :
   sprintf( C_Format, "%%%d.%dG ", length, dp );    /* G4.2 -> %4.2G */
   break;

  case 'Z' :
   if ( dp > 0 ) {
    sprintf( C_Format, "%%%d.%dX ", length, dp );     /* Z4.2 -> %4.2X;    */
   } else {
    sprintf( C_Format, "%%%dX", length );       /* Z4 -> %4X */
   }
   break;

  default:
   err_set( JCM_ERR_UNKNOWN_FFMT );
   cs_copy( " ", C_Format );
   break; 
  }
 *DataType = type;
 *Field_Length = length;
}


