#include <string.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "gtoken.h"
#include "esc_types.h"


/*----------------------------------------------------------------------*/
/*
 *  cs_put_token
 *
 * JCMLIB library routine (TOKEN)
 * Copy from buf to *dest, appending the sep character or until dest is full up to siz.
 * Update the dest pointer to pointeger to the end of the text
 */

void cs_put_token( const char* buf, char** dest, const char sep, integer* siz )
{
 while ( *siz && *buf != '\0' ) 
 {
  *(*dest)++ = *buf++; (*siz)--;
 }
 if ( *siz && sep ) *(*dest)++ = sep;
 **dest = '\0';
}


/*----------------------------------------------------------------------*/
/*
 *
 * cs_get_token
 *
 * Uses: escproto.h: esc_get_char
 * tokenproto.h: cs_copy_token
 * csproto.h: cs_copy
 * iolib.h: tmsg
 */

void cs_get_token( char** buf, char* token, const integer mode, const integer siz )
{
 char  seps[4]=" ";
 char pars[2];

 if ( !buf || !(*buf) ) { *token = '\0'; return; }
 switch( mode ) {
  case TMODE_WORD: 
   pars[0] = *esc_get_char( LEFT_PAREN );
   pars[1] = *esc_get_char( RIGHT_PAREN );
   seps[0]= ' ';
   seps[1]= *esc_get_char( WORD_SEP );
   seps[2]= *esc_get_char( STRING_SEP );  
   seps[3] = 0;
   cs_copy_token( buf, seps, pars, token, siz );
   break;
  case TMODE_STRING: 
   pars[0] = *esc_get_char( LEFT_PAREN );
   pars[1] = *esc_get_char( RIGHT_PAREN );
   seps[0] = *esc_get_char( STRING_SEP );
   seps[1] = 0;
   cs_copy_token( buf, seps, pars, token, siz );
   break;
  case TMODE_LINE:
   cs_copy( *buf, token );
   *buf = NULL;
   break;
  default:
   tmsg("Bad token mode"); 
   break;
 }
}

/*----------------------------------------------------------------------*/
/*
 * cs_put
 *
 */

void cs_put_cs( const char* token, char** buf, integer* siz )
{
 char sep;
 sep = *esc_get_char( STRING_SEP );
 cs_put_token( token, buf, sep, siz );
}

/*----------------------------------------------------------------------*/
/*
 * cs_putw
 *
 */

void cs_put_c( const char* token, char** buf, integer* siz )
{
 cs_put_token( token, buf, ' ', siz );
}

void cs_put_i( const integer token, char** buf, integer* siz )
{
 t_word word;
 cs_put_token( write_i(token, word, WSIZE), buf, ' ', siz );
}

void cs_put_s( const short token, char** buf, integer* siz )
{
 t_word word;
 cs_put_token( write_s(token, word, WSIZE), buf, ' ', siz );
}

void cs_put_r( const real token, char** buf, integer* siz )
{
 t_word word;
 cs_put_token( write_r(token, word, WSIZE), buf, ' ', siz );
}

void cs_put_d( const double token, char** buf, integer* siz )
{
 t_word word;
 cs_put_token( write_d(token, word, WSIZE), buf, ' ', siz );
}



/*----------------------------------------------------------------------*/
/*
 * cs_puts
 *
 */

void cs_put_cn( const char* token, char** buf, integer* siz )
{
 cs_put_token( token, buf, '\0', siz );
}

/*----------------------------------------------------------------------*/
/*
 * cs_get
 *
 */

void cs_get_cs( char** buf, char* token, const integer siz )
{
 cs_get_token( buf, token, TMODE_STRING, siz );
}

/*----------------------------------------------------------------------*/
/*
 * cs_get_c
 *
 */

void cs_get_c( char** buf, char* token, const integer siz )
{
 cs_get_token( buf, token, TMODE_WORD, siz );
}

/*----------------------------------------------------------------------*/
/*
 * cs_getl
 *
 */

void cs_get_cl( char** buf, char* token, const integer siz )
{
 cs_get_token( buf, token, TMODE_LINE, siz );
}


integer cs_get_i( char** buf )
{
 t_word token;
 cs_get_c( buf, token, WSIZE );
 return( read_i( token ) );
}
short cs_get_s( char** buf )
{
 t_word token;
 cs_get_c( buf, token, WSIZE );
 return( read_s( token ) );
}
logical cs_get_q( char** buf )
{
 t_word token;
 cs_get_c( buf, token, WSIZE );
 return( read_q( token ) );
}
real cs_get_r( char** buf )
{
 t_word token;
 cs_get_c( buf, token, WSIZE );
 return( read_r( token ) );
}
double cs_get_d( char** buf )
{
 t_word token;
 cs_get_c( buf, token, WSIZE );
 return( read_d( token ) );
}

integer cs_getd_i( char** buf, const integer xdefault )
{
 t_word token;
 cs_get_c( buf, token, WSIZE );
 if ( cs_is_blank( token ) ) {
  return( xdefault );
 } else {
  return( read_i( token ) );
 }
}
short cs_getd_s( char** buf, const short xdefault )
{
 t_word token;
 cs_get_c( buf, token, WSIZE );
 if( cs_is_blank( token ) ) {
  return( xdefault );
 } else {
  return( read_s( token ) );
 }
}
real cs_getd_r( char** buf,  const real xdefault )
{
 t_word token;
 cs_get_c( buf, token, WSIZE );
 if ( cs_is_blank( token ) ) {
  return( xdefault ); 
 } else {
  return( read_r( token ) );
 }
}
double cs_getd_d( char** buf, const double xdefault )
{
 t_word token;
 cs_get_c( buf, token, WSIZE );
 if ( cs_is_blank( token )) {
  return( xdefault );
 } else {
  return( read_d( token ) );
 }
}



integer list_parse( char* buf, char** list, const integer nsiz )
{
 integer n;

 n = 0;
 while ( buf != NULL && !cs_is_blank( buf ) ) {
  cs_get_c( &buf, list[n++], nsiz );
 }
 return( n );
}

