#include <stdlib.h>
#include <stdio.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "fio_file.h"
#include "esc_types.h"

/* IO stack for C JCMLIB */

#define IO_DEPTH 20

static integer input_stack[IO_DEPTH];
static integer output_stack[IO_DEPTH];
static integer* input = input_stack;
static integer* output = output_stack;
static integer err_chan = 0;
static integer iofs_ilevel = 1;
static integer iofs_olevel = 1;


void io_ipush( integer fid )
{
 if ( iofs_ilevel < IO_DEPTH ) {
  iofs_ilevel++;
  input++;
 } else {
  jcm_err_msg( "Too many files on input IO stack" );
 }
 if ( fid >= 0 ) {
  tpio_cclear( iofs_ilevel );
  io_in( fid );
 }
}


void io_opush( integer fid )
{
 if ( iofs_olevel < IO_DEPTH ) {
  iofs_olevel++;
  output++;
 } else {
  jcm_err_msg( "Too many files on output IO stack" );
 }
 if ( fid >= 0 ) io_out( fid );
}

void io_ipop( integer mode )
{
 integer fid;
 if ( mode >= 0 ) {
  fid = *input;
  if ( !fio_interactive( fid ) ) fio_close( fid );
  tpio_cclear( iofs_ilevel );
 }
 iofs_ilevel--;
 input--;
 if ( iofs_ilevel <= 0 ) {
  io_iclear();
 }
}

void io_opop( integer mode )
{
 integer fid;
 if ( mode >= 0 ) {
  fid = *output;
  if ( !fio_interactive( fid ) ) fio_close( fid );
 }
 iofs_olevel--;
 output--;
 if ( iofs_olevel <= 0 ) {
  iofs_olevel = 1; 
  io_out( 0 );
  dmsg( 1, "Output to *" );
 }
}


integer iofs_get_level( void )
{
 return( iofs_ilevel );
}

void io_iclear( void )
{
 iofs_ilevel = 1;
 input = input_stack;
 io_in( 0 );
 dmsg( 1, "Input to *" ); 
}
/*----------------------------------------------------------------------*/
/*
 * io_init
 *
 * Reinitialize IO stacks
 *
 */

void io_init( void )
{
 input = input_stack;
 output = output_stack;
 io_in( 0 );
 io_out( 0 );
 io_err( 0 );
}


/*----------------------------------------------------------------------*/
/*
 * io_int
 *
 */

logical io_int( integer mode )
{
 if ( mode == 0 ) {
  return( fio_interactive( *input ));
 } else {
  return( fio_interactive( *output ));
 }
}

/*----------------------------------------------------------------------*/
/*
 * io_in
 *
 */

void io_in( integer file_no )
{
 if ( file_no == 0 ) {
  *input = fio_tty_in();
 } else {
  *input = file_no;
 }
}

/*----------------------------------------------------------------------*/
/*
 * io_out
 *
 */
void io_out( integer file_no )
{
 if ( file_no == 0 ) {
  *output = fio_tty_out();
 } else {
  *output = file_no;
 }
}

void io_err( integer file_no )
{
 if ( file_no == 0 ) {
  err_chan = fio_tty_err();
 } else {
  err_chan = file_no;
 }
}


integer io_chan( integer mode )
{
 if ( mode == 0 ) {
  return( *input );
 } else {
  return( *output );
 }
}


void io_print_state( void )
{
 card buf;

 msg( " " );
 dashline( 40 );
 msg( " JCMLIB Basic IO (BIO) Status " );
 dashline( 40 );
 sprintf( buf, "  Input Channel:  %ld", (long)*input ); msg( buf );
 sprintf( buf, "  Output Channel: %ld", (long)*output ); msg( buf );
 sprintf( buf, "  Error Channel:  %ld", (long)err_chan); msg( buf );
}

/*----------------------------------------------------------------------*/
/*
 * input_test_eof
 * Uses fileproto.h fio_test_eof
 */

logical input_test_eof( void )
{
 return ( fio_test_eof( *input ) );
}

/*----------------------------------------------------------------------*/
/*
 * msg
 *
 */

logical msg( char* buf )
{
 return( fio_write( *output, buf ) );
}

/*----------------------------------------------------------------------*/
/*
 * msgl
 *
 */
logical msgl( char* buf )
{
 return( fio_write_prompt( *output, buf ) );
}

logical msg_endline( void )
{
 return( fio_write_eol( *output ) );
}
/*----------------------------------------------------------------------*/
/*
 * io_input
 *
 */
logical io_input( char* buf, integer siz )
{ 
 return( fio_read( *input, buf, siz ) );
} 

/*----------------------------------------------------------------------*/
/*
 * tmsg
 *
 */
logical tmsg( char* buf )
{
 return( fio_write( 0, buf ) );
}

/*----------------------------------------------------------------------*/
/*
 * tmsgl
 *
 */
logical tmsgl( char* buf )
{
 return( fio_write_prompt( 0, buf ) );
}

void jcm_err_msg( const char* buf )
{
 fio_write( err_chan, buf );
}

void jcm_err_msg1( const char* buf, const char* buf2 )
{ 
 t_buf tbuf;
 sprintf( tbuf, "%s %s", buf, buf2 );
 fio_write( err_chan, tbuf );
}

/*----------------------------------------------------------------------*/
/*
 * tty_input
 *
 */
logical tty_input( char* buf, integer siz )
{
 return( fio_read( 0, buf, siz ) );
} 

void dashline( integer n )
{
 card chunk;
 integer m;
 integer k;
 char* ptr;
 m = n;
 while ( m > CSIZE ) {
  k = CSIZE; ptr = chunk; while( k-- ) *ptr++ = '-'; *ptr = '\0';
  msgl( chunk );
  m -= CSIZE;
 } 
 k = m; ptr = chunk; while( k-- ) *ptr++ = '-'; *ptr = '\0';
 msg( chunk );
}







void iofs_get_entry( char* io_type, integer level, char* buf, integer* fid, char* status )
{
 logical inmode;
 integer stack_top;
 integer istat;

 inmode = *io_type == 'I';
 cs_copy( " ", status ); 
 stack_top = inmode ? iofs_ilevel : iofs_olevel;
 if ( level <= 0 ) {
  cs_copy( ">> Invalid IO stack level <<", buf );
 } else if ( level > stack_top ) {
  cs_copy( ">> Beyond top of stack    <<", buf );
 } else {
  *fid = inmode?  input_stack[level-1] : output_stack[level-1];
  if ( *fid == fio_tty_in() ) {
   cs_copy( "(Interactive stream)", buf );
   cs_copy( "[TTY IN]", status );
  } else if ( *fid == fio_tty_out() ) {
   cs_copy( "(Interactive stream)", buf );
   cs_copy( "[TTY OUT]", status );
  } else if ( *fid == 0 ) { 
   cs_copy( "(No file)", buf );
  } else {
   fio_get_name( *fid, buf, FSIZE );
   istat = fio_status( *fid, 1 );
   if ( istat == FILE_EOF ) {
    cs_copy( "[EOF]", status );
   }  else if ( istat == FILE_OPEN ) { 
    cs_copy( "[Open]", status );
   } else if ( istat == FILE_CLOSED ) {
    cs_copy( "(No file)", status );
   } else {
    cs_copy( "(Bad status)", status ); 
   }

  }
 }
}




void iofs_print_top( char* type, char* pfx )
{
 integer stack_top;
 t_filename filename;
 card status;
 card mbuf;
 t_buf buf;
 integer fid;
 stack_top = iofs_get_level();
 iofs_get_entry( type, stack_top, filename, &fid, status );
 if ( fid > 0 ) {
  sprintf( mbuf, " Unit %ld", (long)fid );
 } else {
  cs_copy( " ", mbuf );
 }
 sprintf( buf, "%s IO stream: %s%s%s", pfx, filename, status, mbuf );
 msg( buf );
}


void iofs_print_state( void )
{
 integer top;
 t_word tmp;
 integer iofs_in_ios;
 t_buf buf;
 top = iofs_get_level();
 if ( iofs_status(1) == FILE_EOF )  {
  iofs_in_ios = -1;
 } else {
  iofs_in_ios = 0;
 }
 if ( iofs_eof(1) ) {
  cs_copy( "set", tmp );
 } else {
  cs_copy( "clear", tmp );
 }
 msg( " " );
 dashline( 40 );
 msg( " JCMLIB IO File Stack (IOFS) Status" );
 dashline( 40 );
 sprintf( buf, "  IOFS Input Stack depth: %-6ld IOS Code: %-6ld EOF Flag: %s", (long)top, (long)iofs_in_ios, tmp );
 msg( buf );
 if ( top > 0 ) {
  iofs_print_stack( "I", top );
 }
 top = iofs_olevel;
 sprintf( buf, "  IOFS Output Stack depth: %-6ld", (long)top );
 msg( buf );
 if ( top > 0 ) {
  iofs_print_stack( "O", (long)top );
 }
 msg( " " );
}


void iofs_print_stack( char* type, integer top )
{
 integer level;
 integer fid;
 t_filename filename;
 card status;
 t_buf buf;
 msg( "    ------------------------------" );
 msg( "    | Level    FID   Status      | File ");
 for ( level = top; level > 0; level-- ) {
  iofs_get_entry( type, level, filename, &fid, status );
  sprintf( buf, "    | %-6ld   %-6ld%-12s| %s", (long)level, (long)fid, status, filename );
  msg( buf );
 }
 msg( "    ------------------------------" );
 
}


void iofs_set_input( void )
{
 integer level;
 integer fid;
 t_filename tbuf;
 card ebuf;
 integer lineno;
 level =iofs_get_level();
 fid = iofs_istack(0);  
 while ( level > 0 && fio_test_eof( fid ) ) {
  dmsg( 10, "IOFS Input Pop" );
  io_ipop(1);
  level = iofs_get_level();
  fid = iofs_istack(0);
 }
 if ( dbg_level(10) ) {
  fio_get_name( fid, tbuf, FSIZE );
  lineno = fio_get_lineno( fid );
  sprintf( ebuf, "IOFS INPUT Level %-6ldFID %-6ldLine %-6ldFile %s", (long)level, (long)fid, (long)lineno, tbuf );
  dmsg( 10, ebuf );
 }
}


integer iofs_istack( integer level )
{
 integer stklevel;
 if ( level == 0 ) {
  stklevel = iofs_ilevel;
 } else {
  stklevel = level;
 }
 return( input_stack[stklevel-1] );
}


integer iofs_status( integer mode )
{
 return( fio_status( *input, mode ) );
}


logical iofs_ok( integer mode )
{
 return( fio_status( *input, mode ) == 0 );
}

logical iofs_eof( integer mode )
{
 return( fio_eof( *input, mode ) );
}

void io_rewind( void )
{
 if ( io_int(0) ) {
  jcm_err_msg( "Cannot rewind terminal" );
 } else {
  fio_rewind( *input );
 }
}
