#include "jcmlib.h"

static unsigned short s_nanval = 0x8000;
static unsigned short s_inf    = 0x8001;
static unsigned short s_ninf   = 0x8002;
static integer i_nanval;
static integer i_inf;
static integer i_ninf;
static real r_nanval;
static real r_inf;
static real r_ninf;
static double d_nanval;
static double d_inf;
static double d_ninf;

void nan_init( void )
{
 char* snan_bits = "8000";
 char* sinf_bits = "8001";
 char* sninf_bits= "8002";
 char* nan_bits  = "7FFFFFFF";
 char* inf_bits  = "7F800000";
 char* ninf_bits = "FF800000";
 char* rnan_bits = "77FFFFFF";
 char* rinf_bits = "7EFFFFFF";
 char* rninf_bits= "FEFFFFFF";
 char* dnan_bits = "77FFFFFFFFFFFFFF";
 char* dinf_bits = "7EFFFFFFFFFFFFFF";
 char* dninf_bits= "FEFFFFFFFFFFFFFF";
 nan_set( snan_bits, sinf_bits, sninf_bits, 'S' );
 nan_set( nan_bits, inf_bits, ninf_bits, 'I' );
 nan_set( rnan_bits, rinf_bits, rninf_bits, 'R' );
 nan_set( dnan_bits, dinf_bits, dninf_bits, 'D' );

}


void nan_set( char* cnan, char* cinf, char* cninf, const char mode )
{
 switch( mode ) {
  case 'S':
   s_nanval = read_binary_s( cnan, 'Z' );
   s_inf    = read_binary_s( cinf, 'Z' );
   s_ninf   = read_binary_s( cninf, 'Z' );
   break;
  case 'I':
   i_nanval = read_binary_i( cnan, 'Z' );
   i_inf    = read_binary_i( cinf, 'Z' );
   i_ninf   = read_binary_i( cninf, 'Z' );
   break;
  case 'R':
   r_nanval = read_binary_r( cnan, 'Z' );
   r_inf    = read_binary_r( cinf, 'Z' );
   r_ninf   = read_binary_r( cninf, 'Z' );
   break;
  case 'D':
   d_nanval = read_binary_d( cnan, 'Z' );
   d_inf    = read_binary_d( cinf, 'Z' );
   d_ninf   = read_binary_d( cninf, 'Z' );
   break;
  default:
   break;
 }

}


integer nan_i( void )
{
 return( i_nanval );
}

short nan_s( void )
{
 return( s_nanval );
}

real nan_r( void )
{
 return( r_nanval );
}

double nan_d( void )
{
 return( d_nanval );
}


integer infinity_i( const integer i )
{
 return( i >= 0 ? i_inf : i_ninf );
}

short infinity_s( const integer i )
{
 return( i >= 0 ? s_inf : s_ninf );
}


real infinity_r( const integer i )
{
 return( i >= 0 ? r_inf : r_ninf );
}

double infinity_d( const integer i )
{
 return( i >= 0 ? d_inf : d_ninf );
}


logical qnan_i( const integer i )
{
 return( i == i_nanval );
}

logical qnan_s( const short i )
{
 return( i == s_nanval );
}
logical qnan_r( const real x )
{
 return( x == r_nanval );
}
logical qnan_d( const double x )
{
 return( x == d_nanval );
}


logical qinf_i( const integer i )
{
 return( i == i_inf || i == i_ninf );
}

logical qinf_s( const short i )
{
 return( i == s_inf || i == s_ninf );
}
logical qinf_r( const real x )
{
 return( x == r_inf || x == r_ninf );
}
logical qinf_d( const double x )
{
 return( x == d_inf || x == d_ninf );
}

