#include <stdio.h>
#include "jcmlib.h"


#define PARSE_NTYPES 28
#define NCHARS 128

typedef enum p_ctype { Null, Control, Special, Space, LPar, RPar,
       Op, Sign, Comma, Point, Rel, SLPar, SRPar,
       Underbar, Digit, UC_Let, LC_Let,
       Letter, Letters, Alphanumeric, 
       Integer, Real, Real_E, Double, Logical,
       Word, Number, Symbol } t_parse_type;

t_parse_type p_type[] = {
 Null, 
 Control, Control, Control, Control, Control, 
 Control, Control, Control, Control, Control, 
 Control, Control, Control, Control, Control, 
 Control, Control, Control, Control, Control, 
 Control, Control, Control, Control, Control, 
 Control, Control, Control, Control, Control, 
 Control, Space,   Special, Special, Special,
 Special, Special, Special, Special, LPar,
 RPar,    Op,      Sign,    Comma,   Sign,
 Point,   Op,      Digit,   Digit,   Digit,
 Digit,   Digit,   Digit,   Digit,   Digit,
 Digit,   Digit,   Special, Special, Rel,
 Rel,     Rel,     Special, Special, UC_Let,
 UC_Let, UC_Let, UC_Let, UC_Let, UC_Let,
 UC_Let, UC_Let, UC_Let, UC_Let, UC_Let,
 UC_Let, UC_Let, UC_Let, UC_Let, UC_Let,
 UC_Let, UC_Let, UC_Let, UC_Let, UC_Let,
 UC_Let, UC_Let, UC_Let, UC_Let, UC_Let,
 SLPar,  Special, SRPar,   Special, Underbar, 
 Special, LC_Let, LC_Let, LC_Let, LC_Let,
 LC_Let, LC_Let, LC_Let, LC_Let, LC_Let,
 LC_Let, LC_Let, LC_Let, LC_Let, LC_Let,
 LC_Let, LC_Let, LC_Let, LC_Let, LC_Let,
 LC_Let, LC_Let, LC_Let, LC_Let, LC_Let,
 LC_Let, LC_Let, Special, Special, Special, 
 Special, Special
};

char* p_name[] = { "Null", "Control", "Special Char", "Space",
  "LPar", "RPar", "Op", "Sign", "Comma", "Dot", "Rel", "SLPar",
  "SRPar", "Underbar", "Digit", "UC Letter", "LC Letter", 
  "Letter", "Letters", "Alphanumeric", "Integer", "Real", "Real(E)",
  "Double", "Logical", "Word", "Number", "Symbol"
};


logical parse_compare( t_parse_type code, t_parse_type gcode );
t_parse_type char_code( char c );
void char_type( t_parse_type code, char* type );
t_parse_type parse_namecode( char* name );
void parser_efmt( char* buf, integer* itemcode, integer* itempos, integer* nitems );
void parse_efmtpack( integer i, integer pack,integer* itemcode, 
  integer* itempos, integer* n, integer itemlen, integer epos, integer ecode );
void parse_efmtread( char* item, integer itemlen, integer* epos_p, logical* efmt_p, t_parse_type* ecode_p );
void parser_chars( char* buf, integer* itemcode, integer* nitems );

void parser_groups( char* buf, integer* itemcode, integer* itempos, integer* nitems, logical allow_an );
void parser_words( char* buf, integer* itemcode, integer* itempos, integer* nitems );
void parser_class( integer* itemcode, integer nitems );
void parse_repack( integer i, integer* nitems, integer* itemcode, integer* itempos, integer pack );


void char_name( char c, char* name )
{
 integer i;
 i = (integer)c;
 if ( i < 0 || i >= 2 * NCHARS ) {
  cs_copy( "##", name );
 } else {
  if ( i >= NCHARS ) {
   *name++ = '@';
   i -= NCHARS;
  }
  switch( i ) {
   case 0:
    cs_copy( "NUL", name );
   case 27: 
    cs_copy( "ESC", name ); break;
   case 28:
    cs_copy( "FS", name ); break;
   case 29:
    cs_copy( "GS", name ); break;
   case 30:
    cs_copy( "RS", name ); break;
   case 31:
    cs_copy( "US", name ); break;
   case 32:
    cs_copy( "SP", name ); break;
   case 127:
    cs_copy( "DEL", name ); break;
   default:
    if ( i < 26 ) {
     *name++ = '^';
     *name++ = 'A' + i - 1;
     *name   = '\0';
    } else {
     *name++ = (char)i;
     *name   = '\0';
    }
    break;
  }
 }

}


void char_type( t_parse_type code, char* type )
{
/* Bug fix 1997 Jan 27: code >=0 gives warning, code is unsigned */
 if ( code < PARSE_NTYPES ) {
  cs_copy( p_name[code], type );
 } else {
  cs_copy( "Unknown", type );
 }
}


t_parse_type parse_namecode( char* name )
{
 return( cmatch_c( name, p_name, PARSE_NTYPES ) );
}

t_parse_type char_code( char c )
{
 integer i;

 i = (int)c;
 if ( i >= NCHARS ) {
  return( -1 );
 } else {
  return( p_type[i] );
 }
}

void parser( char* buf, integer* itemcode, integer* itempos, integer* nitems, char* cmode )
{
 t_word mode;
 integer i;
 logical an;

 cs_copy( cmode, mode );
 cs_upper( mode );
 if ( cs_ss( mode, "CHAR" ) ) {
  parser_chars( buf,itemcode, nitems );
  for ( i = 0; i < *nitems; i++ ) {
   itempos[i] = i;
  }
  if ( dbg_level(30) ) parse_disp( buf, itemcode, itempos, *nitems );
 }
 if ( cs_ss( mode, "GROUP" ) ) {
  an = !cs_ss( mode, "NOAN" );
  parser_groups( buf, itemcode, itempos, nitems, an );
 }

 if ( cs_ss( mode, "WORD" ) ) {
  parser_words( buf, itemcode, itempos, nitems );
 }

 if ( cs_ss( mode, "CLASS" ) ) {
  parser_class( itemcode, *nitems );
  if ( dbg_level(30) ) parse_disp( buf, itemcode, itempos, *nitems );
 }
}


void parser_chars( char* buf, integer* itemcode, integer* nitems )
{
 integer nchars;
 integer i;
 nchars = cs_ends( buf );
 for ( i = 0; i < nchars; i++ ) {
  itemcode[i] = char_code( *buf++ );
 }
 *nitems = nchars;
}


void parser_groups( char* buf, integer* itemcode, integer* itempos, integer* nitems, logical allow_an )
{
 integer n;
 integer i;
 t_parse_type icode;
 t_parse_type jcode;
 integer j;
 logical newitem;
 logical q_text, q_an1, q_an2;

 n = cs_ends( buf );
 icode = char_code( buf[0] );
 itemcode[0] = icode;
 j = 0;
 for ( i = 1; i < n; i++ ) {
  jcode = char_code( buf[i] );
  newitem = parse_compare( jcode, Symbol ) && 
   ! ( jcode == Op || jcode == Rel || jcode == Sign || jcode == Underbar );
  if( jcode == icode ) {
   if ( icode == Digit ) itemcode[j] = Integer;
   if ( parse_compare( icode, Letter ) ) itemcode[j] = Letters;
  } else {
   q_text = parse_compare( jcode, Letter ) && parse_compare( icode, Letters );
   q_an1 = parse_compare( icode, Word );
   q_an1  = parse_compare( jcode, Digit ) && parse_compare( icode, Word );
   q_an2  = ( parse_compare( icode, Integer ) || icode == Alphanumeric ) && parse_compare( jcode, Letter );
   if ( q_text ) {
    icode = Letters;
    itemcode[j] = Letters;
   } else if ( allow_an && ( q_an1 || q_an2 ) ) {
    icode = Alphanumeric;    
    itemcode[j] = icode;
   } else {
    newitem = TRUE;
   } 
  }
  if ( newitem ) {
   itempos[j++] = i;
   itemcode[j] = jcode;
   icode = jcode;
  }
 }
 itempos[j] = n;
 *nitems = j+1;
 if ( dbg_level(30) ) parse_disp( buf, itemcode, itempos, *nitems );
}



logical parse_compare( t_parse_type code, t_parse_type gcode )
{
 logical match;
 match = ( code == gcode );
 if ( !match ) {
  if ( gcode == Letter ) {
   match =  ( code == UC_Let || code == LC_Let || code == Underbar );
  } else if ( gcode == Integer ) {
   match =  ( code == Digit );
  } else if ( gcode == Letters ) {
   match = ( code == Letter || code == UC_Let ||
             code == LC_Let || code == Underbar );
  } else if ( gcode == Number ) {
   match = ( code == Digit ) || ( code == Integer ) ||
           ( code == Real  ) || ( code == Real_E ) || ( code == Double );
  } else if ( gcode == Word ) {
   match = ( code == Letter ) || ( code == UC_Let ) || ( code == LC_Let )
           || ( code == Alphanumeric ) || ( code == Letters ) || ( code == Underbar );
  } else if ( gcode == Symbol ) {
   match = ( code == Special ) || ( code == Op ) || ( code == Comma ) ||
   ( code == Rel ) || ( code == Sign ) || ( code == Logical ) || 
   ( code == LPar ) || ( code == RPar ) || ( code == Point ) || ( code == SLPar )
   || ( code == SRPar ) || ( code == Underbar );
  }
 }
 return( match );
}



void parse_disp( char* buf, integer* itemcode, integer* itempos, integer nitems )
{
 char mbuf[513];
 char* ptr;
 t_word name;
 card word;
 card tmp;
 integer siz = 512;
 integer i;
 integer i1,i2;
 integer n;

 ptr = mbuf;
 for ( i = 0; i < nitems; i++ ) {
  char_type( itemcode[i], name );
  i1 = ( i == 0 ) ? 1 : itempos[i-1]+1; 
  i2 = itempos[i];
  cs_get_ss( buf, word, i1, i2 );
  n = sprintf( tmp, "%s[%s] ", name, word );
  siz -= n;
  if (siz > 0 ) {
   cs_copy_siz( tmp, ptr, siz );
   ptr += n;
  } 
 }
 msg( mbuf );
}

void parse_tdisp( char* buf, integer* itemcode, integer* itempos, integer nitems )
{
 char mbuf[513];
 char* ptr;
 t_word name;
 card word;
 card tmp;
 integer siz = 512;
 integer i;
 integer i1,i2;
 integer n;

 ptr = mbuf;
 for ( i = 0; i < nitems; i++ ) {
  char_type( itemcode[i], name );
  i1 = ( i == 0 ) ? 1 : itempos[i-1]+1; 
  i2 = itempos[i];
  cs_get_ss( buf, word, i1, i2 );
  n = sprintf( tmp, "%s[%s] ", name, word );
  siz -= n;
  if (siz > 0 ) {
   cs_copy_siz( tmp, ptr, siz );
   ptr += n;
  } 
 }
 tmsg( mbuf );
}


void parse_read( integer i, char* buf, integer* itempos, char* token, integer* siz )
{
 integer j1, j2;
 if ( i == 0 ) {
  j1 = 1;
 } else {
  j1 = itempos[i-1]+1;
 }
 j2 = itempos[i];
 cs_get_ss( buf, token, j1, j2 );
 *siz = j2 - j1 + 1;
}

void parser_words( char* buf, integer* itemcode, integer* itempos, integer* nitems )
{
 integer n;
 integer i;
 integer icode;

 n = *nitems;
 i = 0;

 while ( i < *nitems ) {
  icode = itemcode[i];

/* LOGICAL */
  if ( icode == Point && i <= *nitems-3 ) {
   if ( parse_compare( itemcode[i+1], Letters )  && itemcode[i+2] == Point ) {
    itemcode[i] = Logical;
    icode = Logical;
    parse_repack( i, nitems, itemcode, itempos, 2 );
   }
  }
  i++;
 }

/* REAL */
 i = 0;
 while ( i < *nitems ) {
  icode = itemcode[i];
  if ( parse_compare( icode, Integer ) && i < *nitems - 1 ) {
   if ( itemcode[i+1] == Point ) {
    itemcode[i] = Real;
    icode = Real;
    if ( i < *nitems - 2 && parse_compare( itemcode[i+2], Integer ) ) {
     parse_repack( i, nitems, itemcode, itempos, 2 );
    } else {
     parse_repack( i, nitems, itemcode, itempos, 1 );     
    }
   }
  } else if ( icode == Point && i < *nitems - 1 ) {
   if ( parse_compare( itemcode[i+1], Integer ) ) {
    itemcode[i] = Real;
    icode = Real;
    parse_repack( i, nitems, itemcode, itempos, 1 );
   }
  }
  i++;
 }

 parser_efmt( buf, itemcode, itempos, nitems );

 i = 0;
 while ( i < *nitems ) {
  icode = itemcode[i]; 
  if ( icode == Space ) {
   if ( i>0 ) itempos[i-1] = itempos[i];
   itemcode[i] = itemcode[i+1];
   parse_repack( i, nitems, itemcode, itempos, 1 );
  }
  i++;
 }
 if ( dbg_level(30) ) parse_disp( buf, itemcode, itempos, *nitems );
}


void parser_class( integer* itemcode, integer nitems )
{
 integer i;
 integer icode;
 for ( i = 0; i < nitems; i++ ) {
  icode = itemcode[i];
  if ( parse_compare( icode, Number ) ) {
   itemcode[i] = Number;
  } else if ( parse_compare( icode, Word )) {
   itemcode[i] = Word;
  } else {
   itemcode[i] = Symbol;
  }
 }
}


void parse_repack( integer i, integer* nitems, integer* itemcode, integer* itempos, integer pack )
{
 integer n;
 integer k;

 n = (*nitems) - pack;
 itempos[i] = itempos[i+pack];
 if ( i < n-1 ) {
  for ( k = i+1; k < n; k++ ) {
   itemcode[k] = itemcode[k+pack];
   itempos[k]  = itempos[k+pack];
  }
 }
 *nitems = n;
}


void parse_efmtread( char* item, integer itemlen, integer* epos_p, logical* efmt_p, t_parse_type* ecode_p )
{
 integer epos, dpos;
 integer ecode;
 logical efmt;
 integer pos;
 integer icode;
 ecode = -1;
 epos = cs_index_char( item, 'E' );
 dpos = cs_index_char( item, 'D' );
 if ( dpos > 0 ) epos = dpos;
 efmt = epos > 0;
 if ( efmt ) {
  pos = 0;
  while( pos < itemlen && efmt ) {
   pos++;
   if ( pos != epos ) {
    icode = char_code( item[pos-1] );
    efmt = icode == Digit || icode == Sign || icode == Point;
   }
  }
 }
 if ( efmt ) {
  if ( dpos > 0 ) {
   ecode = Double;
  } else {
   ecode = Real_E;
  }
 }
 *epos_p = epos;
 *ecode_p = ecode;
 *efmt_p = efmt;
}


void parse_efmtpack( integer i, integer pack,integer* itemcode, 
  integer* itempos, integer* n, integer itemlen, integer epos, integer ecode )
{
 integer j;
 if ( epos != itemlen ) {
  j = pack - 2;
  if ( j > 0 ) parse_repack( i, n, itemcode, itempos, j );
  itemcode[i] = ecode;
 } else if ( i < *n - pack ) {
  j = i + pack;
  if ( itemcode[j-1] == Sign && parse_compare( itemcode[j], Integer ) ) {
   parse_repack( i, n, itemcode, itempos, pack );
   itemcode[i] = ecode;
  } 
 } else if ( parse_compare( itemcode[i+pack-1], Integer ) ) {
  parse_repack( i, n, itemcode, itempos, pack-1 );
  itemcode[i] = ecode;
 }
}


void parser_efmt( char* buf, integer* itemcode, integer* itempos, integer* nitems )
{
 integer i;
 integer icode, kcode;
 integer itemlen;
 logical efmt;
 card item;
 integer epos;
 t_parse_type ecode;

 i = 0;
 while( i < *nitems ) {
  icode = itemcode[i];
  if ( i < *nitems - 1 ) {
   kcode = itemcode[i+1];
   parse_read( i+1, buf, itempos, item, &itemlen );
   cs_upper( item );
  }
  if ( parse_compare( icode, Number ) && i < *nitems -1 ) {
   if ( parse_compare( kcode, Letter ) || kcode == Alphanumeric ) {
    parse_efmtread( item, itemlen, &epos, &efmt, &ecode );
    if ( efmt ) {
     parse_efmtpack( i, 3, itemcode, itempos, nitems, itemlen, epos, ecode );
     icode = itemcode[i];
    }
   }   
  } else if ( icode == Point && i < *nitems - 1 ) {
   if ( parse_compare( kcode, Letter ) || kcode == Alphanumeric ) {
    parse_efmtread( item, itemlen, &epos, &efmt, &ecode );
    if ( efmt && epos != 1 ) {
     parse_efmtpack( i, 3, itemcode, itempos, nitems, itemlen, epos, ecode );
     icode = itemcode[i];
    }
   }
  } else if ( icode == Alphanumeric ) {
   parse_read( i, buf, itempos, item, &itemlen );
   cs_upper( item );
   parse_efmtread( item, itemlen, &epos, &efmt, &ecode );
   if ( efmt && epos != 1 ) {
    parse_efmtpack( i, 2, itemcode, itempos, nitems, itemlen, epos, ecode );
    icode = itemcode[i];
   }
  }
  i++;
 }
}
