#include "jcmlib.h"

real linterp_r( const real* x, const real* y, const integer n, const real x0 )
{
 integer i;
 i = ar_cbisect_r( x, n, x0 );
 i = min_i( i, n-2 );
 return( lint_r( x[i], x[i+1], y[i], y[i+1], x0 ) );
}

double linterp_d( const double* x, const double* y, const integer n, const double x0 )
{
 integer i;
 i = ar_cbisect_d( x, n, x0 );
 i = min_i( i, n-2 );
 return( lint_d( x[i], x[i+1], y[i], y[i+1], x0 ) );
}

void movag_r( const real* yy0, real* y, const integer n, const integer k )
{
 integer i, j, j1, j2;
 integer m1, m2;
 double sum;

 j = ( k - 1 ) / 2;
 j1 = -j-1;
 j2 = j1 + k;
 sum = 0.0;
 for ( i = j1 ; i < j2; i++ ) {
  m1 = min_i( max_i( i, 0 ), n-1 );
  sum += yy0[m1];
 }

 for ( i = 0; i < n; i++ ) {
  m1 = min_i( max_i( j1 + i, 0 ), n-1 );
  m2 = min_i( max_i( j2 + i, 0 ), n-1 );
  sum += yy0[m2] - yy0[m1];
  y[i] = sum/k;
 }
}

void movag_d( const double* yy0, double* y, const integer n, const integer k )
{
 integer i, j, j1, j2;
 integer m1, m2;
 double sum;

 j = ( k - 1 ) / 2;
 j1 = -j-1;
 j2 = j1 + k;
 sum = 0.0;
 for ( i = j1 ; i < j2; i++ ) {
  m1 = min_i( max_i( i, 0 ), n-1 );
  sum += yy0[m1];
 }

 for ( i = 0; i < n; i++ ) {
  m1 = min_i( max_i( j1 + i, 0 ), n-1 );
  m2 = min_i( max_i( j2 + i, 0 ), n-1 );
  sum += yy0[m2] - yy0[m1];
  y[i] = sum/k;
 }
}

double poly_d( const double x, const double* coeffs, const integer degree )
{
 integer order;
 double yval = 0.0;
 for ( order = degree; order >= 0; order-- ) {
  yval = yval * x + coeffs[order];
 }
 return( yval );
}

real poly_r( const real x, const real* coeffs, const integer degree )
{
 integer order;
 real yval = 0.0;
 for ( order = degree; order >= 0; order-- ) {
  yval = yval * x + coeffs[order];
 }
 return( yval );
}

void poleval_d( const double* x, double* y, const integer m, const double* coeffs, const integer degree )
{
 integer n = m;
 while( n-- ) { *y++ = poly_d( *x++, coeffs, degree ); }
}

void poleval_r( const real* x, real* y, const integer m, const real* coeffs, const integer degree )
{
 integer n = m;
 while( n-- ) { *y++ = poly_r( *x++, coeffs, degree ); }
}


void lsfit_r( const real* x, const real* y, const integer n, real* a, real* b )
{
 double xbar, ybar, xvar, sxy, t;
 integer i;

 xbar = 0;
 ybar = 0;
 xvar = 0;
 sxy = 0;
 for ( i = 0; i < n; i++ ) {
  xbar += x[i];
  ybar += y[i];
 }
 xbar /= n;
 ybar /= n;
 for ( i = 0; i < n; i++ ) {
  t = x[i] - xbar;
  xvar += t * t;
  sxy  += t * y[i];
 }
 *b = sxy / xvar;
 *a = ybar - (*b) * xbar;
}

void lsfit_d( const double* x, const double* y, const integer n, double* a, double* b )
{
 double xbar, ybar, xvar, sxy, t;
 integer i;

 xbar = 0;
 ybar = 0;
 xvar = 0;
 sxy = 0;
 for ( i = 0; i < n; i++ ) {
  xbar += x[i];
  ybar += y[i];
 }
 xbar /= n;
 ybar /= n;
 for ( i = 0; i < n; i++ ) {
  t = x[i] - xbar;
  xvar += t * t;
  sxy  += t * y[i];
 }
 *b = sxy / xvar;
 *a = ybar - (*b) * xbar;
}

