#include <stdlib.h>
#include <stdio.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "esc_types.h"
/* Routines added 1997 Jun 24 */

/*
 * Process each line of an ASCII file with the function 
 * process_line, except for blank lines and lines whose first
 * character is #. This function must have a single string
 * argument which is the input line it will process.
 * It returns void. Input lines have tabs converted to spaces
 * before they are sent to process_line.
 * The idea of this routine is that many operations involve
 * processing ASCII files on a line-by-line basis, except for
 * some comment lines, so this code fragment is repeated
 * pretty often.
 */
void process_file( char* filename, void (*process_line)( char* ) )
{
 integer in;
 t_buf line;
 char cmt;
 cmt = esc_special_char( CMT_CHAR );
 in =  fio_open_r( filename );
 if ( !in) {  dmsg( 1, "File open failed" ); return; }
 while( fio_read( in, line, BSIZE ) ) {
  cs_detab( line );
  if ( !cs_is_blank( line ) && line[0] != cmt ) {
   process_line( line );
  }
 }
 fio_close( in );
}

/*
 * A variation on the previous function, with an extra void* argument
 * in both this call and the process_line function.
 * Typical example:
 *   process_file1( myfile, myfunction, mode );
 * ...
 *   void myfunction( char* mode, char* line );
 */
void process_file1( char* filename, void (*process_line)( void*, char* ), void* ptr )
{
 integer in;
 t_buf line;
 char cmt;
 cmt = esc_special_char( CMT_CHAR );
 in =  fio_open_r( filename );
 if ( !in) { dmsg( 1, "File open failed" ); return; }
 while( fio_read( in, line, BSIZE ) ) {
  cs_detab( line );
  if ( !cs_is_blank( line ) && line[0] != cmt ) {
   process_line( ptr, line );
  }
 }
 fio_close( in );
}


