
#include <stdio.h>
#include "jcmlib.h"
#include "bits.h"
#define MAXBITS 128
#define BIG_INT 65536

static integer bit_setbyte( const integer* bits );
static void bit_getbyte( const integer byte, integer* bits );
static void bit_wfmt( const integer* bits, const integer nbits, const integer base_bits, char* buf );
static integer bit_rfmt( const char* buf, integer* bits, const integer base_bits );
static void bit_write( const integer* bits, const integer nbits, const char mode, char* buf );
static integer bit_read( const char* buf, integer* bits, const char mode );

integer bit_setbyte( const integer* bits )
{
 integer bit;
 integer byte = 0;
 for ( bit = 0; bit < BYTE_BITS; bit++ ) {
  if ( bits[bit] ) bit_set_i( &byte, bit );
 }
 return( byte );
}

void bit_getbyte( const integer byte, integer* bits )
{
 integer bit;
 ar_zero_i( bits, BYTE_BITS );
 for ( bit = 0; bit < BYTE_BITS; bit++ ) {
  if ( bit_test_i( byte, bit ) ) bits[bit] = 1;
 }
}

integer bit_ar_get_r( const real value, integer* bits )
{
 integer bytes[BYTES_REAL];
 integer nbytes = BYTES_REAL;
 integer nbits = BYTES_REAL * BYTE_BITS;
 integer byte_no;
 integer bit_pos;

 ar_zero_i( bits, nbits ); 
 bytes_split_r( value, bytes );
 for ( byte_no = 0; byte_no < nbytes; byte_no++ ) {
  bit_pos =  byte_no  * BYTE_BITS;
  bit_getbyte( bytes[byte_no], bits+bit_pos );
 }
 return( nbits );
}

integer bit_ar_get_d( const double value, integer* bits )
{
 integer bytes[BYTES_DOUBLE];
 integer nbytes = BYTES_DOUBLE;
 integer nbits = BYTES_DOUBLE * BYTE_BITS;
 integer byte_no;
 integer bit_pos;

 ar_zero_i( bits, nbits ); 
 bytes_split_d( value, bytes );
 for ( byte_no = 0; byte_no < nbytes; byte_no++ ) {
  bit_pos =  byte_no  * BYTE_BITS;
  bit_getbyte( bytes[byte_no], bits+bit_pos );
 }
 return( nbits );
}

integer bit_ar_get_s( const short value, integer* bits )
{
 integer bytes[BYTES_SHORT];
 integer nbytes = BYTES_SHORT;
 integer nbits = BYTES_SHORT * BYTE_BITS;
 integer byte_no;
 integer bit_pos;

 ar_zero_i( bits, nbits ); 
 bytes_split_s( value, bytes );
 for ( byte_no = 0; byte_no < nbytes; byte_no++ ) {
  bit_pos =  byte_no  * BYTE_BITS;
  bit_getbyte( bytes[byte_no], bits+bit_pos );
 }
 return( nbits );
}

integer bit_ar_get_i( const integer value, integer* bits )
{
 integer bytes[BYTES_INT];
 integer nbytes = BYTES_INT;
 integer nbits = BYTES_INT * BYTE_BITS;
 integer byte_no;
 integer bit_pos;

 ar_zero_i( bits, nbits ); 
 bytes_split_i( value, bytes );
 for ( byte_no = 0; byte_no < nbytes; byte_no++ ) {
  bit_pos =  byte_no  * BYTE_BITS;
  bit_getbyte( bytes[byte_no], bits+bit_pos );
 }
 return( nbits );
}


short bit_ar_set_s( const integer* bits )
{
 short value;
 integer bytes[BYTES_SHORT];
 integer nbytes = BYTES_SHORT;
 integer byte_no;
 integer bit_pos;

 for ( byte_no = 0; byte_no < nbytes; byte_no++ ) {
  bit_pos =  byte_no  * BYTE_BITS;
  bytes[byte_no] = bit_setbyte( bits+bit_pos );
 }
 value = bytes_combine_s( bytes );
 return( value );
}

integer bit_ar_set_i( const integer* bits )
{
 integer value;
 integer bytes[BYTES_INT];
 integer nbytes = BYTES_INT;
 integer byte_no;
 integer bit_pos;
 for ( byte_no = 0; byte_no < nbytes; byte_no++ ) {
  bit_pos =  byte_no  * BYTE_BITS;
  bytes[byte_no] = bit_setbyte( bits+bit_pos );
 }
 value = bytes_combine_i( bytes );
 return( value );
}

real bit_ar_set_r( const integer* bits )
{
 real value;
 integer bytes[BYTES_REAL];
 integer nbytes = BYTES_REAL;
 integer byte_no;
 integer bit_pos;
 for ( byte_no = 0; byte_no < nbytes; byte_no++ ) {
  bit_pos =  byte_no  * BYTE_BITS;
  bytes[byte_no] = bit_setbyte( bits+bit_pos );
 }
 value = bytes_combine_r( bytes );
 return( value );
}

double bit_ar_set_d( const integer* bits )
{
 double value;
 integer bytes[BYTES_DOUBLE];
 integer nbytes = BYTES_DOUBLE;
 integer byte_no;
 integer bit_pos;
 for ( byte_no = 0; byte_no < nbytes; byte_no++ ) {
  bit_pos =  byte_no  * BYTE_BITS;
  bytes[byte_no] = bit_setbyte( bits+bit_pos );
 }
 value = bytes_combine_d( bytes );
 return( value );
}

integer bits_unsigned( const short value )
{
 integer result = ( value < 0 ) ? value + BIG_INT : value;
 return( result );
}


integer bits_val( const integer* bits, const integer m )
{
 integer n = m;
 integer result = 0;
 const integer* ptr = bits + n - 1;
 while ( n-- ) {
  result = result * 2 + *ptr--;
 }
 return( result );
}

void bit_wfmt( const integer* bits, const integer nbits, const integer base_bits, char* buf )
{
 char digits[]= "0123456789ABCDEF";
 integer bit;
 integer pos;
 integer value;
 integer tpos;
 integer step;
 integer nsteps;
 integer r;
/*
 * Convert an array of bits to a string in hex, octal or binary notation.
 * Case of binary (base_bits = 1):
 *  the leftmost character in the output string is the most significant 
 *  bit, which is the last bit in the array: bits[nbits-1].
 *  Each bit corresponds to a 0 or 1 in the output string.
 * Case of octal or hex: We work in steps of base_bits.
 *  For each step, we accumulate the value from the bits
 *  and then convert it to a character.
 */   
 buf[0] = '\0';
 if ( base_bits == 1 ) {
  for ( bit = 0; bit < nbits; bit++ ) {
   pos = bits[nbits-bit-1];
   buf[bit] = digits[pos];
  }

  buf[nbits] = '\0';
 } else {
  nsteps = nbits / base_bits;
  if ( nsteps * base_bits < nbits ) nsteps++;

  for ( tpos = 0; tpos < nsteps; tpos++ ) {
   step = nsteps - tpos - 1;
   value = 0;
   for ( pos = base_bits - 1; pos >= 0; pos-- ) {
    bit = base_bits * step + pos;
    r   = ( bit >= nbits ) ? 0 : bits[bit];
    value = value * 2 + r;
   }
   buf[tpos] = digits[value];

  }
  buf[nsteps] = '\0';

 }
}

/*
 *  Parse a string containing a bit pattern, return an array of 0s and 1s
 */
integer bit_rfmt( const char* buf, integer* bits, const integer base_bits )
{
 integer nbits;
 integer buf_size;
 integer bit_no;
 integer pos;
 integer byte_pos;
 integer hex_byte;

 buf_size = cs_ends( buf );
 nbits = buf_size * base_bits;
 if ( base_bits == 1 ) {
/* B */  /* Note O or Z branch will also work for this case, but for clarity we treat this specially */
  for ( pos = 0; pos < buf_size; pos++ ) {
   bit_no = nbits - pos - 1;
   bits[bit_no] = read_digit( buf[pos] );
  }
 } else {
/* O or Z */
  for ( pos = 0; pos < buf_size; pos++ ) {
   hex_byte = read_hex_digit( buf[pos] );
   for ( byte_pos = 0; byte_pos < base_bits; byte_pos++ ) {
    bit_no = nbits - base_bits * ( pos + 1 ) + byte_pos;
    bits[bit_no] = hex_byte % 2;
    hex_byte /= 2;
   }
  }  
 }
 return( nbits );
}


void bit_write( const integer* bits, const integer nbits, const char mode, char* buf )
{
 integer base_bits;
 switch( mode ) {
  case 'B': 
   base_bits = 1;
   break;
  case 'O':
   base_bits = 3;
   break;
  case 'Z':
   base_bits = 4;
   break;
  default:
   base_bits = 0;
   break;
 }
 bit_wfmt( bits, nbits, base_bits, buf ); 
}

integer bit_read( const char* buf, integer* bits, const char mode )
{
 integer nbits;
 integer base_bits;
 switch( mode ) {
  case 'B': 
   base_bits = 1;
   break;
  case 'O':
   base_bits = 3;
   break;
  case 'Z':
   base_bits = 4;
   break;
  default:
   base_bits = 0;
   break;
 }
 nbits = bit_rfmt( buf, bits, base_bits );
 return( nbits );
}


void write_binary_i( const integer value, char* buf, const char mode )
{
 integer bits[MAXBITS];
 integer n;
 n = bit_ar_get_i( value, bits );
 bit_write( bits, n, mode, buf );
}

void write_binary_s( const short value, char* buf, const char mode )
{
 integer bits[MAXBITS];
 integer n;
 n = bit_ar_get_s( value, bits );
 bit_write( bits, n, mode, buf );
}

void write_binary_r( const real value, char* buf, const char mode )
{
 integer bits[MAXBITS];
 integer n;
 n = bit_ar_get_r( value, bits );
 bit_write( bits, n, mode, buf );
}

void write_binary_d( const double value, char* buf, const char mode )
{
 integer bits[MAXBITS];
 integer n;
 n = bit_ar_get_d( value, bits );
 bit_write( bits, n, mode, buf );
}

short read_binary_s( const char* buf, const char mode )
{
 short value;
 integer bits[MAXBITS];
 integer n;
 n = bit_read( buf, bits, mode );
 value = bit_ar_set_s( bits );
 return( value );
}

integer read_binary_i( const char* buf, const char mode )
{
 integer value;
 integer bits[MAXBITS];
 integer n;
 n = bit_read( buf, bits, mode );
 value = bit_ar_set_i( bits );
 return( value );
}

real read_binary_r( const char* buf, const char mode )
{
 real value;
 integer bits[MAXBITS];
 integer n;
 n = bit_read( buf, bits, mode );
 value = bit_ar_set_r( bits );
 return( value );
}

double read_binary_d( const char* buf, const char mode )
{
 double value;
 integer bits[MAXBITS];
 integer n;
 n = bit_read( buf, bits, mode );
 value = bit_ar_set_d( bits );
 return( value );
}


