#include "jcmlib.h"
#include "jcmp.h"


#define FUNC_TOL 1.0e-5
#define PREFER_HEAPSORT 1000

/*  An IDX is a C array index counting from 0 */
/*  An ID  is a Fortran style index from 1 */

typedef integer idx;
typedef integer id;

static void sort_equals( const real* x, const integer n, integer* jsort )
{
/* Perform sorting of equal values in presorted array */
 logical loop;
 integer index;
 integer previous_index;
 integer sorted_index;
 integer sorted_previous_index;


 for ( index = 1; index <n; index++ )
 {
  sorted_index = jsort[index]-1;
  previous_index = index - 1;
  loop = TRUE;
  while( previous_index >= 0 && loop )
  { 
   
   sorted_previous_index = jsort[previous_index]-1;
   loop = ( x[sorted_index] == x[sorted_previous_index] && sorted_index < sorted_previous_index );
   if ( loop ) {
    jsort[previous_index+1] = sorted_previous_index + 1;  
    jsort[previous_index] = sorted_index + 1;  
   }
   previous_index--;
  }
 }
}

void heapsort_r( const  real* x, const integer n, id* jsort )
{
 real y;
 logical loop;
 integer index;
 integer sorted_index;
 integer index1;
 integer index2;
 integer tindex1;
 integer tindex2;
 integer sorted_tindex2;
 integer sorted_next;

 for ( index = 0; index < n; index++ )
  jsort[index] = index+1;

 index1 = n / 2;
 index2 = n-1;
 loop = TRUE;
 while ( loop ) {
  if ( index1 > 0 ) {
   index1--;
   sorted_index = jsort[index1]-1;
   y = x[sorted_index];
  } else {
   sorted_index = jsort[index2] - 1;
   y = x[sorted_index];
   jsort[index2]= jsort[0];
   index2--;
   if ( index2 == 0 ) {
    jsort[0] = sorted_index + 1;
    loop = FALSE;
   }
  }
  if ( loop ) {
   tindex1 = index1;
   tindex2 = 2* index1 +1;
   while ( tindex2 <= index2  ) {
    if ( tindex2  < index2 ) {
     sorted_tindex2 = jsort[tindex2]-1;
     sorted_next    = jsort[tindex2+1] -1;
     if ( x[sorted_tindex2] < x[sorted_next] ) tindex2++;
    }
    if ( y < x[jsort[tindex2]-1] ) {
     jsort[tindex1] = jsort[tindex2];
     tindex1 = tindex2;
     tindex2 = 2*tindex2 + 1;
    } else {
     tindex2 = index2 + 1;
    }
   }
  jsort[tindex1] = sorted_index + 1;
  }
 } /* end while loop */
 sort_equals( x, n, jsort );

}

void shellsort_r( const real* x, const integer n, id * jsort )
{
 integer log2n;
 idx i;
 logical loop;
 integer index;

 integer index1;
 integer index2;
 integer sorted_index1;
 integer sorted_index2;
 integer chunk;
 integer segment;
 integer segment_size;

 log2n = logbase_i( n, 2 );
 for ( i = 0; i < n; i++ )
 { jsort[i] = i+1; }

 segment_size = n ;
 
 for ( segment = 0; segment < log2n; segment++ )
 { /* outer loop */

  segment_size = segment_size / 2;
  chunk = n - segment_size;
  for ( index = 0; index < chunk; index++ )
  { /* second loop */
   index1 = index;
   loop = TRUE;
   while( loop )
   {  /* inner loop */
    index2 = index1 + segment_size;
    sorted_index1 = jsort[index1]-1;
    sorted_index2 = jsort[index2]-1;
    if ( x[sorted_index2] < x[sorted_index1] ) {
     jsort[index1] = sorted_index2 + 1;
     jsort[index2] = sorted_index1 + 1;
     index1 = index1 - segment_size;
     loop = ( index1 >= 0 );
    } else {
     loop = FALSE;
    }
   } /* end inner loop */
  } /* end second loop */
 } /* end outer loop */

/* Pass to check for equality */
 sort_equals( x, n, jsort );

}



void sort_r( const real* x, const integer n, integer* jsort )
{
 if ( n > PREFER_HEAPSORT )
  heapsort_r( x, n, jsort );
 else if ( n <= 1 ) 
  jsort[0] = 1;
 else
  shellsort_r( x, n, jsort );
}



void resort_s( const short* x, const integer n, short* y, integer* jsort )
{
 integer i;
 for ( i = 0; i < n; i++ ) {
  y[i] = x[jsort[i]-1];
 }

}
void cresort_s( const short* x, const integer n, short* y, integer* jsort )
{
 integer i;
 for ( i = 0; i < n; i++ ) {
  y[i] = x[jsort[i]];
 }
}

void resort_d( const double* x, const integer n, double* y, integer* jsort )
{
 integer i;
 for ( i = 0; i < n; i++ ) {
  y[i] = x[jsort[i]-1];
 }

}
void cresort_d( const double* x, const integer n, double* y, integer* jsort )
{
 integer i;
 for ( i = 0; i < n; i++ ) {
  y[i] = x[jsort[i]];
 }
}
void resort_r( const real* x, const integer n, real* y, integer* jsort )
{
 integer i;
 for ( i = 0; i < n; i++ ) {
  y[i] = x[jsort[i]-1];
 }

}
void cresort_r( const real* x, const integer n, real* y, integer* jsort )
{
 integer i;
 for ( i = 0; i < n; i++ ) {
  y[i] = x[jsort[i]];
 }
}

void resort_i( const integer* x, const integer n, integer* y, integer* jsort )
{
 integer i;
 for ( i = 0; i < n; i++ ) {
  y[i] = x[jsort[i]-1];
 }

}
void cresort_i( const integer* x, const integer n, integer* y, integer* jsort )
{
 integer i;
 for ( i = 0; i < n; i++ ) {
  y[i] = x[jsort[i]];
 }
}

