#include "jcmlib.h"

/* Bisection and search routines */

/* Fortran convention */

/*
 *  x0 < x(1)     Return 1
 *  x0 > x(n)     Return n
 *  x0 = x(m)-e   Return m-1
 *  x0 = x(m)     Return m
 *  x0 = x(m)+e   Return m
 *  so  x(m) <= x0 < x(m+1)
 */

integer ar_bisect_d( const double* x, const integer n, const double x0 )
{
 /* Simple bisection search to find position of x0 in array x */
 integer element;
 integer offset;
 integer ju, jl, jm;
 if ( x0 <= x[0] ) {
  element = 1;
 } else if ( x0 >= x[n-1] ) {
  element = n;
 } else {
  jl = 0;
  ju = n+1;
  while( (ju-jl) > 1 ) {
   jm = (ju+jl)/2;
   if ( x0 > x[jm-1] ) {
    jl = jm;
   } else {
    ju = jm;
   }
  }
  element = jl;   /* ie (jl-1) + 1 */
  offset = element - 1;
  /* We know 
   *    x0 in  [x(element),x(element+1)]
   * ie    x[offset] and x[offset+1]
   * We want to return 
   *    x0 in [x(element),x(element+1))
   */
  if( x[offset + 1] == x0 ) element++;
 }
 return( element );
}

integer ar_cbisect_d( const double* x, const integer n, const double x0 )
{
 return( ar_bisect_d( x, n, x0 ) - 1 );
}

integer ar_bisect_r( const real* x, const integer n, const real x0 )
{
 integer j, ju, jl, jm;
 if ( x0 <= x[0] ) {
  j = 1;
 } else if ( x0 >= x[n-1] ) {
  j = n;
 } else {
  jl = 0;
  ju = n+1;
  while( (ju-jl) > 1 ) {
   jm = (ju+jl)/2;
   if ( x0 > x[jm-1] ) {
    jl = jm;
   } else {
    ju = jm;
   }
  }
  j = jl;
  if( x[jl] == x0 ) j++;
 }
 return( j );
}

integer ar_cbisect_r( const real* x, const integer n, const real x0 )
{
 return( ar_bisect_r( x, n, x0 ) - 1 );
}


integer ar_bisect_s( const short* x, const integer n, const short x0 )
{
 integer j, ju, jl, jm;
 if ( x0 <= x[0] ) {
  j = 1;
 } else if ( x0 >= x[n-1] ) {
  j = n;
 } else {
  jl = 0;
  ju = n+1;
  while( (ju-jl) > 1 ) {
   jm = (ju+jl)/2;
   if ( x0 > x[jm-1] ) {
    jl = jm;
   } else {
    ju = jm;
   }
  }
  j = jl;
  if( x[jl] == x0 ) j++;
 }
 return( j );
}

integer ar_cbisect_s( const short* x, const integer n, const short x0 )
{
 return( ar_bisect_s( x, n, x0 ) - 1 );
}


integer ar_bisect_i( const integer* x, const integer n, const integer x0 )
{
 integer j, ju, jl, jm;
 if ( x0 <= x[0] ) {
  j = 1;
 } else if ( x0 >= x[n-1] ) {
  j = n;
 } else {
  jl = 0;
  ju = n+1;
  while( (ju-jl) > 1 ) {
   jm = (ju+jl)/2;
   if ( x0 > x[jm-1] ) {
    jl = jm;
   } else {
    ju = jm;
   }
  }
  j = jl;
  if( x[jl] == x0 ) j++;
 }
 return( j );
}

integer ar_cbisect_i( const integer* x, const integer n, const integer x0 )
{
 return( ar_bisect_i( x, n, x0 ) - 1 );
}


void gquot( integer val, integer* ival, integer nval, integer* k, integer* irem )
{
 integer m;
 if ( val < ival[0] ) {
  *k = 0; *irem = val;
 } else if ( val > ival[nval-1] ) {
  *k = nval; *irem = val - ival[nval-1];
 } else {
  m = 0;
  while( ival[m] < val && m < nval -1 ) { m++; }
  if ( ival[m] == val ) {
   *k = m + 1;
   *irem = 0;
  } else {
   *k = m;
   *irem = val - ival[m-1];
  }
 }
}

integer prio_select( integer* uprio, logical* there, integer n )
{
 integer k;
 integer prio;
 integer i;
 k = 0; prio = 0;
 for ( i = 0; i < n; i++ ) {
  if ( there[i] && uprio[i] > prio ) {
   prio = uprio[i];
   k = i+1;
  }
 }
 return( k );
}
