#include <math.h>
#include "jcmlib.h"
#include "sex.h"
const double pi = M_PI;
const double deg_rad = 2 * M_PI / CIRCLE;


/*----------------------------------------------------------------------*/
/*
 * ssexout_i
 *
 * Converts a sexg tuple to an integer value (number of seconds).
 * 
 */

integer ssexout_i( const integer sign, const integer deg, const integer min, const integer sec )
{
 integer val;
 val = (integer) sign * ( ( deg * SXG + min ) * SXG + sec );
 return ( val );
}

/* Tuple to Value */

/*----------------------------------------------------------------------*/
/*
 * ssexout_d
 *
 * Converts a sexg tuple with fraction to a double value (number of seconds).
 * Uses: sex.h
 */

double ssexout_d( const integer sign, const integer deg, const integer min, const integer sec, const double frac )
{
 double val;
 val = sign *  ( (double) (( deg * SXG + min ) * SXG + sec ) + frac );
 return ( val );
}


/*----------------------------------------------------------------------*/
/*
 * ssexin_d
 *
 * Converts a value to sexagesimal
 * Uses: cmathproto.h
 * Uses: <math.h> fabs
 */

/* Value to Tuple */

void ssexin_d( const double v, integer* sign, integer* deg, integer* min, integer* sec, double* frac )
{
 integer tot_sec;

 *sign = ( v >= 0 ) ?  1 : -1 ;
 frac_d( fabs(v), &tot_sec, frac );
 *deg  = rmod_i( rmod_i( tot_sec, sec, SXG ), min, SXG );
}


/*----------------------------------------------------------------------*/
/*
 * ssexout_d
 *
 * Converts a sexg tuple with fraction to a real value (number of seconds).
 * Uses: sex.h
 */

real ssexout_r( const integer sign, const integer deg, const integer min, const integer sec, const real frac )
{
 real val;
 val = sign *  ( (real) (( deg * SXG + min ) * SXG + sec ) + frac );
 return ( val );
}


/*----------------------------------------------------------------------*/
/*
 * ssexin_d
 *
 * Converts a value to sexagesimal
 * Uses: cmathproto.h
 * Uses: <math.h> fabs
 */

/* Value to Tuple */

void ssexin_r( const real v, integer* sign, integer* deg, integer* min, integer* sec, real* frac )
{
 integer tot_sec;

 *sign = ( v >= 0 ) ?  1 : -1 ;
 frac_r( rabs(v), &tot_sec, frac );
 *deg  = rmod_i( rmod_i( tot_sec, sec, SXG ), min, SXG );
}


real wsexout_r( const logical sign, const integer d, const integer m, const integer s, const real f, const integer w )
{
 double r;
 r = ssexout_d( iplus( sign ), d, m, s, (double)f );
 return( (real) (r * (CIRCLE/w) / (SXG*SXG) ));
}

void wsexin_r( const real r, const integer w, logical* sign, integer* d, integer* m, integer* s, real* f )
{
 double y;
 integer signi;
 double g;
 y = r * SXG * SXG / ( CIRCLE / w );
 ssexin_d( y, &signi, d, m, s, &g );
 *f = g;
 *sign = signi >= 0;
}

double wsexout_d( const logical sign, const integer d, const integer m, const integer s, const double f, const integer w )
{
 double r;
 r = ssexout_d( iplus( sign ), d, m, s, f );
 return( r * (CIRCLE/w) / (SXG*SXG) );
}

void wsexin_d( const double r, const integer w, logical* sign, integer* d, integer* m, integer* s, double* f )
{
 double y;
 integer signi;
 y = r * SXG * SXG / ( CIRCLE / w );
 ssexin_d( y, &signi, d, m, s, f );
 *sign = signi >= 0;
}



void psexin_d( const double r, logical* sg, integer* d, integer* m, integer* s, double* f )
{
 integer j;
 integer k;
 const integer sx = 100;
 *sg = r >= 0.0;
 frac_d( fabs(r), &j, f );
 k = rmod_i( j, s, sx );
 *d = rmod_i( k, m, sx );
}

void psexin_r( const real r, logical* sg, integer* d, integer* m, integer* s, real* f )
{
 double g;
 double df;
 g = (double) r;
 psexin_d( g, sg, d, m, s, &df );
 *f = (real) df;
}

double psexout_d( const logical sg, const integer d, const integer m, const integer s, const double f )
{
 const integer sx = 100;
 return(  iplus( sg ) * ( ( d * sx + m ) * sx + s + f ) );
}

real psexout_r( const logical sg, const integer d, const integer m, const integer s, const real f )
{
 double g;
 g = psexout_d( sg, d, m, s, (double)f );
 return( (real)g );
}

