#include <stdio.h>
#include <math.h>
#include "jcmlib.h"
#include "jcmp.h"

#define NPFX 8
static char siu[] = "yzafpnum kMGTPEZYcdDh";
static integer siv[] = { -24, -21, -18, -15, -12, -9, -6, -3, 0, 3, 6, 9, 12, 15, 18, 21, 24, 
                      -2, -1, 1, 2 };
static char si_micro[] = "\\mu";
enum s_list { s_centi = 17, s_deci, s_deka, s_hecto };

/* Return the SI prefix string */


void si_getpfx( integer exponent, char* pfx, integer* remainder )
{
 integer si_step;
 *remainder = 0;
 if ( exponent == 2 ) {
  *pfx = siu[s_hecto];
 } else if ( exponent == 1 ) {
  *pfx = siu[s_deka];
 } else if ( exponent == -1 ) {
  *pfx = siu[s_deci];
 } else if ( exponent == -2 ) {
  *pfx = siu[s_centi];
 } else if ( exponent >= -NPFX && exponent <= NPFX ) {  
  si_step = rmod_i( exponent, remainder, 3 );
  *pfx = siu[si_step+NPFX];
 } else {
  *pfx = ' ';
  *remainder = exponent;
 }
}

/*
 *   Return the SI exponent given the prefix.
 *   Also return a status, which is false if the prefix
 *   is not valid.
 */
integer siexp( char* pfx, logical* ok )
{
 integer pos;
 integer step;
 *ok = TRUE;
 for ( pos = -NPFX; pos <= +NPFX+4; pos++ ) {
  step = pos + NPFX;
  if ( *pfx == siu[step] ) { 
   *ok = TRUE;
   return( siv[step] );
  }
 }

 if ( cs_eq( pfx, si_micro ) ) {
  return( -6 );
 }
 *ok = FALSE;
 return( 0 );
}


integer si_eval_r( real x )
{
 integer exponent;
 real exponent1; 
 real rem;
 if ( x == 0.0 ) {
  exponent = 0;
 } else {
  exponent1 = rlog10( rabs( x ) );
  exponent = 3 * rmod_r( exponent1, &rem, 3.0 );
 } 
 return( exponent );
}

integer si_eval_d( double x )
{
 integer exponent;
 double exponent1;
 double rem;
 if ( x == 0.0 ) {
  exponent = 0;
 } else {
  exponent1 = log10( fabs( x ) );
  exponent = 3 * rmod_d( exponent1, &rem, 3.0 );
 } 
 return( exponent );
}


void si_calc_d( double x, double* y, char* pfx, integer* exponent, char* mode )
{
 integer ex;
 integer si;

 *pfx = ' ';
 *(pfx+1) = '\0';
 ex = si_eval_d( x );
 si = ex / 3;
 if ( abs( si ) <= NPFX ) {
  *y = x / tpow_d( ex );
  if ( si == -2 && cs_ss_char( mode, 'u' ) ) {
   cs_copy( si_micro, pfx );
  } else if ( si == -1 && fabs( *y ) >= 10.0  && cs_ss_char( mode, 'c' ) ) {
   *y /= 10.0;
   ex = -2;
   *pfx = siu[s_centi];
  } else {
   *pfx = siu[si + NPFX];
  }
 } else {
  *y = x;
  *pfx = '-';
  ex = 0;
 }
 *exponent = ex;
}



void si_write_d( double x, char* cy )
{
 integer exponent;
 const integer siz = 9;
 char pfx[4];
 double y;
 si_calc_d( x, &y, pfx, &exponent, " " );
 if ( *pfx == '-' ) {
  writef_d( x, "%8.2E ", cy, siz );
 } else {
  writef_d( y, "%7.2f", cy, siz );
  cy[8] = *pfx;
 }
}

double si_read_d( char* cy )
{
 t_word cy1;
 logical ok;
 char last_char;
 integer exponent;
 double value;
 double scaled_value;
 integer slen;
 slen = cs_ends( cy );
 value = 0.0;
 last_char = cy[slen-1];
 if ( digit_char( last_char ) ) {
  value = read_d( cy );
 } else {
  if ( slen == 1 ) {
   scaled_value = 1.0;
  } else {
   cs_copy_siz( cy, cy1, slen-1 );
   scaled_value = read_d( cy1 );
  }
  exponent = siexp( cy+slen-1, &ok );
  if ( !ok ) {
   value = nan_d();
   jcm_err_msg1( "SI READ Failed: ", cy );
  } else {
   value = scaled_value * tpow_d( exponent );
  }
 }
 return( value );
}


void si_calc_r( real x, real* y, char* pfx, integer* exponent, char* mode )
{
 integer ex;
 integer si;

 *pfx = ' ';
 *(pfx+1) = '\0';
 ex = si_eval_r( x );
 si = ex / 3;
 if ( abs( si ) <= NPFX ) {
  *y = x / tpow_r( ex );
  if ( si == -2 && cs_ss_char( mode, 'u' ) ) {
   cs_copy( si_micro, pfx );
  } else if ( si == -1 && rabs( *y ) >= 10.0  && cs_ss_char( mode, 'c' ) ) {
   *y /= 10.0;
   ex = -2;
   *pfx = siu[s_centi];
  } else {
   *pfx = siu[si + NPFX];
  }
 } else {
  *y = x;
  *pfx = '-';
  ex = 0;
 }
 *exponent = ex;
}



void si_write_r( real x, char* cy )
{
 integer exponent;
 const integer siz = 9;
 char pfx[4];
 real y;
 si_calc_r( x, &y, pfx, &exponent, " " );
 if ( *pfx == '-' ) {
  writef_r( x, "%8.2E ", cy, siz );
 } else {
  writef_r( y, "%7.2f", cy, siz );
  cy[8] = *pfx;
 }
}

real si_read_r( char* cy )
{
 t_word cy1;
 logical ok;
 char last_char;
 integer exponent;
 real value;
 real scaled_value;
 integer slen;
 slen = cs_ends( cy );
 value = 0.0;
 last_char = cy[slen-1];
 if ( digit_char( last_char ) ) {
  value = read_r( cy );
 } else {
  if ( slen == 1 ) {
   scaled_value = 1.0;
  } else {
   cs_copy_siz( cy, cy1, slen-1 );
   scaled_value = read_r( cy1 );
  }
  exponent = siexp( cy+slen-1, &ok );
  if ( !ok ) {
   value = nan_r();
   jcm_err_msg1( "SI READ Failed: ", cy );
  } else {
   value = scaled_value * tpow_r( exponent );
  }
 }
 return( value );
}



void si_check( char* unit, integer* factor, integer* pos )
{
 integer slen;
 char utmp[2];
 logical ok;
 slen = cs_ends( unit );
 if ( slen > 3 && cs_eq_siz( unit, si_micro, 3 ) ) {
  *factor = -6;
  *pos = 4;
 } else if ( slen == 1 ) {
  *factor = 0;
  *pos = 1;
 } else {
  cs_copy_siz( unit, utmp, 1 );
  *factor = siexp( utmp, &ok );
  *pos = ok ? 2 : 1;
 }
}

