#include <stdio.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "fio_file.h"

static logical sio_gread( integer fid, void* buf, integer n, integer* nw, size_t siz );
static logical sio_gwrite( integer fid, void* buf, integer n, integer* nw, size_t siz );


logical sio_write_q( integer fid, logical* buf, integer n, integer* nw )
{
 return( sio_gwrite( fid, (void *)buf, n, nw, sizeof( logical ) ) );
}

logical sio_write_d( integer fid, double* buf, integer n, integer* nw )
{
 return( sio_gwrite( fid, (void *)buf, n, nw, sizeof( double ) ) );
}

logical sio_write_r( integer fid, real* buf, integer n, integer* nw )
{
 return( sio_gwrite( fid, (void *)buf, n, nw, sizeof( real ) ) );
}

logical sio_write_i( integer fid, integer* buf, integer n, integer* nw )
{
 return( sio_gwrite( fid, (void *)buf, n, nw, sizeof( integer ) ) );
}

logical sio_write_s( integer fid, short* buf, integer n, integer* nw )
{
 return( sio_gwrite( fid, (void *)buf, n, nw, sizeof( short ) ) );
}

logical sio_write_c( integer fid, char* buf, integer n, integer* nw )
{
 return( sio_gwrite( fid, (void *)buf, n, nw, sizeof( char ) ) );
}


static logical sio_gwrite( integer fid, void* buf, integer n, integer* nw, size_t siz )
{
 FILE* fptr;
 integer m;
 logical status;

 fptr = fio_file_fptr( fid );
 m = fwrite( (void *)buf, siz, n, fptr );
 status = (m == n );
 fio_set_status( fid, status );
 fio_delta_pos( fid, m * siz );
 *nw = m;
 return( status );
}

static logical sio_gread( integer fid, void* buf, integer n, integer* nw, size_t siz )
{
 FILE* fptr;
 integer m;
 logical status;

 fptr = fio_file_fptr( fid );
 m = fread( (void *)buf, siz, n, fptr );
 status = ! ( feof( fptr ) || ferror( fptr ) );
 fio_set_status( fid, status );
 fio_delta_pos( fid, m * siz );
 *nw = m;
 return( status );
}

logical sio_read_q( integer fid, logical* buf, integer n, integer* nw )
{
 return( sio_gread( fid, (void *)buf, n, nw, sizeof( logical ) ) );
}

logical sio_read_d( integer fid, double* buf, integer n, integer* nw )
{
 return( sio_gread( fid, (void *)buf, n, nw, sizeof( double ) ) );
}

logical sio_read_c( integer fid, char* buf, integer n, integer* nw )
{
 return( sio_gread( fid, (void *)buf, n, nw, sizeof( char ) ) );
/* bug fix 1997 Mar 10 */
}

logical sio_read_r( integer fid, real* buf, integer n, integer* nw )
{
 return( sio_gread( fid, (void *)buf, n, nw, sizeof( real ) ) );
}

logical sio_read_i( integer fid, integer* buf, integer n, integer* nw )
{
 return( sio_gread( fid, (void *)buf, n, nw, sizeof( integer ) ) );
}

logical sio_read_s( integer fid, short* buf, integer n, integer* nw )
{
 return( sio_gread( fid, (void *)buf, n, nw, sizeof( short ) ) );
}


integer sio_open_r( char* filename )
{
 integer file_no;
 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "SIO R", 0 );
 }
 return(file_no); 
}


integer sio_open_w( char* filename )
{
 integer file_no;
 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "SIO W", 0 );
 }
 return(file_no); 
}


integer sio_open_rw( char* filename )
{
 integer file_no;
 if ( cs_is_blank( filename )  || cs_eq( filename, "*" ) ) {
  file_no = 0;
 } else {
  file_no = fio_open_retry( filename, "SIO RW", 0 );
 }
 return(file_no); 
}



logical sio_seek( integer fid, integer pos )
{
 FILE* fptr;
 logical status;
 fio_clear_eof( fid );
 fptr = fio_file_fptr( fid );
 status = fseek( fptr, pos, 0 );
 return( !status );
}



void sio_flush( integer fid )
{
 FILE* fptr;
 fptr = fio_file_fptr( fid );
 (void)fflush( fptr );
}

void sio_flush_stdout( void )
{
 (void)fflush(stdout);
}

