
#include <stdio.h>
#include "jcmlib.h"

/*======================================================================*/
/*
 * Token parsing routines
 *
 */
/*======================================================================*/


/*----------------------------------------------------------------------*/
/*
 * check_pars
 *
 * Local routine to check parenthesis level in string
 *
 */

static logical check_pars( const char source, integer* paren_level, const logical paren_use, const char lpar, const char rpar )
{
 logical status;

 if ( !paren_use ) return( TRUE );
 status = FALSE;
 if ( source == lpar ) 
  (*paren_level)++;
 else if ( source == rpar )
  (*paren_level)--;
 else
  status = (*paren_level <= 0 );

 return( status );
}


/*----------------------------------------------------------------------*/
/*
 * cs_copy_token
 *
 * Copy from *buf to dest until you get to one of the seps characters lying
 * outside of all parentheses; or until dest is full up to siz.
 * 1996 Aug 26 JCM
 * 1997 Aug  5 JCM modified to return the separator found.
 * Args:
 *	buf		(i/o) 	Pointer to input buffer
 *	dest		(o)	Output buffer
 *	seps		(i)	Token separator characters
 *	pars		(i)	Left and right paren chars
 *	siz		(i)	Max allowed size of output buffer
 */

char cs_copy_token( char** buf, const char* seps, const char* pars, char* dest, const integer gsiz )
{
 integer siz = gsiz;
 integer paren_level;
 logical paren_use;
 logical loop;
 char* source;
 char found = 0;

 if ( !buf || !(*buf) ) {
  cs_copy( " ", dest );
  return( 0 );
 }
 /* Advance to next non blank character.  */
 cs_advptr( buf );
 source = *buf;
 if ( *source != '\0' ) {
  paren_level = 0;
  paren_use   = !cs_eq_siz( pars, "  ", 2 );
 /* Copy source characters to destination */
  loop = TRUE;
  while ( *source != '\0' && loop ) {
   if ( check_pars ( *source, &paren_level, paren_use, pars[0], pars[1] ) ) {
    loop = !cs_ss_char( seps, *source );
    if( !loop ) found = *source;
    }
   if ( loop && siz ) { *dest++ = *source; siz--; }
   source++;
  }  /* end while */

 /* Advance source pointer to end of token */

  *buf = source;
 /* Advance to next non blank character.  */

  cs_advptr( buf );
 }  /* end if non null */
 if ( cs_is_blank( *buf ) ) { *buf = NULL;
  }
 *dest = '\0'; 
 return( found );
}




