#include <stdio.h>
#include <math.h>
#include "jcmlib.h"
#include "jcmp.h"

/*
 *--------------------------------------------------------------------------------
 */

/*
 *   Here we parse a unit string into a number of unit items, an array of base
 *   units, an array of exponents, and an array of SI prefix factors.
 */
void unit_parse( const char* unit, integer* n_uo, char** base_unit, double* exp_unit, integer* si_factor )
{
 t_word unit_item;
 char* ptr;
 double exponent;
 t_word base_token;
 integer n;
 logical loop;

 n = 0;
 cs_copy( "10", base_unit[n] );
 exp_unit[n] = 0.0;
 si_factor[n] = 0;
 loop = !cs_is_blank( unit );
 ptr = (char *)unit;
 while ( loop && ptr != NULL ) {
  cs_get_c( &ptr, unit_item, WSIZE );
  unit_parse_exp( unit_item, base_token, &exponent );
  if ( n == 0 && cs_eq( base_token, "10" ) ) {
   exp_unit[n] = exponent;
  } else {
   n++;
   cs_copy( base_token, base_unit[n] );
   exp_unit[n] = exponent;
   si_factor[n] = 0;
  }
 }
 *n_uo = n+1;
}

/*
 *--------------------------------------------------------------------------------
 */


void unit_parse_exp( const char* unit_item, char* base_unit, double* exp_unit )
{
 integer j;
 integer k;
 const char* ptr;
 t_word tmp;

 /* We look for a ^ character indicating the start of an exponent */

 j = cs_index_char( unit_item, '^' );
 if ( j > 0 ) {
  cs_copy_siz( unit_item, base_unit, j-1 );
  ptr = (unit_item + j); 

  /* Handle the case of TeX parens  x^{y} */

  if ( *ptr == '{' ) {
   k = cs_index_char( ptr, '}' );
   cs_copy_siz( ptr, tmp, k-1 );
   *exp_unit = read_d( tmp );
  } else {

  /* or the case when we have a plain exponent   x^y */

   *exp_unit = read_d( ptr );
  }
 } else {

 /* Default case is an exponent of 1 */

  cs_copy( unit_item, base_unit );
  *exp_unit = 1.0;
 }
 unit_parse_mantissa( base_unit, exp_unit );
}

/*
 *--------------------------------------------------------------------------------
 */

void unit_parse_mantissa( char* base_unit, double* exp_unit )
{
 logical ok;
 double log_value;
 integer j,k;
 logical found_ten;
 t_word number;
 double value;

 /* Check for a number that is not 10 */

 ok = digit_char( *base_unit ) && !cs_eq( base_unit, "10" );
 log_value = 0;

 if ( ok ) {
  found_ten = FALSE;
  k = cs_ends( base_unit );

  /* check for TeX  \times 10^2  syntax */

  j = cs_index( base_unit, "\\times10" );
  if ( j > 0 && j+7 == k ) {
   cs_copy_siz( base_unit, number, j-1 );
   found_ten = TRUE;
  } else {
  /*  check for normal x10^2 syntax */
   j = cs_index( base_unit, "x10" );
   if ( j > 0 && j+2 == k ) {
    cs_copy_siz( base_unit, number, j-1 );
    found_ten = TRUE;
   }
  }
/* if we have a leading number, make the base unit 10 and
 * take the log of the number to put in the base unit's exponent
 */
  if ( !found_ten ) {
   cs_copy( base_unit, number );
   if ( *exp_unit != 1.0 ) jcm_err_msg1( "Weird unit ", base_unit );
  }
  value = read_d( number );
  if ( value > 0.0 ) log_value = log10( value );
  if ( found_ten ) {
   *exp_unit += log_value;
  } else {
   *exp_unit = log_value;
  }
  cs_copy( "10", base_unit );
 }

}



/*
 *--------------------------------------------------------------------------------
 */


void unit_compose( char** base_unit, double* exp_unit, integer* si_fac, integer n_uo, logical texmode, char* unit, integer siz )
{
 integer ten_exp_i;
 double ten_mantissa;
 char* ptr;
 integer i;
 ptr = unit;
 cs_copy( " ", unit );
 if ( n_uo == 0 ) return;
 if ( cs_eq( base_unit[0], "10" ) && exp_unit[0] != 0.0 ) {
  frac_d( exp_unit[0], &ten_exp_i, &ten_mantissa );
  if ( ten_mantissa != 0.0 ) {
   cs_put_d( tpow_d( ten_mantissa ), &ptr, &siz );
   if ( texmode ) {
    cs_put_c( "\times", &ptr, &siz );
   } else {
    cs_put_c( "x", &ptr, &siz );
   }
  }
  unit_compitem( &ptr, base_unit[0], (double)ten_exp_i, 0, &siz );
  if ( !siz ) return;
 }

 for( i = 1; i < n_uo; i++ ) {
  if ( exp_unit[i] > 0.0 ) {
   unit_compitem( &ptr, base_unit[i], exp_unit[i], si_fac[i], &siz );
   if ( !siz ) return;
  }
 }
 for ( i = 1; i < n_uo; i++ ) {
  if ( exp_unit[i] < 0.0 ) {
   unit_compitem( &ptr, base_unit[i], exp_unit[i], si_fac[i], &siz );
   if ( !siz ) return;
  }
 }
}

/*
 *--------------------------------------------------------------------------------
 */



void unit_compitem( char** ptr, char* item, double exp_unit, integer si_fac, integer* siz )
{
 integer k;
 double x;
 integer n;
 char pfx[6];
 integer ix;
 double rx;
 t_word tmp;

 si_getpfx( si_fac, pfx, &k );
 if ( cs_is_blank( pfx ) ) { 
  cs_copy( item, tmp );
 } else {
  *tmp = *pfx;
  cs_copy( item, tmp+1 );
 }
 cs_put_c( tmp, ptr, siz );
 x = exp_unit;
 if ( x != 1.0 ) {
  frac_d( x, &ix, &rx );
  if ( rx == 0.0 ) {
   n = sprintf( *ptr, "^{%ld}", (long)ix );
  } else {
   n = sprintf( *ptr, "^{%s}", write_d( x, tmp, WSIZE ) );
  }
  *ptr += n;
  *siz -= n;
 }
 **ptr = ' ';
 (*ptr)++;
}

/*
 *--------------------------------------------------------------------------------
 */


void si_tmult( integer fac1, double exponent1, integer fac2, double exponent2, integer* fac, double* mantissa )
{
 double result_factor;
 double exponent;

 result_factor = fac1 * exponent1 + fac2 * exponent2;
 exponent = exponent1 + exponent2;
 if ( exponent == 0.0 ) {
  *fac = 0;
  *mantissa = result_factor;
 } else {
  *fac = fac1;
  *mantissa = result_factor - exponent * fac1;
 }
}

/*
 *--------------------------------------------------------------------------------
 */

void unit_tmult( integer* n1, char** base1, double* exponent1, integer* si_fac1,
                 integer* n2, char** base2, double* exponent2, integer* si_fac2, integer power )
{
 integer i;
 integer fac;
 integer k;
 double mantissa;
 for ( i = 0; i < *n2; i++ ) {
  k = cmatch_c( base2[i], base1, *n1 );
  if ( k >= 0 ) {
   si_tmult( si_fac1[k], exponent1[k], si_fac2[i], power * exponent2[i], 
     &fac, &mantissa );
   exponent1[k] += power * exponent2[i];
   si_fac1[k] = fac;
   exponent1[1] += mantissa;
  } else {
   (*n1)++;
   k = *n1;
   cs_copy( base2[i], base1[k] );
   exponent1[k] = power * exponent2[i];
   si_fac1[k] = si_fac2[i];
  } 
 }
}
