#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "jcmlib.h"
#include "jcmp.h"
#include "unit.h"
#define UDBG 2
#define MANT_POS 0

static logical texmode = TRUE;

t_unit* alloc_unit( const integer n )
{
 t_unit* unit;
 unit = (t_unit *)malloc( sizeof( t_unit ) );
 unit->nmax = n;
 unit->n = 0;
 unit->base = alloc_array_c( WSIZE, n );
 unit->exp  = (double *)malloc( n * sizeof( double )) ;
 unit->fac  = (integer *)malloc( n * sizeof( integer ) );
 return( unit );
}

void free_unit( t_unit* unit )
{
 free( unit->fac );
 free( unit->exp );
 free_array_c( unit->base );
 free( unit );
}



void unit_regresolve( t_unit* reg1, t_unit* reg2, char** udf_unit, char** udf_def, integer nu )
{
 integer i;
 integer j;
 for ( i = 0; i < reg1->n; i++ ) {
  j = cmatch_c( reg1->base[i], udf_unit, nu );
  if ( j >= 0 ) {
   if ( !cs_is_blank( udf_def[j] ) ) {
    unit_regparse( udf_def[j], reg2 );
    unit_regexp( reg2, reg1->exp[i] );
    reg2->exp[MANT_POS] += reg1->fac[i] * reg1->exp[i];
    reg1->exp[i] = 0.0;
    unit_ddisp( reg1 );
    unit_ddisp( reg2 );
    unit_regmult( reg1, reg2, +1 );
    unit_ddisp( reg1 );
   }
  }
 }

}

void unit_regdedup( t_unit* reg, t_unit* temp )
{
 integer i;
 integer j;
 integer fac;
 double exponent;
 for ( i = 2; i < reg->n; i++ ) {
  j = cmatch_c( reg->base[i], reg->base, i-1 );
  if ( j >= 0 ) {
   si_tmult( reg->fac[i], reg->exp[i], temp->fac[j], temp->exp[j], &fac, &exponent );
   reg->fac[j] = fac;
   reg->exp[MANT_POS] += exponent;
   reg->exp[j] += reg->exp[i];
   reg->exp[i] = 0.0;
  }
 }
 unit_ddisp( reg );

}

void unit_regcompose( t_unit* reg, char* unit, const integer siz )
{
 unit_compose( reg->base, reg->exp, reg->fac, reg->n, texmode, unit, siz );
}

void unit_ddisp( t_unit* reg )
{
 integer i;
 t_word tmp;
 card buf;
 if ( dbg_level( UDBG ) ) {
  dashline( 60 );
  for ( i = 0; i < reg->n; i++ ) {
   sprintf( buf, "%6ld %6ld %-10s %s", (long)i, (long)reg->fac[i], reg->base[i], write_d( reg->exp[i], tmp, WSIZE ));
   tmsg( buf );
  }
  dashline( 60 );
 }
}

void unit_regparse( const char* unit, t_unit* reg )
{
 unit_parse( unit, &(reg->n), reg->base, reg->exp, reg->fac );
 unit_regsi( reg );
}

void unit_regsi( t_unit* reg )
{
 unit_si_resolve( reg->n, reg->base, reg->fac );
}



void unit_regexp( t_unit* reg, const double power )
{
 integer i;
 for ( i = 0; i < reg->n; i++ ) {
  reg->exp[i] *= power;
 }
}


void unit_regmult( t_unit* reg1, t_unit* reg2, const integer power2 )
{
 unit_tmult( &reg1->n, reg1->base, reg1->exp, reg1->fac,
             &reg2->n, reg2->base, reg2->exp, reg2->fac, power2 );
}

void unit_texmode( const logical q )
{
 texmode = q;
}




