#include <math.h>
#include "jcmlib.h"

const double VZERO_D[3] = { 0.0, 0.0, 0.0 };


void vcopy_d( const double* p, double* r )
{
 integer n = 3;
 while( n-- ) *r++ = *p++;
}

void vzero_d( double* p )
{
 integer n = 3;
 while( n-- ) *p++ = 0.0;
}

void vadd_d( const double* p, double* r )
{
 integer n =3;
 while( n-- ) *r++ += *p++;
}

void vsub_d( const double* p, double* r )
{
 integer n =3;
 while( n-- ) *r++ -= *p++;
}

void vcmult_d( double* p, const double c )
{
 integer n = 3;
 while( n-- ) *p++ *= c;
}


double vdot_d( const double* a, const double* b )
{
 return( a[0] * b[0] + a[1] * b[1] + a[2] * b[2] );
}

double vnorm_d( const double* a )
{
 return( sqrt( a[0]*a[0] + a[1]*a[1] + a[2]*a[2] ) );
}

void vcross_d( const double* a, const double* b, double* c )
{
 c[0] = a[1] * b[2] - a[2] * b[1];
 c[1] = a[2] * b[0] - a[0] * b[2];
 c[2] = a[0] * b[1] - a[1] * b[0];
}


double vmaxmod_d( const double* r )
{
 double result = 0;
 double a;
 integer n = 3;
 while ( n-- ) { a = fabs( *r++ ); result = ( a > result ) ? a : result; }
 return( result );
}

double vunit_d( double* r )
{
 double r0;
 integer n = 3;
 r0 = vnorm_d( r );
 if ( r0 > 0.0 ) {
  while( n-- ) *r++ /= r0;
 }
 return( r0 );
}


real vdot_r( const real* a, const real* b )
{
 return( a[0] * b[0] + a[1] * b[1] + a[2] * b[2] );
}

real vnorm_r( const real* a )
{
 return( rsqrt( a[0]*a[0] + a[1]*a[1] + a[2]*a[2] ) );
}

void vcross_r( const real* a, const real* b, real* c )
{
 c[0] = a[1] * b[2] - a[2] * b[1];
 c[1] = a[2] * b[0] - a[0] * b[2];
 c[2] = a[0] * b[1] - a[1] * b[0];
}


real vmaxmod_r( const real* r )
{
 real result = 0;
 real a;
 integer n = 3;
 while ( n-- ) { a = rabs( *r++ ); result = ( a > result ) ? a : result; }
 return( result );
}

real vunit_r( real* r )
{
 real r0;
 integer n = 3;
 r0 = vnorm_r( r );
 if ( r0 > 0.0 ) {
  while( n-- ) *r++ /= r0;
 }
 return( r0 );
}




void vcopy_r( const real* p, real* r )
{
 integer n = 3;
 while( n-- ) *r++ = *p++;
}

void vzero_r( real* p )
{
 integer n = 3;
 while( n-- ) *p++ = 0.0;
}

void vadd_r( const real* p, real* r )
{
 integer n =3;
 while( n-- ) *r++ += *p++;
}

void vsub_r( const real* p, real* r )
{
 integer n =3;
 while( n-- ) *r++ += *p++;
}

void vcmult_r( real* p, const real c )
{
 integer n = 3;
 while( n-- ) *p++ *= c;
}


void mat_prex_d( double** aa, const double* y, double* x )
{
 integer i,j;
 for ( i = 0; i < 3; i++ ) {
  *x = 0.0;
  for( j = 0; j < 3; j++ ) {
   *x += y[j] * aa[j][i];
  }
  x++;
 }
}

void mat_prex_r( real** aa, const real* y, real* x )
{
 integer i,j;
 for ( i = 0; i < 3; i++ ) {
  *x = 0.0;
  for( j = 0; j < 3; j++ ) {
   *x += y[j] * aa[j][i];
  }
  x++;
 }
}


void mat_postx_d( double** aa, const double* y, double* x )
{
 integer i,j;
 for ( i = 0; i < 3; i++ ) {
  *x = 0.0;
  for( j = 0; j < 3; j++ ) {
   *x += y[j] * aa[i][j];
  }
  x++;
 }
}

void mat_postx_r( real** aa, const real* y, real* x )
{
 integer i,j;
 for ( i = 0; i < 3; i++ ) {
  *x = 0.0;
  for( j = 0; j < 3; j++ ) {
   *x += y[j] * aa[i][j];
  }
  x++;
 }
}

void mat_mult_d( double** a, double** b, double** c )
{
 integer i,j,k;
 double w;
 for ( i = 0; i < 3; i++ ) {
  for( j = 0; j < 3; j++ ) {
   w = 0.0;
   for( k = 0; k < 3; k++ ) {
    w += a[i][k] * b[k][j];
   }
   c[i][j] = w;
  }
 }
}

void mat_mult_r( real** a, real** b, real** c )
{
 integer i,j,k;
 real w;
 for ( i = 0; i < 3; i++ ) {
  for( j = 0; j < 3; j++ ) {
   w = 0.0;
   for( k = 0; k < 3; k++ ) {
    w += a[i][k] * b[k][j];
   }
   c[i][j] = w;
  }
 }
}


