#include <stdio.h>

#include "jcmlib.h"
#define JCMPAR_SRC
#include "jcmpar.h"
#include "jcmpar2.h"
#include "saopar.h"

#define MAXPARS 500
static integer iraf = 0;  /* Global variable */
static void par_open_internal( ParameterList* parlist, char* parfilename );

/* Open parameter file. Look for IRAF mode flag, iraf, on command line and if so use IRAF parameter interface */


/*
 * par_init
 *
 * We process the argument list twice, first to look for 
 * a non-default parameter file, second to override values in that
 * parameter file.
 */


ParameterList* par_init( char* default_mode, char* default_parfile, char* args[], integer nargs )
{
 t_filename parfilename;
 ParameterList* parlist;
 t_word mode; 
 int debug;

 parlist = par_alloc( MAXPARS );
 par_set_args( args, nargs, parlist );
 debug = par_get_i( parlist, "debug", 0 );
 if ( debug > 0 ) dbg_set( debug );
 (void)par_get( parlist, "par", default_parfile, parfilename, FSIZE );
 (void)par_get( parlist, "parkernel", default_mode, mode, WSIZE );
 iraf = ( cs_uceq( mode, "sao" ) || cs_uceq( mode, "iraf" ) ) ;
 par_open_internal( parlist, parfilename );
 par_set_args( args, nargs, parlist );
 return( parlist );
}

/*
 * Simple parameter file open ignoring command line args,
 * assume iraf flag already set.
 */

ParameterList* par_open( char* parfilename )
{
 ParameterList* parlist;
 parlist = par_alloc( MAXPARS );
 par_open_internal( parlist, parfilename );
 return( parlist );
}

static void par_open_internal( ParameterList* parlist, char* parfilename )
{
 dmsg1( 1, "Read parameter file ", parfilename );
 if ( iraf ) {
  dmsg( 1, "SAO/IRAF parameter interface" );
  sao_par_open( parfilename, parlist );
 } else {
  dmsg( 1, "JCM parameter interface" );
  par_read_file( parfilename, parlist );
 }
}


