
/* Parameter list routines */

/* We make the ParameterList object opaque to external users of the library */

#ifdef JCMPAR_SRC
typedef struct {
 char** name;		/* Parameter names */
 char** value;		/* Parameter values */
 integer n;		/* Number of pars */
 integer nmax;		/* Number allocated */
} ParameterList;
#else
typedef void ParameterList;
#endif

extern ParameterList* par_init( char* default_mode, char* default_parfile, char* args[], integer nargs );
/*
 *  Read a parameter file. Allowed modes are "iraf", "jcm" for different parameter file formats.
 *  However, jcm mode will also attempt to recognize iraf data and switch accordingly.
 *  Also reads in command line parameters supplied as (args, nargs) by user.
 *  Returns a pointer to a parameter list object. 
 *  The current implementation supports only a finite (MAXPARS = 500) number of parameters per parameter list.
 */

extern ParameterList* par_open( char* default_parfile );
/*
 *    Read parameter file and return parameter list. If there has been a previous call to par_init, inherit the mode.
 */

extern void par_read_append( ParameterList* parlist, char* default_parfile );
/*
 *  Read parameter file and append parameters to input parameter list. 
 *  This may also be used with par_alloc to read a larger (or smaller) parameter list than the default.
 *
 */

extern void           par_free( ParameterList* parlist );
/*
 *    Free a parameter list object.
 */

extern integer        par_get( ParameterList* parlist, const char* name, const char* vdefault, char* value, const integer maxlen );
/*
 *   Return a parameter from the parameter list, searching by name and returning a string in 'value' (whose size should
 *   be at least maxlen).
 *   If the parameter is not present, value is set to the contents of vdefault.
 * 
 */

extern void           par_set( ParameterList* parlist, const char* name, const char* value );
/*
 *  Set a parameter to have the given value. Adds it to the parameter list if it doesn't already exist.
 */

extern void           par_list( ParameterList* parlist );
/*
 *   Print the contents of the parameter list to the message stream.
 */

extern integer        par_get_i( ParameterList* parlist, const char* name, const integer vdefault );
/*
 *   Return an integer valued parameter, with the default value vdefault if parameter is not present.
 */

extern double         par_get_d( ParameterList* parlist, const char* name, const double vdefault );
/*
 *   Return a double valued parameter, with the default value vdefault if parameter is not present.
 */

extern void           par_get_vec_d( ParameterList* parlist, const char* name, const double* vec0, double* vec );
/*
 *   Return a 3-vector of doubles (in the parameter file separated by spaces or commas).
 *   A default 3-vector vec0 is provided; if the pointer vec0 is NULL, the default is the zero 3-vector.
 */


extern ParameterList* par_alloc( integer nmax );
/*
 *    Allocate a parameter list of the given max number of parameters.
 */

