#include <stdio.h>
#include <stdlib.h>
#include "jcmlib.h"
#define JCMPAR_SRC
#include "jcmpar.h"
#include "jcmpar2.h"
#include "saopar.h"
/*
 *
 * alloc_pars
 *
 */
ParameterList* par_alloc( const integer nmax )
{
 ParameterList* parlist;

 parlist = (ParameterList* )malloc( sizeof( ParameterList ) );
 parlist->name = alloc_array_c( WSIZE, nmax );
 parlist->value = alloc_array_c( CSIZE, nmax );
 parlist->n = 0;
 parlist->nmax = nmax;
 return( parlist );
}

void par_free( ParameterList* parlist )
{
 free_array_c( parlist->name );
 free_array_c( parlist->value );
}

/*
 *
 * par_set
 *
 */
void par_set( ParameterList* parlist, const char* name, const char* value )
{
 t_word keyword;
 integer parno;

 cs_copy_siz( name, keyword, WSIZE );
 cs_upper( keyword );
 parno = cmatch_c( keyword, parlist->name, parlist->n );
 if ( parno < 0 ) {   
  if ( parlist->n >= parlist->nmax ) return;
  (parlist->n)++;
  parno = parlist->n - 1;
  cs_copy( keyword, parlist->name[parno] );
 }
 cs_copy_siz( value, parlist->value[parno], CSIZE ); 
}

/*
 *
 * par_get
 *
 */
integer par_get_g( ParameterList* parlist, const char* name, char* value, const integer maxlen )
{
 t_word keyword;
 integer parno;
 cs_copy_siz( name, keyword, WSIZE );
 cs_upper( keyword );
 parno = cmatch_c( keyword, parlist->name, parlist->n );
 if ( parno < 0 ) {   
  cs_copy( "", value );
 } else {
  cs_copy_siz( parlist->value[parno], value, maxlen );
 }
 return( parno );
}

integer par_get( ParameterList* parlist, const char* name, const char* vdefault, char* value, const integer maxlen )
{
 integer parno;
 parno = par_get_g( parlist, name, value, maxlen );
 if ( parno < 0 ) cs_copy_siz( vdefault, value, maxlen );
 return( parno );
}

integer par_get_i( ParameterList* parlist, const char* name, const integer vdefault )
{
 integer parno;
 t_word value;
 integer ival;
 parno = par_get_g( parlist, name, value, WSIZE );
 if ( parno < 0 ) {
  ival = vdefault;
 } else {
  ival = read_i( value );
 }
 return( ival );
}


void par_get_vec_d( ParameterList* parlist, const char* name, const double* vec0, double* vec )
{
 integer parno;
 t_buf value;
 char* ptr;
 parno = par_get_g( parlist, name, value, BSIZE );
 if ( parno < 0 ) {
  if ( vec0 ) {
   vcopy_d( vec0, vec );
  } else {
   vzero_d( vec );
  }
 } else {
  ptr = value;
  vec[0] = cs_get_d( &ptr );
  vec[1] = cs_get_d( &ptr );
  vec[2] = cs_get_d( &ptr );
 }
}

double par_get_d( ParameterList* parlist, const char* name, const double vdefault )
{
 integer parno;
 t_word value;
 double dval;
 parno = par_get_g( parlist, name, value, WSIZE );
 if ( parno < 0 ) {
  dval = vdefault;
 } else {
  dval = read_d( value );
 }
 return( dval );
}

/*
 * par_read_file
 *
 * Update a parameter list from a file
 */
void par_read_file( char* filename, ParameterList* parlist )
{
 integer file;
 t_buf line;
 char* ptr;
 t_word keyword;
 logical start = TRUE;

 file = fio_open_r( filename );
 if ( file == 0 ) return;
 while ( fio_read( file, line, BSIZE ) ) {
  ptr = line;
  cs_get_c( &ptr, keyword, WSIZE );
  if ( keyword[0] != '#' ) {
   if ( start ) {
    start = FALSE;
    if ( cs_index( line, "," ) == ptr - line ) { 
     tmsg( "Detected IRAF syntax: using SAO parameter interface" );
     fio_close( file );
     sao_par_open( filename, parlist );
     return;
    }
   }
   par_set( parlist, keyword, ptr );
  }
 }
 fio_close( file );

}

/*
 * par_set_args
 *
 * Set parameters from a set of strings of the form name=value
 * Ignore the first element of the array, for compatibility with 
 * Unix/C command line args.
 */

void par_set_args( char** args, integer nargs, ParameterList* parlist )
{
 integer i;
 const char* value;
 t_word keyword;
 integer pos;

 for ( i = 1; i < nargs; i++ ) {
  pos = cs_index( args[i], "=" );
  if ( pos > 0 ) {
   cs_get_ss( args[i], keyword, 1, pos-1 );
   value = args[i] + pos;
   par_set( parlist, keyword, value );
  } else {
   par_set( parlist, args[i], " " );
  }
 }
}

/*
 * par_list
 *
 */
void par_list( ParameterList* parlist )
{
 integer i;
 t_buf buf;
 for ( i = 0; i < parlist->n; i++ ) {
  sprintf( buf, "%-32s %s", parlist->name[i], parlist->value[i] );
  msg( buf );
 }
}


