#include "xgain.h"

/*
 *
 * Read a list of pixels from an ASCII file
 * Store them in the Source structure.
 */
Source* source_read( char* file, int nmax )
{
 int in;
 card line;
 char* ptr;
 card buf;
 int n;
 Source* source;

 source = alloc_source( nmax );
 in = fio_open_r( file );
 if ( in == 0 ) {
  sprintf( buf, "Failed to open source file %s\n", file );
  tmsg( buf );
  return( NULL );
 }
 n = 0;
 while( fio_read( in, line, CSIZE ) ) {
  n++;
  if ( n >= nmax ) {
   tmsg( "Too many pixels in region" );
   fio_close( in );
   source->n = 0;
   return( source );
  }
  ptr = line;
  source->xpos[n-1] = cs_get_d( &ptr );
  source->ypos[n-1] = cs_get_d( &ptr );
 }
 fio_close( in );
 sprintf( buf, "Read %d pixel positions", n );
 tmsg( buf );
 source->n = n;
 return( source );
}

/*
 * Allocate source structure. Don't bother mallocing an
 * array for the common case of only one pixel.
 */

Source* alloc_source( int n )
{
 Source* source;
 source = (Source* )malloc( sizeof( Source ) );
 source->x0 = 0.0;  /* Default, but not useful */
 source->y0 = 0.0;  /* Default, but not useful */

 if ( n > 1 ) {
  source->xpos = (double*)malloc( n*sizeof( double ) );
  source->ypos  = (double*)malloc( n*sizeof( double ) );
  source->n = 0;
 } else {
  source->xpos = &source->x0;
  source->ypos = &source->y0;
  source->n  = 1;
 }
 source->nmax = n;
 return( source );
}

/*
 * Free source structure
 */

void free_source( Source* source )
{
 if ( source->nmax > 1 ) {
  free( source->xpos );
  free( source->ypos  );
 }
 free( source );
}



